#!/usr/bin/ksh

##-------------------------------------------------------------------#
## %PS
##    
## Licensed Internal Code - Property of IBM
##    
## 2105 Licensed Internal Code
##    
## (C) Copyright IBM Corp. 1999 All Rights Reserved.
##    
## US Government Users Restricted Rights - Use, duplication or
## disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
## %EPS
##
## COMPONENT_NAME (RAS WUI)
##
##-------------------------------------------------------------------#
##
## Filename: rsExecuteTask.sh
##
## Subsystem: RAS WUI
##
## Input:	rsExecuteTask.sh -v -u username -p password -a filepath -s serverName -b backup_server taskNames
##
##		-v = verbose
##		-u = username
##		-p = password
##              -a = filepath
##		-s = name of copy services server
##              -b = name of backup copy services server
##		taskNames = one or more pre-defined Copy Services 
##			    Server tasks which are to be executed
##
## Description: Execute one or more Copy Services tasks
##
##
## Change History:
##   Modifier               Date                             Change ID
##-------------------------------------------------------------------#
##  Richard Kirchhofer     12/03/1999			     28254
##      Initial version
##  Richard Kirchhofer     04 Feb 2000			     29810
##      Standalone ORB conversion
##  Anne Chau              02-18-2000                      31101
##      Packaging simplification..call rsENV to setup INSTALL & JAVA_HOME
##  Anne Chau		   03-08-2000			     32263
##      CLI function rsTestConnection does not work
##  Ramandeep Kaur &
##  Ruth Azevedo           05-30-2000                      35611, 35433
##      Command invocation fixed
##  Ruth Azevedo           05-30-2000                      34871       
##      Modify INSTALL directory for AIXi, SUN and HP                           
##  John Wrobel & 
##  Ruth Azevedo           07-17-2000                      38510
##      Test connection to primary/backup server before executing task
##      Added backup server parameter to command line
##  John Wrobel            07-21-2000                      38511
##      Modified return codes 
##  Ramandeep Kaur         08-08-2000                      39105
##      Modified return codes
##  Ramandeep Kaur         10-05-2000                      41892
##      Added the functionality of giving Backup Server alone 
##  Brian McCain           10-14-2000                      41890
##      Modified error checking to catch bad parameters
##  Ramandeep Kaur         10-19-2000                      42290
##      Modified env variable CLASSPATH
##  Brian McCain           10-20-2000                      43123
##      Display usage if no parameters specified
##  Ramandeep Kaur         11-02-2000                      43575
##      Added new flag -a
##  Ramandeep Kaur         11-14-2000                      43880
##      Included check if user gave space between - and flag 
##-------------------------------------------------------------------#
## End of PROLOG

OS=`uname | cut -c 1-3`

##-------------------------------------------------------------------#
## Set defaults
##-------------------------------------------------------------------#
SERVERNAME=
PRIMARY_SERVER=
BACKUP_SERVER=
USERNAME=
PASSWORD=
FILEPATH=
VERBOSE=
TASKNAMES=
RC=

##-------------------------------------------------------------------#
## AIX specific code to set up INSTALL and JAVA_HOME
##-------------------------------------------------------------------#
if [[ $OS = "AIX" ]]; then

export INSTALL=/usr/opt/ibm2105cli
export JAVA_HOME=/usr/jdk_base

fi
##-------------------------------------------------------------------#
## Sun specific code
##-------------------------------------------------------------------#
if [[ $OS = "Sun" ]]; then

export INSTALL=/opt/ibm2105cli
export JAVA_HOME=/usr/java

fi
##-------------------------------------------------------------------#
## HP specific code
##-------------------------------------------------------------------#
if [[ $OS = "HP-" ]]; then

export INSTALL=/opt/ibm2105cli
export JAVA_HOME=/opt/java

fi

##-------------------------------------------------------------------#
## Process arguments
##-------------------------------------------------------------------#
USAGE='usage: rsExecuteTask.sh [-v] -u username & -p password | -a securityfilepath -s primaryserver -b backupserver taskNames'

if [[ -z $1 ]]; then
   echo "rsExecuteTask: No paramaters specified"
   echo $USAGE 
   exit 21
fi

while getopts ":s:u:p:b:a:v" opt; do
        if [[ $OPTARG = -* ]]; then 
           echo "rsExecuteTask: Missing argument for -$opt"
           echo $USAGE 
           exit 20
        fi
        case $opt in
          u  ) USERNAME=$OPTARG;;
          p  ) PASSWORD=$OPTARG;;
	  s  ) PRIMARY_SERVER=$OPTARG;;
          b  ) BACKUP_SERVER=$OPTARG;;
          a  ) FILEPATH=$OPTARG;;
          v  ) VERBOSE=-v;;
          \? ) echo "rsExecuteTask: Invalid parameter specified"
               echo $USAGE 
               exit 19
        esac
done
shift $(($OPTIND - 1))

while [[ ! -z $1 ]]; do
        if [[ $1 = - ]];then
             echo "rsExecuteTask: Invalid parameter specified"
             echo $USAGE
             exit 19
        fi
	if [[ -z $TASKNAMES ]]; then
		TASKNAMES=$1
	else
		TASKNAMES="$TASKNAMES $1"
	fi
	shift
done

if [[ -z $PRIMARY_SERVER && -z $BACKUP_SERVER ]]; then
        echo "rsExecuteTask: No primary or backup server name specified"
        exit 3 
fi

if [[ -z $TASKNAMES ]]; then
	echo "rsExecuteTask: No task names specified"
	exit 4 
fi

if [[ ! -z  $FILEPATH ]];then
  if  [[ ! -a $FILEPATH ]];then
     echo "rsExecuteTask: security file $FILEPATH could not be found"
     exit 23
  fi
  if [[ ! -s $FILEPATH ]];then
     echo "rsExecuteTask: format of security file $FILEPATH is incorrect"
     exit 24
  fi
fi
##-------------------------------------------------------------------#
## Set-up environmental variables
##-------------------------------------------------------------------#
export CLASSPATH=.:$INSTALL/ibmjbrt.zip:$INSTALL/iioprt.jar:$INSTALL/omg-1.1.zip:$INSTALL/JavaRAS_1_0.zip:$INSTALL/rsCopyServices.jar:$JAVA_HOME/lib/classes.zip:$CLASSPATH
export PATH=$INSTALL:$JAVA_HOME/bin:$PATH
export LIBPATH=$INSTALL:/usr/lib


##-------------------------------------------------------------------#
## Set-up command options
##-------------------------------------------------------------------#
OPT_USERNAME=${USERNAME:+"-u $USERNAME"}
OPT_PASSWORD=${PASSWORD:+"-p $PASSWORD"}
OPT_FILEPATH=${FILEPATH:+"-a $FILEPATH"}

##-------------------------------------------------------------------#
## Test Connections if only Primary Server is given 
##-------------------------------------------------------------------#
if [[ ! -z $PRIMARY_SERVER && -z $BACKUP_SERVER ]];then
   SNAME="-Server=$PRIMARY_SERVER"
   $JAVA_HOME/bin/java rsCopyServices.rsWebTest $SNAME 2>&1 > /dev/null
   RC=$?
   if [[ $RC -ne 0 ]];then
      echo "rsExecuteTask: The primary server is down and there is no backup server specified."
      echo "               The task is not completed."
      exit 41
   elif [[ $RC -eq 0 ]]; then
      SERVERNAME=$PRIMARY_SERVER
   fi
fi

##-------------------------------------------------------------------#
## Test Connections if only Backup Server is given
##-------------------------------------------------------------------#
if [[ -z $PRIMARY_SERVER && ! -z $BACKUP_SERVER ]];then
   SNAME="-Server=$BACKUP_SERVER"
   $JAVA_HOME/bin/java rsCopyServices.rsWebTest $SNAME 2>&1 > /dev/null
   RC=$?
   if [[ $RC -ne 0 ]];then
      echo "rsExecuteTask: The backup server is down and there is no primary server specified."
      echo "               The task is not completed."
      exit 45 
   elif [[ $RC -eq 0 ]]; then
      SERVERNAME=$BACKUP_SERVER
   fi
fi

##-------------------------------------------------------------------#
## Test Connections if both of Primary  and Backup Server is given
##-------------------------------------------------------------------#
if [[ ! -z $PRIMARY_SERVER && ! -z $BACKUP_SERVER ]];then
   SNAME="-Server=$PRIMARY_SERVER"
   $JAVA_HOME/bin/java rsCopyServices.rsWebTest $SNAME 2>&1 > /dev/null
   RC=$?

   if [[ $RC -ne 0 ]]; then
        if [[ ! -z $VERBOSE ]]; then
           echo "rsExecuteTask: The primary server is down, trying the specified backup server ..."
        fi
        SNAME="-Server=$BACKUP_SERVER"
        $JAVA_HOME/bin/java rsCopyServices.rsWebTest $SNAME 2>&1 > /dev/null
        RC=$?
        if [[ $RC -ne 0 ]]; then
           echo "rsExecuteTask: The primary and backup server are down.  The task is not completed."
           exit 42 
        elif [[ $RC -eq 0 ]]; then
           if [[ ! -z $VERBOSE ]]; then
             echo "rsExecuteTask: The task will be run off the backup server."
           fi
           SERVERNAME=$BACKUP_SERVER
        fi
    elif [[ $RC -eq 0 ]]; then
      SERVERNAME=$PRIMARY_SERVER
    fi 
fi

##-------------------------------------------------------------------#
## Setting up the variable COMMAND 
##-------------------------------------------------------------------#
COMMAND=
if [[ ! -z $VERBOSE ]]; then
	COMMAND=$VERBOSE
fi

if [[ ! -z $SERVERNAME ]]; then
	COMMAND="$COMMAND -s $SERVERNAME"
fi

if [[ ! -z $COMMAND ]]; then
	COMMAND="$COMMAND $TASKNAMES"
else
	COMMAND=$TASKNAMES
fi

##-------------------------------------------------------------------#
## Call rsExecuteTask command 
##-------------------------------------------------------------------#
if [[ $VERBOSE != "-v" ]]; then
     $JAVA_HOME/bin/java rsCopyServices.rsExecuteTask $OPT_USERNAME $OPT_PASSWORD $OPT_FILEPATH $COMMAND 2>&1 > /dev/null
else
    $JAVA_HOME/bin/java rsCopyServices.rsExecuteTask $OPT_USERNAME $OPT_PASSWORD $OPT_FILEPATH $COMMAND
fi

##-------------------------------------------------------------------#
## Quit
##-------------------------------------------------------------------#
RC=$?
exit $RC
