#!/usr/bin/ksh

##-------------------------------------------------------------------#
## %PS
##    
## Licensed Internal Code - Property of IBM
##    
## 2105 Licensed Internal Code
##    
## (C) Copyright IBM Corp. 1999 All Rights Reserved.
##    
## US Government Users Restricted Rights - Use, duplication or
## disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
## %EPS
##
## COMPONENT_NAME (RAS WUI)
##
##-------------------------------------------------------------------#
##
## Filename: rsPrimeServer.sh
##
## Subsystem: RAS WUI
##
## Input:	rsPrimeServer.sh -v -u username -p password -a securityfilepath -d hostname -server name -backup server
##
##		-v = verbose mode 
##		-u = username
##		-p = password
##              -a = securityfilepath
##              -s = name of copy services sever
##              -b = name of copy services backup server
##              -d = name of the hostname to be deleted
##
## Description: Prime the copy services server with host information
##
##
## Change History:
##   Modifier               Date                      Change ID
##-------------------------------------------------------------------#
##  Richard Kirchhofer     11-08-1999			27625
##      Initial version
##  Richard Kirchhofer     12-02-1999			28254
##      Improvements for HP and SUN
##  Richard Kirchhofer     04 Feb 2000			29810
##      Standalone ORB conversion
##  Anne Chau              02-18-2000                          31101
##      Packaging simplification..call rsENV to setup INSTALL & JAVA_HOME
##  Anne Chau              03-08-2000                          32263
##      CLI function rsTestConnection does not work
##  Ramandeep Kaur &
##  Ruth Azevedo           05-30-2000                 35981
##      Command invocation fixed
##  Ruth Azevedo           05-30-2000                 34871
##      Modify INSTALL directory for AIX, SUN  and HP
##      Exit if OS is HP (temporary) 
##  Ramandeep Kaur         06-28-2000                      35050
##      changed iioprt.zip to iioprt.jar
##  Ramandeep Kaur         08-01-2000                      35549
##      included flag -d
##  John Wrobel            08-03-2000                      38511
##      Modified return codes
##      Removed HP exit
##  John Wrobel            08-03-2000                      38510
##      Added backup server and primary server to command line 
##      Test to see if primary and backup server functioning
##  Ramandeep Kaur         08-08-2000                      39105
##      Modified return codes    
##  Ramandeep Kaur         10-05-2000                      41892
##      Added the functionality of giving Backup
##      server by itself
##  Brian McCain           10-14-2000                      41890
##      Modified error checking to catch bad arguments
##  Ramandeep Kaur         10-19-2000                      42290
##      Modified env variable CLASSPATH
##  Brian McCain           10-20-2000                      43123
##      Display usage if no parameters specified
##  Ramandeep Kaur         11-02-2000                      43575
##      Added /a flag
##  Ramandeep Kaur         11-13-2000                      43880
##      Included a check if user gave space between - and flag 
##-------------------------------------------------------------------#
## End of PROLOG

OS=`uname | cut -c 1-3`

##-------------------------------------------------------------------#
## Set defaults
##-------------------------------------------------------------------#
SERVERNAME=
USERNAME=
PASSWORD=
FILEPATH=
OPTIONS=
PRIMARY_SERVER=
BACKUP_SERVER=
RC=

##-------------------------------------------------------------------#
## AIX specific code to set up INSTALL and JAVA_HOME
##-------------------------------------------------------------------#
if [[ $OS = "AIX" ]]; then

export INSTALL=/usr/opt/ibm2105cli
export JAVA_HOME=/usr/jdk_base

fi
##-------------------------------------------------------------------#
## Sun specific code
##-------------------------------------------------------------------#
if [[ $OS = "Sun" ]]; then

export INSTALL=/opt/ibm2105cli
export JAVA_HOME=/usr/java

fi
##-------------------------------------------------------------------#
## HP specific code
##-------------------------------------------------------------------#
if [[ $OS = "HP-" ]]; then

export INSTALL=/opt/ibm2105cli
export JAVA_HOME=/opt/java
fi

##-------------------------------------------------------------------#
## Process arguments
##-------------------------------------------------------------------#
USAGE='usage: rsPrimeServer.sh [-v] -u username & -p password | -a securityfilepath -d hostname -s primaryserver -b backupserver'

if [[ -z $1 ]]; then
    echo "rsPrimeServer: No parameters specified"
    echo $USAGE 
    exit 21
fi
   
while getopts ":vu:p:s:d:b:a:" opt; do
        if [[ $OPTARG = -* ]]; then
           echo "rsPrimeServer: Missing argument for -$opt"
           echo $USAGE
           exit 20
        fi
        case $opt in
          u  ) USERNAME=$OPTARG;;
          p  ) PASSWORD=$OPTARG;;
          a  ) FILEPATH=$OPTARG;;
          s  ) PRIMARY_SERVER=$OPTARG;;
          b  ) BACKUP_SERVER=$OPTARG;;
	  v  ) OPTIONS="-v";;
          d  ) UNREGISTERHOST=$OPTARG;;
          \? ) echo "rsPrimeServer: Invalid parameter specified"
               echo $USAGE
               exit 19
        esac
done
shift $(($OPTIND - 1))

while [[ ! -z $1 ]];do
    if [[ $1 = - ]]; then
         echo "rsPrimeServer: Invalid parameter specified"
         echo $USAGE
         exit 19
    fi
done

if [[ -z $PRIMARY_SERVER && -z $BACKUP_SERVER ]]; then
	echo "rsPrimeServer: No primary and backup server name specified"
        exit 3 
fi

if [[ ! -z $FILEPATH ]];then
  if  [[ ! -a $FILEPATH ]];then
     echo "rsExecuteTask: security file $FILEPATH could not be found"
     exit 23
  fi
  if [[ ! -s $FILEPATH ]];then
    echo "rsExecuteTask: format of security file $FILEPATH is incorrect"
    exit 24
  fi
fi

OPT_USERNAME=${USERNAME:+"-u $USERNAME"}
OPT_PASSWORD=${PASSWORD:+"-p $PASSWORD"}
OPT_FILEPATH=${FILEPATH:+"-a $FILEPATH"}
OPT_UNREGISTERHOST=${UNREGISTERHOST:+"-d $UNREGISTERHOST"}

##-------------------------------------------------------------------#
## Set-up environmental variables
##-------------------------------------------------------------------#
export CLASSPATH=.:$INSTALL/ibmjbrt.zip:$INSTALL/iioprt.jar:$INSTALL/omg-1.1.zip:$INSTALL/JavaRAS_1_0.zip:$INSTALL/rsCopyServices.jar:$JAVA_HOME/lib/classes.zip:$CLASSPATH
export PATH=$INSTALL:$JAVA_HOME/bin:$PATH
export LIBPATH=$INSTALL:/usr/lib


##-------------------------------------------------------------------#
## Test Connections if only Primary Server is given
##-------------------------------------------------------------------#
if [[ ! -z $PRIMARY_SERVER && -z $BACKUP_SERVER ]];then
   SNAME="-Server=$PRIMARY_SERVER"
   $JAVA_HOME/bin/java rsCopyServices.rsWebTest $SNAME 2>&1 > /dev/null
   RC=$?
   if [[ $RC -ne 0 ]];then
      echo "rsPrimeServer: The primary server is down and there is no backup server specified."
      echo "               The task is not completed."
      exit 41
   elif [[ $RC -eq 0 ]]; then
      SERVERNAME=$PRIMARY_SERVER
   fi
fi

##-------------------------------------------------------------------#
## Test Connections if only Backup Server is given
##-------------------------------------------------------------------#
if [[ -z $PRIMARY_SERVER && ! -z $BACKUP_SERVER ]];then
   SNAME="-Server=$BACKUP_SERVER"
   $JAVA_HOME/bin/java rsCopyServices.rsWebTest $SNAME 2>&1 > /dev/null
   RC=$?
   if [[ $RC -ne 0 ]];then
      echo "rsPrimeServer: The backup server is down and there is no primary server specified."
      echo "               The task is not completed."
      exit 45
   elif [[ $RC -eq 0 ]]; then
      SERVERNAME=$BACKUP_SERVER
   fi
fi

##-------------------------------------------------------------------#
## Test Connections if both of Primary  and Backup Server is given
##-------------------------------------------------------------------#
if [[ ! -z $PRIMARY_SERVER && ! -z $BACKUP_SERVER ]];then
   SNAME="-Server=$PRIMARY_SERVER"
   $JAVA_HOME/bin/java rsCopyServices.rsWebTest $SNAME 2>&1 > /dev/null
   RC=$?

   if [[ $RC -ne 0 ]]; then
        if [[ ! -z $OPTIONS ]]; then
           echo "rsPrimeServer: The primary server is down, trying the specified backup server."
        fi
        SNAME="-Server=$BACKUP_SERVER"
        $JAVA_HOME/bin/java rsCopyServices.rsWebTest $SNAME 2>&1 > /dev/null
        RC=$?
        if [[ $RC -ne 0 ]]; then
           echo "rsPrimeServer: The primary and backup server are down. The task is not completed."
           exit 42
        elif [[ $RC -eq 0 ]]; then
           if [[ ! -z $OPTIONS ]]; then
             echo "rsPrimeServer: The task will be run off the backup server."
           fi
           SERVERNAME=$BACKUP_SERVER
        fi
    elif [[ $RC -eq 0 ]]; then
      SERVERNAME=$PRIMARY_SERVER
    fi
fi
OPT_SERVERNAME=${SERVERNAME:+"-s $SERVERNAME"}

##-------------------------------------------------------------------#
## Find mapping of 2105 volumes
##-------------------------------------------------------------------#
RESULTS=`$INSTALL/rsList2105s.sh -n`
RC=$?

if [[ $RC -ne 0 ]]; then
	exit $RC
fi

##-------------------------------------------------------------------#
## Build command sequence for volume pairs
##-------------------------------------------------------------------#
COUNT=0
COMMAND=

for NEXT in $RESULTS; do
        case $COUNT in
                0) PREFIX="-l" ;;
                1) PREFIX="-r" ;;
        esac

        if [[ $COUNT -eq 1 ]]; then

                SERIALNUMBER="$PREFIX $NEXT"
                COMMAND="$COMMAND $DISKNAME $SERIALNUMBER"
                COUNT=0

        else

                DISKNAME="$PREFIX $NEXT"
                COUNT=$(($COUNT + 1))

        fi

done

##-------------------------------------------------------------------#
## Call PrimeServer command
##-------------------------------------------------------------------#
if [[ $OPTIONS != "-v" ]]; then
	$JAVA_HOME/bin/java rsCopyServices.rsPrimeServer $OPTIONS $OPT_USERNAME $OPT_PASSWORD $OPT_FILEPATH $OPT_UNREGISTERHOST $OPT_SERVERNAME $COMMAND 2>&1 > /dev/null
else
        $JAVA_HOME/bin/java rsCopyServices.rsPrimeServer $OPTIONS $OPT_USERNAME $OPT_PASSWORD $OPT_FILEPATH $OPT_UNREGISTERHOST $OPT_SERVERNAME $COMMAND
fi
RC=$?
	
##-------------------------------------------------------------------#
## Quit
##-------------------------------------------------------------------#
exit $RC
