#!/usr/bin/ksh

##-------------------------------------------------------------------#
## %PS
##    
## Licensed Internal Code - Property of IBM
##    
## 2105 Licensed Internal Code
##    
## (C) Copyright IBM Corp. 1999 All Rights Reserved.
##    
## US Government Users Restricted Rights - Use, duplication or
## disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
## %EPS
##
## COMPONENT_NAME (RAS WUI)
##
##-------------------------------------------------------------------#
##
## Filename: rsQueryComplete.sh
##
## Subsystem: RAS WUI
##
## Input:	rsQueryComplete.sh -v -u username -p password -a securityfilepath -m minutes -t threshold -s serverName -b backup_server taskName
##
##		-v = verbose
##		-u = username
##		-p = password
##              -a = securityfilepath
##              -b = backup server
##		-m = minutes to wait between checks of whether PPRC
##		     copy initialization has completed
##		-t = threshold percentage which determines 
##		     completion. Default=100%
##		-s = name of copy services server
##		taskName = name of pre-defined Copy Services Server
##			   PPRC task.
##
## Description: Wait on the completion of PPRC copy initialization.
##
##
## Change History:
##   Modifier               Date                             Change ID
##-------------------------------------------------------------------#
##  Richard Kirchhofer     12/03/1999			     28254
##      Initial version
##  Richard Kirchhofer     04 Feb 2000				29810
##      Standalone ORB conversion
##  Anne Chau              02-18-2000                          31101
##      Packaging simplification..call rsENV to setup INSTALL & JAVA_HOME
##  Anne Chau              03-08-2000                          32263
##      CLI function rsTestConnection does not work
##  Ruth E. Azevedo        05-30-2000                          34871
##      Modify INSTALL directory for AIX, SUN and HP          
##  Ramandeep Kaur         06-28-2000                      35050
##      changed iioprt.zip to iioprt.jar
##  John Wrobel            07-21-2000                          38511
##      Modified return codes
##  John Wrobel            07-21-2000                          38510
##      Added backup server parameter to command line
##      Test to see if primary/backup servers functioning
##  Ramandeep Kaur         08-08-2000                          39105
##      Modified return codes
##  Ramandeep Kaur         10-10-2000                          42071
##      Included functionality of giving more than one task
##  Ramandeep Kaur         10-12-2000                          41892
##      Added the functionality of giving Backup Server alone
##  Brian McCain           10-14-2000                          41890
##      Modified error checking to catch bad arguments
##  Ramandeep Kaur         10-19-2000                          42290
##      Modified env variable CLASSPATH
##  Brian McCain           10-20-2000                          43123
##      Display usage if no parameters specified
##  Ramandeep Kaur         11-02-2000                          43575
##      Added flag -a
##  Ramandeep Kaur         11-14-2000                          43880
##      Included check if user gave space between - and flag
##-------------------------------------------------------------------#
## End of PROLOG

OS=`uname | cut -c 1-3`
##-------------------------------------------------------------------#
## Set defaults
##-------------------------------------------------------------------#
SERVERNAME=
PRIMARY_SERVER=
BACKUP_SERVER=
USERNAME=
PASSWORD=
FILEPATH=
VOLUME=
MINUTES=
THRESHOLD=
VERBOSE=
TASKNAMES=
RC=

##-------------------------------------------------------------------#
## AIX specific code to set up INSTALL and JAVA_HOME
##-------------------------------------------------------------------#
if [[ $OS = "AIX" ]]; then

export INSTALL=/usr/opt/ibm2105cli
export JAVA_HOME=/usr/jdk_base

fi

##-------------------------------------------------------------------#
## Sun specific code
##-------------------------------------------------------------------#
if [[ $OS = "Sun" ]]; then

export INSTALL=/opt/ibm2105cli
export JAVA_HOME=/usr/java

fi

##-------------------------------------------------------------------#
## HP specific code
##-------------------------------------------------------------------#
if [[ $OS = "HP-" ]]; then

export INSTALL=/opt/ibm2105cli
export JAVA_HOME=/opt/java

fi

##-------------------------------------------------------------------#
## Process arguments
##-------------------------------------------------------------------#
USAGE='usage: rsQueryComplete.sh [-v] -u username & -p password | -a securityfilepath [-m minutes] [-t threshold] -s primaryserver -b backupserver taskName'

if [[ -z $1 ]]; then
    echo "rsQueryComplete: No parameters specified"
    echo $USAGE
    exit 21
fi

while getopts ":vm:t:s:u:p:b:a:" opt; do
        if [[ $OPTARG = -* ]]; then
           echo "rsQueryComplete: Missing argument for -$opt"
           echo $USAGE
           exit 20
        fi
        case $opt in
          m  ) MINUTES=$OPTARG;;
          t  ) THRESHOLD=$OPTARG;;
          u  ) USERNAME=$OPTARG;;
          p  ) PASSWORD=$OPTARG;;
	  a  ) FILEPATH=$OPTARG;;
	  s  ) PRIMARY_SERVER=$OPTARG;;
          b  ) BACKUP_SERVER=$OPTARG;;
          v  ) VERBOSE=-v;;
          \? ) echo "rsQueryComplete: Invalid parameter specified"
               echo $USAGE
               exit 19
        esac
done
shift $(($OPTIND - 1))

while [[ ! -z $1 ]]; do
        if [[ $1 = - ]];then
            echo "rsQueryComplete: Invalid parameter specified"
            echo $USAGE
            exit 19
        fi
        if [[ -z $TASKNAMES ]]; then
                TASKNAMES=$1
        else
                TASKNAMES="$TASKNAMES $1"
        fi
        shift
done

if [[ -z $PRIMARY_SERVER && -z $BACKUP_SERVER ]]; then
        echo "rsQueryComplete: No primary or backup server name specified"
        exit 3
fi

if [[ -z $TASKNAMES ]]; then
	echo "rsQueryComplete: No task names specified"
	exit 4 
fi

if [[ ! -z $FILEPATH ]];then
  if  [[ ! -a $FILEPATH ]];then
     echo "rsExecuteTask: security file $FILEPATH could not be found"
     exit 23
  fi
  if [[ ! -s $FILEPATH ]];then
    echo "rsExecuteTask: format of security file $FILEPATH is incorrect"
    exit 24
  fi
fi

##-------------------------------------------------------------------#
## Set-up environmental variables
##-------------------------------------------------------------------#
export CLASSPATH=.:$INSTALL/ibmjbrt.zip:$INSTALL/iioprt.jar:$INSTALL/omg-1.1.zip:$INSTALL/JavaRAS_1_0.zip:$INSTALL/rsCopyServices.jar:$JAVA_HOME/lib/classes.zip:$CLASSPATH
export PATH=$INSTALL:$JAVA_HOME/bin:$PATH
export LIBPATH=$INSTALL:/usr/lib

##-------------------------------------------------------------------#
## Test Connections if only Primary Server is given
##-------------------------------------------------------------------#
if [[ ! -z $PRIMARY_SERVER && -z $BACKUP_SERVER ]];then
   SNAME="-Server=$PRIMARY_SERVER"
   $JAVA_HOME/bin/java rsCopyServices.rsWebTest $SNAME 2>&1 > /dev/null
   RC=$?
   if [[ $RC -ne 0 ]];then
      echo "rsQueryComplete: The primary server is down and there is no backup server specified."
      echo "                 The task is not completed."
      exit 41
   elif [[ $RC -eq 0 ]]; then
      SERVERNAME=$PRIMARY_SERVER
   fi
fi

##-------------------------------------------------------------------#
## Test Connections if only Backup Server is given
##-------------------------------------------------------------------#
if [[ -z $PRIMARY_SERVER && ! -z $BACKUP_SERVER ]];then
   SNAME="-Server=$BACKUP_SERVER"
   $JAVA_HOME/bin/java rsCopyServices.rsWebTest $SNAME 2>&1 > /dev/null
   RC=$?
   if [[ $RC -ne 0 ]];then
      echo "rsQueryComplete: The backup server is down and there is no primary server specified."
      echo "                 The task is not completed."
      exit 45
   elif [[ $RC -eq 0 ]]; then
      SERVERNAME=$BACKUP_SERVER
   fi
fi

##-------------------------------------------------------------------#
## Test Connections if both of Primary  and Backup Server is given
##-------------------------------------------------------------------#
if [[ ! -z $PRIMARY_SERVER && ! -z $BACKUP_SERVER ]];then
   SNAME="-Server=$PRIMARY_SERVER"
   $JAVA_HOME/bin/java rsCopyServices.rsWebTest $SNAME 2>&1 > /dev/null
   RC=$?

   if [[ $RC -ne 0 ]]; then
        if [[ ! -z $VERBOSE ]]; then
           echo "rsQueryComplete: The primary server is down, trying the specified backup server ..."
        fi
        SNAME="-Server=$BACKUP_SERVER"
        $JAVA_HOME/bin/java rsCopyServices.rsWebTest $SNAME 2>&1 > /dev/null
        RC=$?
        if [[ $RC -ne 0 ]]; then
           echo "rsQueryComplete: The primary and backup server are down.  The task is not completed."
           exit 42
        elif [[ $RC -eq 0 ]]; then
           if [[ ! -z $VERBOSE ]]; then
             echo "rsQueryComplete: The task will be run off the backup server."
           fi
           SERVERNAME=$BACKUP_SERVER
        fi
    elif [[ $RC -eq 0 ]]; then
      SERVERNAME=$PRIMARY_SERVER
    fi
fi

##-------------------------------------------------------------------#
## Setting up the variable COMMAND
##-------------------------------------------------------------------#
COMMAND=
if [[ ! -z $VERBOSE ]]; then
	COMMAND=$VERBOSE
fi

if [[ ! -z $THRESHOLD ]]; then
	if [[ $THRESHOLD -lt 100 ]]; then
		COMMAND="$COMMAND -t $THRESHOLD"
	fi
fi

if [[ ! -z $MINUTES ]]; then
	COMMAND="$COMMAND -m $MINUTES"
fi

if [[ ! -z $SERVERNAME ]]; then
	COMMAND="$COMMAND -s $SERVERNAME"
fi

if [[ ! -z $COMMAND ]]; then
	COMMAND="$COMMAND $TASKNAMES"
else
	COMMAND=$TASKNAMES
fi

OPT_USERNAME=${USERNAME:+"-u $USERNAME"}
OPT_PASSWORD=${PASSWORD:+"-p $PASSWORD"}
OPT_FILEPATH=${FILEPATH:+"-a $FILEPATH"}

##-------------------------------------------------------------------#
## Call QueryComplete command with specified options
##-------------------------------------------------------------------#
$JAVA_HOME/bin/java rsCopyServices.rsQueryComplete $OPT_USERNAME $OPT_PASSWORD $OPT_FILEPATH $COMMAND

##-------------------------------------------------------------------#
## Quit
##-------------------------------------------------------------------#
RC=$?
exit $RC 
