/*----------------------------------------------------------------------
 * IBM Confidential
 * OCO Source Materials
 *
 * (C) Copyright IBM Corporation 1998.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *--------------------------------------------------------------------*/

package com.ibm.ras;

import java.util.*;

/**
 * <code>RASEvent</code> is the parent of all
 * <code>RASMessageEvent</code> and <code>RASTraceEvent</code> objects.  
 * <I>These classes are used within this RAS system to encapsulate
 * the message and trace data created by an application.  An 
 * application should not need to use them.  They would only be used
 * by a RAS implementation that wishes to extend the function of this
 * RAS Toolkit.</I>
 * <p>
 * <code>RASEvent</code> contains the following information:
 * <ul>
 * <li>A flag which identifies this event as a message event or a trace
 * event.
 * <li>A time stamp, generated automatically when the event is created.
 * <li>A "type" field, which defines the particular flavor of the event.
 * For example, a message event might be one of "informational," "warning,"
 * or "error."  These types are defined by the extensions to this class.  See
 * <a href=RASObject.html><code>RASObject</code></a>
 * for additional information on this important point.
 * <li>The text of the event.
 * <li>An array of run-time parameters which may be inserted into the text
 * or displayed with the text.
 * <li>Zero or more additional attributes that are specific to different RAS 
 * event types.
 * </ul>
 *
 * @see RASMessageEvent
 * @see RASTraceEvent
 * @see RASLogger
 * @see RASHandler
 */

public abstract class RASEvent implements RASIEvent
{
private static final String S = RASCopyright.COPYRIGHT_98;

/**
 * @serial
 *   A flag which is set <code>true</code> when the event is a 
 *   message event or <code>false</code> when the event is a 
 *   trace event.
 */

private boolean isMessageEvent;

/**
 * @serial
 *   The type of event.  This value may be the logical
 *   OR of any of the <code>RASIMessageEvent</code> or
 *   <code>RASITraceEvent TYPE_XXXX</code> constants.
 */

private long type = 0;

/**
 * @serial
 *   The time at which this event was created.  This value
 *   is the output of the <code>System.currentTimeMillis</code>
 *   method.
 */

private long timeStamp;

/**
 * @serial
 *   The event msgText (message).
 */

private String msgText = "";

/**
 * @serial
 *   A set of <code>Strings</code> to be inserted into the event text.
 */

private String[] msgParms = null;

/**
 * @serial
 *   A <code>Hashtable</code> containing the additional attributes that
 *   make up the RAS event.
 */

private Hashtable attribs = null;


/*----------------------------------------------------------------------
 * Name:  RASEvent
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASEvent</code>.  The time stamp is generated
 * at this time.
 *  
 * @param source The <code>Object</code> which created this event.
 */

public RASEvent() 
{
setTimeStamp(System.currentTimeMillis());
}


/*----------------------------------------------------------------------
 * Name:  getTimeStamp
 *--------------------------------------------------------------------*/
/**
 * Gets the time at which this event was created.
 * 
 * @param timeStamp The event time stamp, as generated by 
 *                  <code>System.currentTimeMillis</code>.
 **/

public long getTimeStamp()
{
return timeStamp;
}


/*----------------------------------------------------------------------
 * Name:  setTimeStamp
 *--------------------------------------------------------------------*/
/**
 * Sets the time at which this event was created.  This value is 
 * expected to be of the format returned by 
 * <code>System.currentTimeMillis</code>.
 * 
 * @param timeStamp The event time stamp.
 **/

public void setTimeStamp(long timeStamp)
{
this.timeStamp = timeStamp;
}


/*----------------------------------------------------------------------
 * Name:  getType
 *--------------------------------------------------------------------*/
/**
 * Gets the type of event:  "informational message," for example.
 * Specific types are defined by the classes which extend <code>RASEvent</code>.
 * 
 * @return The event type.
 **/

public long getType()
{
return type;
}


/*----------------------------------------------------------------------
 * Name:  setType
 *--------------------------------------------------------------------*/
/**
 * Sets the type of event:  "informational message," for example.
 * Specific types are defined by the classes which extend <code>RASEvent</code>.
 * 
 * @param type The event type.
 **/

public void setType(long type)
{
this.type = type;
}


/*----------------------------------------------------------------------
 * Name:  getText
 *--------------------------------------------------------------------*/
/**
 * Gets the event text.
 * 
 * @return The event text.

 **/

public String getText()
{
return msgText;
}


/*----------------------------------------------------------------------
 * Name:  setText
 *--------------------------------------------------------------------*/
/**
 * Sets the event text.
 * 
 * @param text The event text.
 **/

public void setText(String text)
{
this.msgText = text;
}


/*----------------------------------------------------------------------
 * Name:  getParameters
 *--------------------------------------------------------------------*/
/**
 * Gets the text parameters:  <code>Strings</code> to be displayed with
 * the message text.
 * 
 * @return The text parameters.
 **/

public String[] getParameters()
{
return msgParms;
}


/*----------------------------------------------------------------------
 * Name:  setParameters
 *--------------------------------------------------------------------*/
/**
 * Gets the text parameters:  <code>Strings</code> to be displayed with
 * the message text.
 * 
 * @param parms The text parameters.
 **/

public void setParameters(String[] parms)
{
msgParms = parms;
}


/*----------------------------------------------------------------------
 * Name:  getAttribute
 *--------------------------------------------------------------------*/
/**
 * Gets an attribute that is saved as part of a RAS event.
 * 
 * @param name The name of the attribute. 
 * 
 * @return The attribute associated with the name or <code>null</code> 
 *         if the named attribute was not found.
 **/

public Object getAttribute(String name)
{
Object attrib = null;

if (name != null && attribs != null)
  attrib = attribs.get(name);

return attrib;
}


/*----------------------------------------------------------------------
 * Name:  setAttribute
 *--------------------------------------------------------------------*/
/**
 * Sets an attribute, saving it as part of a RAS event.  If the name or 
 * the attribute are <code>null</code>, nothing is saved.
 * 
 * @param name   The name of the attribute. 
 * @param attrib The attribute associated with the name.
 **/

public void setAttribute(String name, Object attrib)
{
if (name != null && attrib != null)
  {
  if (attribs == null) 
    attribs = new Hashtable();
  attribs.put(name, attrib);
  }
}


/*----------------------------------------------------------------------
 * Name:  setMessageEvent
 *--------------------------------------------------------------------*/
/**
 * Sets a flag that that indicates whether this object is classified 
 * as a message event or a trace event.
 *
 * @param flag A <code>boolean</code> set <code>true</code> if this
 *             object is a message event and <code>false</code> if it
 *             is a trace event.
 **/

public void setMessageEvent(boolean flag)
{
isMessageEvent = flag;
}


/*----------------------------------------------------------------------
 * Name:  isMessageEvent
 *--------------------------------------------------------------------*/
/**
 * Determines if this object is a message event or a trace event.
 *
 * @return <code>true</code> if this object is a message event and 
 *         <code>false</code> if it is a trace event.
 **/

public boolean isMessageEvent()
{
return isMessageEvent;
}


/*----------------------------------------------------------------------
 * Name:  getSupportedTypes
 *--------------------------------------------------------------------*/
/**
 * Gets the complete set of event types supported by this class.
 * <p>
 * This method can be used by a graphical log manager to display the 
 * set of types and allow a user to select those to be monitored.  Every
 * extending class should override this method to add the types it supports
 * to the <code>Hashtable</code> returned by its parent.
 *
 * @return A <code>Hashtable</code> containing the values of all of the
 *         event types.  The names of the types are used as keys and
 *         are returned in the current locale.  The type values,
 *         normally kept as <code>ints</code>, are returned as
 *         <code>Integers</code> because of the <code>Hashtable</code>
 *         requirement that all elements be <code>Objects</code>.
 */

public Hashtable getSupportedTypes()
{

// This class does not define any event types.  It will just return
// an empty Hashtable.

return new Hashtable();
}

}
