/*----------------------------------------------------------------------
 * IBM Confidential
 * OCO Source Materials
 *
 * (C) Copyright IBM Corporation 1998.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *--------------------------------------------------------------------*/

package com.ibm.ras;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <code>RASFormatter</code> is the parent of all classes which format
 * RAS events for display.  RAS formatters are used by those RAS handlers
 * which generate human-readable RAS events.
 * <p>
 * This class provides functions to format the date and time at which
 * a RAS event was created.  If not specified, the default
 * <code>SimpleDateFormat</code> pattern used to format the date is
 * "yyyy.MM.dd" and the pattern used for the time is "HH:mm:ss.SSS".
 * These defaults can be changed by the <code>setDateFormat</code> and
 * <code>setTimeFormat</code> methods.
 * <p>
 * Programming Note:  classes which extend <code>RASFormatter</code>
 * should, in their constructors, call the <code>addEventClass</code>
 * method to register the <code>RASIEvent</code> classes which the
 * formatter supports.
 *
 * @see RASMessageFormatter
 * @see RASTraceFormatter
 * @see RASHandler
 */

public abstract class RASFormatter extends RASObject implements RASIFormatter
{
private static final String S = RASCopyright.COPYRIGHT_98;

/**
 * The <code>SimpleDateFormat</code> object used to format the 
 * date.
 */

private SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy.MM.dd");

/**
 * The <code>SimpleDateFormat</code> object used to format the 
 * time.
 */

private SimpleDateFormat timeFormat = new SimpleDateFormat("HH:mm:ss.SSS");

/**
 * The <code>String</code> used to separate fields in the formatted
 * output.
 */

private String separator = " ";

/**
 * The names of the <code>RASIEvent</code> classes which this formatter 
 * is able to process.
 */

private Vector eventClasses = new Vector();


/*----------------------------------------------------------------------
 * Name:  RASFormatter
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASFormatter</code>.
 */

public RASFormatter()
{
}


/*----------------------------------------------------------------------
 * Name:  getSeparator
 *--------------------------------------------------------------------*/
/**
 * Gets the <code>String</code> used to separate fields in the formatted
 * output.

 * @return The separator <code>String</code>, whose default value is " ".
 **/

public String getSeparator()
{
return separator;
}


/*----------------------------------------------------------------------
 * Name:  setSeparator
 *--------------------------------------------------------------------*/
/**
 * Sets the <code>String</code> used to separate fields in the formatted
 * output.
 *
 * @param sep The separator <code>String</code>.  If the separator is 
 *            <code>null</code>, the current separator is not changed.
 **/

public void setSeparator(String sep)
{
if (sep != null)
  separator = sep;
}


/*----------------------------------------------------------------------
 * Name:  getDateFormat
 *--------------------------------------------------------------------*/
/**
 * Gets the <code>SimpleDateFormat</code> object used to format the date
 * at which the event was created.
 *
 * @return The date format.
 **/

public SimpleDateFormat getDateFormat()
{
return dateFormat;
}


/*----------------------------------------------------------------------
 * Name:  setDateFormat
 *--------------------------------------------------------------------*/
/**
 * Sets the <code>SimpleDateFormat</code> object used to format the date
 * at which the event was created.
 *
 * @param format The date format.  If the format is <code>null</code>,
 *               the current format is not changed.
 **/

public void setDateFormat(SimpleDateFormat format)
{
if (format != null)
  dateFormat = format;
}


/*----------------------------------------------------------------------
 * Name:  getTimeFormat
 *--------------------------------------------------------------------*/
/**
 * Gets the <code>SimpleDateFormat</code> object used to format the time
 * at which the event was created.
 *
 * @return The time format.
 **/

public SimpleDateFormat getTimeFormat()
{
return timeFormat;
}


/*----------------------------------------------------------------------
 * Name:  setTimeFormat
 *--------------------------------------------------------------------*/
/**
 * Sets the <code>SimpleDateFormat</code> object used to format the time
 * at which the event was created.
 *
 * @param format The time format.  If the format is <code>null</code>,
 *               the current format is not changed.
 **/

public void setTimeFormat(SimpleDateFormat format)
{
if (format != null)
  timeFormat = format;
}


/*----------------------------------------------------------------------
 * Name:  getDate
 *--------------------------------------------------------------------*/
/**
 * Gets the date on which this event was created.
 *
 * @param timeStamp The timestamp at which the event was created, as
 *                  formed by <code>System.currentTimeMillis</code>.
 *
 * @return The event date, formatted for display.
 **/

public String getDate(long timeStamp)
{
return dateFormat.format(new Date(timeStamp));
}


/*----------------------------------------------------------------------
 * Name:  getTime
 *--------------------------------------------------------------------*/
/**
 * Gets the time at which this event was created.
 *
 * @param timeStamp The timestamp at which the event was created, as
 *                  formed by <code>System.currentTimeMillis</code>.
 *
 * @return The event time, formatted for display.
 **/

public String getTime(long timeStamp)
{
return timeFormat.format(new Date(timeStamp));
}


/*----------------------------------------------------------------------
 * Name:  addEventClass
 *--------------------------------------------------------------------*/
/**
 * Adds the name of a <code>RASIEvent</code> class which this formatter
 * can process. If the event class is <code>null</code> or is already 
 * registered, this method does nothing.
 *
 * @param name The event class name.
 */

public void addEventClass(String name)
{
if (name != null && !eventClasses.contains(name))
  {
  eventClasses.addElement(name);
  }
}


/*----------------------------------------------------------------------
 * Name:  removeEventClass
 *--------------------------------------------------------------------*/
/**
 * Removes the name of a <code>RASIEvent</code> class from the list of
 * classes which this formatter can process. If the event class is 
 * <code>null</code> or is not registered, this method does nothing.
 *
 * @param name The event class name.
 */

public void removeEventClass(String name)
{
if (name != null && eventClasses.contains(name))
  eventClasses.removeElement(name);
}


/*----------------------------------------------------------------------
 * Name:  getEventClasses
 *--------------------------------------------------------------------*/
/**
 * Gets the set of <code>RASIEvent</code> classes which this formatter
 * can process.
 *
 * @return An <code>Enumeration</code> of RAS event class names.  If
 *         no event classes are registered, the <code>Enumeration</code>
 *         is empty.
 */

public Enumeration getEventClasses()
{
return eventClasses.elements();
}


/*----------------------------------------------------------------------
 * Name:  format
 *--------------------------------------------------------------------*/
/**
 * Formats the RAS event.
 * 
 * @param event The event to format.
 * 
 * @return The formatted event.
 **/

public abstract String format(RASIEvent event);
}
