/*----------------------------------------------------------------------
 * IBM Confidential
 * OCO Source Materials
 *
 * (C) Copyright IBM Corporation 1998.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *--------------------------------------------------------------------*/

package com.ibm.ras;

import java.io.*;
import java.util.*;

/**
 * <code>RASHandler</code> implements the <code>RASIHandler</code>
 * interface and is the parent of all classes which process
 * the output of the <code>RASLogger</code> classes.
 * At least one <code>RASHandler</code> must be associated with 
 * each <code>RASLogger</code>. This is done through the 
 * <code>RASLogger.addHandler</code> method.  
 * <p>
 * Each handler operates on its own thread so that log entries can be sent
 * to their destination at whatever rate the transport can manage.  This
 * means that an application, for example, does not have to wait for each trace
 * event to be written to disk as it runs.  The message and trace data is 
 * accepted from the loggers in the form of <code>RASIEvent</code> objects.
 * These are simply an encapsulation of the message or trace data that the 
 * application creates through the <code>RASLogger</code> methods.
 * <p>
 * It is possible to direct both message and trace data to the
 * same destination (the console or a socket, for example).
 * However, it is certainly possible to configure separate handlers to direct 
 * message and trace data to different destinations (two separate files, for 
 * example).
 * <p>
 * Each event class which a handler processes may be associated with a 
 * formatter, which defines the presentation of the log entries.  The use of a 
 * formatter depends on the function of the handler.  For example, if the 
 * handler directs data to the console, it should be formatted.  However, if 
 * the handler is designed to serialize the message or trace data and transport
 * it across a network, formatting may not be required.  The 
 * <code>RASHandler write</code> methods require a formatter, but may be
 * overridden if desired.
 * (Because a formatter is not required, the methods to get and set the 
 * message and trace formatters are not included in the 
 * <code>RASIHandler</code> interface, which this object implements.)
 * <p>
 * <code>RASMessageFormatter</code> and <code>RASTraceFormatter</code> are
 * automatically registered to process the <code>RASMessageEvent</code> and
 * <code>RASTraceEvent</code> classes, respectively.
 *
 * @see RASIHandler
 * @see RASConsoleHandler
 * @see RASFileHandler
 * @see RASSocketHandler
 * @see RASLogger
 * @see RASEvent
 * @see RASMessageFormatter
 * @see RASTraceFormatter
 */

public abstract class RASHandler extends RASObject implements RASIHandler, Runnable
{
private static final String S = RASCopyright.COPYRIGHT_98;

/**
 * A flag which indicates whether the device to which this handler sends log
 * entries is open.  It is set <code>true</code> when the device is open and
 *<code>false</code> otherwise.
 */

public boolean deviceOpen = false;

/**
 * A flag set <code>true</code> when the handler has been asked to stop
 * and its queue is empty and <code>false</code> otherwise.
 */

private boolean done = false;

/**
 * A flag which indicates whether this handler has been asked to stop.
 */

private boolean stopRequested = false;

/**
 * The RAS events and their associated formatters.
 */

protected Hashtable formatters = new Hashtable();

/**
 * The mask which determines the types of messages that will
 * be processed.
 */

private long messageMask = RASIMessageEvent.DEFAULT_MESSAGE_MASK;

/**
 * The mask which determines the types of trace points that will
 * be processed.
 */

private long traceMask = RASITraceEvent.DEFAULT_TRACE_MASK;

/**
 * The <code>PrintWriter</code> that sends a formatted log
 * event to its destination.
 */

protected PrintWriter pWriter = null;

/**
 * A queue of <code>RASEvent</code> objects to be 
 * processed.
 */

private Vector logQueue = new Vector(25, 25);

/**
 * The set of <code>RASIMaskChangeListeners</code> which want to be
 * informed of changes to the <code>RASHandler</code> message or trace
 * mask configuration
 */

private Vector maskChangeListeners = new Vector();


/*----------------------------------------------------------------------
 * Name:  RASHandler
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASHandler</code>.  The name and description of
 * this object are empty strings.
 */

public RASHandler() 
{
this("", "");
}


/*----------------------------------------------------------------------
 * Name:  RASHandler
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASHandler</code>.  The description of this object
 * is an empty string.
 *
 * @param name The name of this object.
 */

public RASHandler(String name) 
{
this(name, "");
}


/*----------------------------------------------------------------------
 * Name:  RASHandler
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASHandler</code>.
 *
 * @param name The name of this object.
 * @param desc The description of this object.
 */

public RASHandler(String name, String desc) 
{
super(name, desc);

// Add default formatters for the event classes in this package.

addFormatter(new RASMessageFormatter());
addFormatter(new RASTraceFormatter());

// Start this handler on its own thread so that it can asynchronously
// process any log event objects put on its queue -- without adversely 
// affecting the performance of the application.

Thread thread = new Thread(this);
thread.setDaemon(true);
thread.setName(this.getClass().getName() + ":" + this.getName());
thread.start();
}


/*----------------------------------------------------------------------
 * Name:  getConfig
 *--------------------------------------------------------------------*/
/**
 * Gets the configuration of this object.
 *
 * @return A <code>Hashtable</code> containing the configuration.
 *         This object inserts the following key/value pairs into the
 *         the configuration:
 *         <pre>
 *         "messageMask" The mask which selects the message types to
 *                       be processed.
 *         "traceMask"   The mask which selects the trace types to be
 *                       processed.
 *         </pre>
 *         All values are <code>Strings</code>.
 *         The parent and extensions of this object may use additional keys.
 **/

public Hashtable getConfig()
{
Hashtable ht = super.getConfig();

ht.put("messageMask", RASMessageEvent.maskToString(getMessageMask()));
ht.put("traceMask", RASTraceEvent.maskToString(getTraceMask()));

return ht;
}


/*----------------------------------------------------------------------
 * Name:  setConfig
 *--------------------------------------------------------------------*/
/**
 * Sets the configuration of this object.  This method is used by a 
 * <code>RASManager</code> to initialize a RAS object.  It should not 
 * be necessary for an application to use this method.
 *
 * @param ht  A <code>Hashtable</code> containing the configuration.
 *            This object searches for the following keys:
 *            <pre>
 *            "messageMask" The mask which selects the message types to
 *                          be processed.
 *            "traceMask"   The mask which selects the trace types to be
 *                          processed.
 *            </pre>
 *            All values are <code>Strings</code>.
 *            If a key is not found, that configuration element is not
 *            updated.
 *            The parent and extensions of this object may use additional keys.
 **/

public void setConfig(Hashtable ht)
{
super.setConfig(ht);

if (ht.containsKey("messageMask"))
  setMessageMask(RASMessageEvent.maskLongValue((String) ht.get("messageMask")));
if (ht.containsKey("traceMask"))
  setTraceMask(RASTraceEvent.maskLongValue((String) ht.get("traceMask")));
}


/*----------------------------------------------------------------------
 * Name:  getMessageMask
 *--------------------------------------------------------------------*/
/**
 * Gets the mask which defines the set of message types that will be 
 * processed.  Possible values are defined by the logical OR of the  
 * <code>RASIMessageEvent.TYPE_XXXX</code> constants.
 *
 * @return The trace mask.
 */

public long getMessageMask() 
{
return messageMask;
}


/*----------------------------------------------------------------------
 * Name:  setMessageMask
 *--------------------------------------------------------------------*/
/**
 * Sets the mask which defines the set of message types that will be 
 * processed.  Possible values are defined by the 
 * <code>RASIMessageEvent.TYPE_XXXX</code> constants.
 * The mask value is not validated against these types.
 *
 * @param mask The message mask.
 */

public void setMessageMask(long mask) 
{
messageMask = mask;
fireMaskChangedEvent();
}


/*----------------------------------------------------------------------
 * Name:  getTraceMask
 *--------------------------------------------------------------------*/
/**
 * Gets the mask which defines the set of trace types that will be 
 * processed.  Possible values are defined by the logical OR of the
 * <code>RASITraceEvent.TYPE_XXXX</code> constants.
 *
 * @return The trace mask.
 */

public long getTraceMask() 
{
return traceMask;
}


/*----------------------------------------------------------------------
 * Name:  setTraceMask
 *--------------------------------------------------------------------*/
/**
 * Sets the mask which defines the set of trace types that will be 
 * processed.  Possible values are defined by the 
 * <code>RASITraceEvent.TYPE_XXXX</code> constants.
 * The mask value is not validated against these types.
 *
 * @param mask The trace mask.
 */

public void setTraceMask(long mask) 
{
traceMask = mask;
fireMaskChangedEvent();
}


/*----------------------------------------------------------------------
 * Name:  fireMaskChangedEvent
 *--------------------------------------------------------------------*/
/**
 * Tell all <code>RASIMaskChangeListeners</code> that the message or
 * trace mask has been changed.
 */

protected void fireMaskChangedEvent()
{
RASIMaskChangeListener c;
Enumeration e = getMaskChangeListeners();
while (e.hasMoreElements())
  {
  c = (RASIMaskChangeListener) e.nextElement();
  c.maskValueChanged();
  }
}


/*----------------------------------------------------------------------
 * Name:  addMaskChangeListener
 *--------------------------------------------------------------------*/
/**
 * Adds an object that wishes to be informed of changes in the 
 * handler's message or trace mask configuration.  If the listener is
 * <code>null</code> or is already registered, this method does nothing.
 *
 * @param listener The mask change listener.
 */

public void addMaskChangeListener(RASIMaskChangeListener listener)
{
if (listener != null && !maskChangeListeners.contains(listener))
  {
  maskChangeListeners.addElement(listener);
  }
}


/*----------------------------------------------------------------------
 * Name:  removeMaskChangeListener
 *--------------------------------------------------------------------*/
/**
 * Removes an object from the list that wishes to be informed of changes
 * in the handler's message or trace mask configuration.  If the 
 * listener is <code>null</code> or is not registered, this method does
 * nothing.
 *
 * @param listener The mask change listener.
 */

public void removeMaskChangeListener(RASIMaskChangeListener listener)
{
if (listener != null && maskChangeListeners.contains(listener))
  maskChangeListeners.removeElement(listener);
}


/*----------------------------------------------------------------------
 * Name:  getMaskChangeListeners
 *--------------------------------------------------------------------*/
/**
 * Gets the set of objects that wish to be informed of changes in the
 * handler's message or trace mask configuration.
 *
 * @return An <code>Enumeration</code> of mask change listeners.  If
 *         no listeners are registered, the <code>Enumeration</code>
 *         is empty.
 */

public Enumeration getMaskChangeListeners()
{
return maskChangeListeners.elements();
}


/*----------------------------------------------------------------------
 * Name:  addFormatter
 *--------------------------------------------------------------------*/
/**
 * Registers a RAS event formatter with this handler.  The formatter
 * must be configured with the set of <code>RASIEvent</code> classes
 * which it supports before this method is called.
 * If the formatter is <code>null</code>, this method does nothing.
 * If the formatter supports an event class which is already registered
 * with this handler, this new formatter replaces the old one.
 *
 * @param formatter The event formatter.
 **/

public void addFormatter(RASIFormatter formatter)
{
if (formatter != null)
  {
  Enumeration e = formatter.getEventClasses();
  while (e.hasMoreElements())
    {
    String name = (String) e.nextElement();
    formatters.put(name, formatter);
    }
  }
}


/*----------------------------------------------------------------------
 * Name:  removeFormatter
 *--------------------------------------------------------------------*/
/**
 * Removes a RAS event formatter from this handler.  
 * If the formatter is <code>null</code> or is not registered, this 
 * method does nothing.
 *
 * @param formatter The event formatter.
 **/

public void removeFormatter(RASIFormatter formatter)
{
if (formatter != null)
  {
  Enumeration e = formatter.getEventClasses();
  while (e.hasMoreElements())
    {
    String name = (String) e.nextElement();
    formatters.remove(name);
    }
  }
}


/*----------------------------------------------------------------------
 * Name:  getFormatters
 *--------------------------------------------------------------------*/
/**
 * Gets the set of formatters associated with this handler.
 *
 * @return An <code>Enumeration</code> of registered formatters.  If no
 *         formatters have been registered, the <code>Enumeration</code>
 *         is empty.
 **/

public Enumeration getFormatters()
{
return formatters.elements();
}


/*----------------------------------------------------------------------
 * Name:  run
 *--------------------------------------------------------------------*/
/**
 * Processes any log event objects placed on the handler's queue by a 
 * logger.
 */

public void run()
{
RASEvent event;

while (!done)
  {
  synchronized(logQueue)
    {

    // Wait for something to be put on the queue or for a request
    // to stop the handler.

    while (logQueue.isEmpty() && !stopRequested)
      {
      try
        {
        logQueue.wait();
        }
      catch (InterruptedException ie) 
        {
        }
      }

    // If something is on the queue, remove it.

    event = null;
    if (!logQueue.isEmpty())
      {
      event = (RASEvent) logQueue.firstElement();
      logQueue.removeElementAt(0); 
      }

    logQueue.notify();
    }
      
  // We took a log event off the queue.  Write it.

  if (event != null)
    {
    writeEvent(event);
    }

  // The queue is empty.  If we have been asked to stop, we can now.

  else
    {
    synchronized(logQueue)
      {
      if (stopRequested)
        {
        done = true;
        logQueue.notify();
        }
      }
    }
  }
}


/*----------------------------------------------------------------------
 * Name:  stop
 *--------------------------------------------------------------------*/
/**
 * Stops the handler.  This method should be called when a handler is
 * no longer needed.  Be careful not to stop a handler prematurely if it
 * is shared by more than one logger.
 **/

public void stop()
{
synchronized(logQueue)
  {
  stopRequested = true;
  logQueue.notify();

  // Wait for the handler to empty the queue before returning.  This 
  // ensures that all log entries are written before an application
  // terminates.

  while (!done)
    {
    try
      {
      logQueue.wait();
      }
    catch (InterruptedException ie) 
      {
      }
    }
  }
}


/*----------------------------------------------------------------------
 * Name:  openDevice
 *--------------------------------------------------------------------*/
/**
 * Opens the device to which the handler will send its log entries.
 * If an error occurs during this process, the handler will write a
 * message to <code>System.err</code>.
 * <p>
 * The caller should ensure that this method is not called if the device
 * is already open or if cannot tolerate attempts to reopen it.
 **/

public abstract void openDevice();


/*----------------------------------------------------------------------
 * Name: logEvent
 *--------------------------------------------------------------------*/
/**
 * Passes a RAS event asynchronously from a logger to a handler.
 * The log entry is saved for processing by the handler.  The logger 
 * must compare the message type against the handler's message 
 * mask before calling this method.  If the log event is <code>null</code>,
 * this method does nothing.
 * 
 * @param event A RAS event.
 **/

// Add the message to the log queue and notify the handler that work
// awaits.

public void logEvent(RASIEvent event)
{
if (event != null)
  {
  synchronized(logQueue)
    {
    logQueue.addElement(event);
    logQueue.notify();
    }
  }
}


/*----------------------------------------------------------------------
 * Name: writeEvent
 *--------------------------------------------------------------------*/
/**
 * Passes a RAS event synchronously from a logger to a handler.
 * The handler processes the log entry immediately by calling the
 * <code>format</code> method of the configured formatter and then the
 * <code>println</code> method of the configured print writer.
 * The logger must compare the event type against the handler's message 
 * or trace mask before calling this method.  If the log event is 
 * <code>null</code>, this method does nothing.
 * 
 * @param event A RAS event.
 **/

public void writeEvent(RASIEvent event)
{
if (!deviceOpen)
  openDevice();

if (deviceOpen)
  {
  RASIFormatter formatter = findFormatter(event);
  if (formatter != null)
    {
    String msg = formatter.format(event);
    pWriter.println(msg);
    }
  }
}


/*----------------------------------------------------------------------
 * Name: findFormatter
 *--------------------------------------------------------------------*/
/**
 * Finds the formatter associated with a RAS event class.  If the class
 * is not registered, all superclasses will be checked for a registered
 * formatter.
 * 
 * @param event A RAS event.
 * 
 * @param formatter The associated formatter, or <code>null</code> if
 *                  the event class is not registered.
 **/

protected RASIFormatter findFormatter(RASIEvent event)
{
boolean found = false;
Class eventClass = event.getClass();
RASIFormatter formatter = null;

while (!found && eventClass != null)
  {
  String className = eventClass.getName();
  if (formatters.containsKey(className))
    {
    found = true;
    formatter = (RASIFormatter) formatters.get(className);
    }
  else
    {
    eventClass = eventClass.getSuperclass();
    }
  }

return formatter;
}

}
