/*----------------------------------------------------------------------
 * IBM Confidential
 * OCO Source Materials
 *
 * (C) Copyright IBM Corporation 1998.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *--------------------------------------------------------------------*/

package com.ibm.ras;

import java.util.*;

/**
 * <code>RASIHandler</code> defines the methods that must be implemented
 * to process <code>RASEvents</code> generated by a <code>RASLogger</code>.
 * Within the RAS Toolkit, <code>RASIHandler</code> is implemented by
 * the <code>RASHandler</code> class.
 */

public interface RASIHandler extends RASIObject
{

/*----------------------------------------------------------------------
 * Name:  getConfig
 *--------------------------------------------------------------------*/
/**
 * Gets the configuration of this object.
 *
 * @return A <code>Hashtable</code> containing the configuration.
 *         This object inserts the following key/value pairs into the
 *         the configuration:
 *         <pre>
 *         "messageMask"   The mask which selects the message types
 *                         to be processed.
 *         "traceMask"     The mask which selects the trace types
 *                         to be processed.
 *         </pre>
 *         All values are <code>Strings</code>.
 *         Extensions to this object may add additional keys.
 **/

public Hashtable getConfig();


/*----------------------------------------------------------------------
 * Name:  setConfig
 *--------------------------------------------------------------------*/
/**
 * Sets the configuration of this object.  This method is used by a 
 * <code>RASManager</code> to initialize a RAS object.  It should not 
 * be necessary for an application to use this method.
 *
 * @param ht  A <code>Hashtable</code> containing the configuration.
 *            This object searches for the following keys:
 *            <pre>
 *            "messageMask"   The mask which selects the message types
 *                            to be processed.
 *            "traceMask"     The mask which selects the trace types
 *                            to be processed.
 *            </pre>
 *            All values are <code>Strings</code>.
 *            If a key is not found, that configuration element is not
 *            updated.
 *            Extensions to this object may add additional keys.
 **/

public void setConfig(Hashtable ht);


/*----------------------------------------------------------------------
 * Name:  getMessageMask
 *--------------------------------------------------------------------*/
/**
 * Gets the mask which defines the set of message types that will be 
 * processed.  Possible values are defined by the logical OR of the  
 * <code>RASIMessageEvent.TYPE_XXXX</code> constants.
 *
 * @return The trace mask.
 */

public long getMessageMask();


/*----------------------------------------------------------------------
 * Name:  setMessageMask
 *--------------------------------------------------------------------*/
/**
 * Sets the mask which defines the set of message types that will be 
 * processed.  Possible values are defined by the logical OR of the  
 * <code>RASIMessageEvent.TYPE_XXXX</code> constants.
 * The mask value is not validated against these types.
 *
 * @param mask The message mask.
 */

public void setMessageMask(long mask);


/*----------------------------------------------------------------------
 * Name:  getTraceMask
 *--------------------------------------------------------------------*/
/**
 * Gets the mask which defines the set of trace types that will be 
 * processed.  Possible values are defined by the logical OR of the
 * <code>RASITraceEvent.TYPE_XXXX</code> constants.
 *
 * @return The trace mask.
 */

public long getTraceMask();


/*----------------------------------------------------------------------
 * Name:  setTraceMask
 *--------------------------------------------------------------------*/
/**
 * Sets the mask which defines the set of trace types that will be 
 * processed.  Possible values are defined by the
 * <code>RASITraceEvent.TYPE_XXXX</code> constants.
 * The mask value is not validated against these types.
 *
 * @param mask The trace mask.
 */

public void setTraceMask(long mask);


/*----------------------------------------------------------------------
 * Name:  addMaskChangeListener
 *--------------------------------------------------------------------*/
/**
 * Adds an object that wishes to be informed of changes in the 
 * handler's message or trace mask configuration.  If the listener is
 * <code>null</code> or is already registered, this method does nothing.
 *
 * @param listener The mask change listener.
 */

public void addMaskChangeListener(RASIMaskChangeListener listener);


/*----------------------------------------------------------------------
 * Name:  removeMaskChangeListener
 *--------------------------------------------------------------------*/
/**
 * Removes an object from the list that wishes to be informed of changes
 * in the handler's message or trace mask configuration.  If the 
 * listener is <code>null</code> or is not registered, this method does
 * nothing.
 *
 * @param listener The mask change listener.
 */

public void removeMaskChangeListener(RASIMaskChangeListener listener);


/*----------------------------------------------------------------------
 * Name:  getMaskChangeListeners
 *--------------------------------------------------------------------*/
/**
 * Gets the set of objects that wish to be informed of changes in the
 * handler's message or trace mask configuration.
 *
 * @return An <code>Enumeration</code> of mask change listeners.  If
 *         no listeners are registered, the <code>Enumeration</code>
 *         is empty.
 */

public Enumeration getMaskChangeListeners();


/*----------------------------------------------------------------------
 * Name:  addFormatter
 *--------------------------------------------------------------------*/
/**
 * Registers a RAS event formatter with this handler.  The formatter
 * must be configured with the set of <code>RASIEvent</code> classes
 * which it supports before this method is called.
 * If the formatter is <code>null</code>, this method does nothing.
 * If the formatter supports an event class which is already registered
 * with this handler, this new formatter replaces the old one.
 *
 * @param formatter The event formatter.
 **/

public void addFormatter(RASIFormatter formatter);


/*----------------------------------------------------------------------
 * Name:  removeFormatter
 *--------------------------------------------------------------------*/
/**
 * Removes a RAS event formatter from this handler.  
 * If the formatter is <code>null</code> or is not registered, this 
 * method does nothing.
 *
 * @param formatter The event formatter.
 **/

public void removeFormatter(RASIFormatter formatter);


/*----------------------------------------------------------------------
 * Name:  getFormatters
 *--------------------------------------------------------------------*/
/**
 * Gets the set of formatters associated with this handler.
 *
 * @return An <code>Enumeration</code> of registered formatters.  If no
 *         formatters have been registered, the <code>Enumeration</code>
 *         is empty.
 **/

public Enumeration getFormatters();


/*----------------------------------------------------------------------
 * Name:  stop
 *--------------------------------------------------------------------*/
/**
 * Stops the handler.  This method should be called when a handler is
 * no longer needed.  Be careful not to stop a handler prematurely if it
 * is shared by more than one logger.
 **/

public void stop();


/*----------------------------------------------------------------------
 * Name:  openDevice
 *--------------------------------------------------------------------*/
/**
 * Opens the device to which the handler will send its log entries.
 * If an error occurs during this process, the handler will write a
 * message to <code>System.err</code>.
 * <p>
 * The caller should ensure that this method is not called if the device
 * is already open or if cannot tolerate attempts to reopen it.
 **/

public void openDevice();


/*----------------------------------------------------------------------
 * Name: logEvent
 *--------------------------------------------------------------------*/
/**
 * Passes a RAS event asynchronously from a logger to a handler.
 * The log entry is saved for processing by the handler.  The logger 
 * must compare the message type against the handler's message 
 * mask before calling this method.  If the log event is <code>null</code>,
 * this method does nothing.
 * 
 * @param event A RAS event.
 **/

public void logEvent(RASIEvent event);


/*----------------------------------------------------------------------
 * Name: writeEvent
 *--------------------------------------------------------------------*/
/**
 * Passes a RAS event synchronously from a logger to a handler.
 * The handler processes the log entry immediately by calling the
 * <code>format</code> method of the configured formatter and then the
 * <code>println</code> method of the configured print writer.
 * The logger must compare the event type against the handler's message 
 * or trace mask before calling this method.  If the log event is 
 * <code>null</code>, this method does nothing.
 * 
 * @param event A RAS event.
 **/

public void writeEvent(RASIEvent event);
}
