/*----------------------------------------------------------------------
 * IBM Confidential
 * OCO Source Materials
 *
 * (C) Copyright IBM Corporation 1998.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *--------------------------------------------------------------------*/

package com.ibm.ras;

import java.util.*;

/**
 * <code>RASLogger</code> implements the <code>RASILogger</code>
 * interface and is the parent of all classes which create message and
 * trace data.  These classes provide the primary interface to the 
 * application programmer by supplying the methods for information
 * logging.  
 * <p>
 * Before you can use a <code>RASLogger</code>, you must associate with it
 * at least one <code>RASHandler</code> (implemented here by the 
 * <code>RASHandler</code> classes).  Use the <code>addHandler</code>
 * method to accomplish this.  A handler directs the log events to
 * a destination (such as a file, a console or a TCP socket).  It is 
 * possible to associate more than one handler with a logger to send the
 * data to multiple destinations, possibly formatted differently for 
 * each.  Multiple loggers may also be associated with a single handler.
 * <p>
 * <code>RASLogger</code> provides several levels of control over which log
 * events are processed.  At the highest level, the <code>public boolean</code>
 * variable <code>isLogging</code> is set <code>true</code> when the 
 * logger is "on" (logging data) and <code>false</code> when it is "off."
 * To improve performance, this variable may be tested to eliminate
 * unnecessary method calls.  For example:
 * <pre>
 * if (traceLogger.isLogging)
 *   traceLogger.trace(...);
 * </pre>
 * <p>
 * For finer control, each message or trace event is assigned a "type," which
 * defines the particular flavor of the object.  For example, a message
 * might be one of "informational," "warning," or "error."  These types
 * are defined by the classes which extend <code>RASEvent</code>.
 * <p>
 * In general, the type assigned to a log event may be the logical OR of any
 * of the defined types.  In some cases, this does not make much sense.  (A
 * message might be informational or a warning, but not both).  But
 * <code>RASEvent</code> extentions might define additional types for which 
 * this would make sense.  For example, an error might be "internal" (a 
 * failure detected within the application) or "external" (a failure detected 
 * outside the application).  In this case, one might define the log event type 
 * to be <code>TYPE_ERROR | TYPE_INTERNAL</code>.
 * <p>
 * Every logger and handler is assigned a "mask," which is the subset of
 * <code>RASEvent</code> types that it will process.  If any of the types
 * assigned to a log entry are contained in the mask, the logger or handler
 * will process the entry.  For example, a trace entry is assigned a type of
 * <code>TYPE_PUBLIC | TYPE_ERROR_EXC</code>.  The trace logger mask is set to
 * <code>TYPE_PUBLIC | TYPE_PRIVATE | TYPE_STATIC</code>.  Since 
 * <code>TYPE_PUBLIC</code> is in the log entry type and the logger's mask,
 * this entry will be processed.
 * <p>
 * The default message and trace masks are 
 * <code>RASIMessageEvent.DEFAULT_MESSAGE_MASK</code>
 * and <code>RASITraceEvent.DEFAULT_TRACE_MASK</code>, respectively.  These values 
 * can be changed through the <code>setMessageMask</code> and 
 * <code>setTraceMask</code> methods.
 * <p>
 * In practice, one is most likely to manipulate the mask of the logger <I>or</I>
 * the handlers, but not both -- although nothing prevents customizing
 * both masks.  Different effects can be achieved by manipulating the masks.
 * For example:
 * <ul>
 * <li>Logger A is set to process informational messages.  Logger B is set to
 * process error messages.  Both are connected to the same console handler, which
 * will accept any type of message.  Different components within the application 
 * can be set to provide varying levels of information.
 * <li>Loggers A and B are set to process any type of message.  Both loggers are
 * connected to the same console handler and file handler.  The console handler
 * is set to process only error messages, but the file handler will accept all
 * messages.  The effect is that critical errors go to the console, where they 
 * are seen immediately, and all messages go to the file, where they are archived.
 * </ul>
 * <p>
 * The <code>isLoggable</code> method compares the type associated with a message
 * or trace point with the mask of the logger and each configured handler.  
 * It returns <code>true</code> if the logger and at least one of the handlers
 * will process the log point.  Thus, the overhead of building the log entry is 
 * avoided if no object is configured to process it.
 * <pre>
 * if (traceLogger.isLoggable(TYPE_PUBLIC)
 *   traceLogger.trace(TYPE_PUBLIC,...);
 * </pre>
 * <p>
 * A logger can send data to its attached handlers either synchronously or
 * asynchronously.  In synchronous mode, a log entry is passed to a handler and
 * written to its destination immediately.  The application using the logger is,
 * in effect, "blocked" until this operation completes.  In asynchronous mode,
 * a log entry is passed to a handler and queued for processing at whatever rate
 * the handler is capable.  The logger returns to the calling application more
 * quickly in this mode.  Asynchronous operation is the default.  The mode can be
 * changed with the <code>setSynchronous</code> method and tested via
 * <code>isSynchronous</code>.
 * <p>
 * <code>RASLogger</code> has several optional fields which may be 
 * included in the message.  These fields should not vary among 
 * messages produced by a given <code>RASLogger</code>, so they are 
 * pecified through a <code>RASLogger</code> constructor 
 * or by the appropriate "set" and "get" methods of this class.  These fields
 * are:
 * <ul>
 * <li>A server associated with the creation of the messages.
 * <li>The client on whose behalf the messages are created.
 * </ul>
 * If not specified, each of these fields defaults to an empty string.
 * <p>
 * <code>RASLogger</code> is an abstract class, primarily because it lacks methods to 
 * send information to its associated handlers.  (The <code>message()</code> 
 * methods of <code>RASMessageLogger</code> and the <code>trace()</code> methods 
 * of <code>RASTraceLogger</code> provide this needed function.)
 *
 * @see RASILogger
 * @see RASMessageLogger
 * @see RASTraceLogger
 * @see RASHandler
 */

public abstract class RASLogger extends RASObject 
  implements RASILogger, RASIMaskChangeListener
{
private static final String S = RASCopyright.COPYRIGHT_98;

/**
 * A flag which is <code>true</code> when this object is logging data
 * and <code>false</code> otherwise.
 **/

public boolean isLogging = false;

/**
 * A flag which is <code>true</code> when this object is logging data
 * synchronously <code>false</code> otherwise.  Synchronous logging
 * means that a RAS event is passed directly to the handler's 
 * <code>write</code> method and is not queued for processing.
 **/

private boolean isSync = false;

/**
 * This mask is the logical OR of the message or trace mask of all of the 
 * attached handlers.  This value is then logically AND'ed with the logger's
 * message or trace mask.  The resulting value indicates the log types that
 * will be processed by the logger and at least one handler.
 **/

protected long isLoggableMask = 0;

/**
 * The name of the client.
 */

private String client = "";

/**
 * The name of the server.
 */

private String server = "";

/**
 * The handlers which are attached to this logger.
 **/

private Vector handlers = new Vector(1);


/*----------------------------------------------------------------------
 * Name:  RASLogger
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASLogger</code>.  The name and description of this
 * object are empty strings.
 */

public RASLogger() 
{
this("", "");
}


/*----------------------------------------------------------------------
 * Name:  RASLogger
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASLogger</code>.  The description of this object
 * is an empty string.
 *
 * @param name The name of this object.
 */

public RASLogger(String name) 
{
this(name, "");
}


/*----------------------------------------------------------------------
 * Name:  RASLogger
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASLogger</code>.
 *
 * @param name   The name of this object.
 * @param desc   The description of this object.
 */

public RASLogger(String name, String desc)
{
super(name, desc);
}


/*----------------------------------------------------------------------
 * Name:  RASLogger
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASLogger</code>.
 *
 * @param name   The name of this object.
 * @param desc   The description of this object.
 * @param server The server.
 * @param client The client.
 */

public RASLogger(String name, String desc,
                 String server, String client)
{
this(name, desc);
setServer(server);
setClient(client);
}


/*----------------------------------------------------------------------
 * Name:  getConfig
 *--------------------------------------------------------------------*/
/**
 * Gets the configuration of this object.
 *
 * @return A <code>Hashtable</code> containing the configuration.
 *         This object inserts the following key/value pairs into the
 *         configuration:
 *         <pre>
 *         "isLogging" "true" if the logger is logging data; 
 *                     otherwise, "false".
 *         "isSync"    "true" if the logger is logging
 *                     synchronously; otherwise, "false".
 *         "server"    The server.
 *         "client"    The client.
 *         </pre>
 *         All values are <code>Strings</code>.
 *         The parent and extensions of this object may add additional keys.
 **/

public Hashtable getConfig()
{
Hashtable ht = super.getConfig();

ht.put("isLogging", new Boolean(isLogging).toString());
ht.put("isSync", new Boolean(isSync).toString());
ht.put("server", getServer());
ht.put("client", getClient());

return ht;
}


/*----------------------------------------------------------------------
 * Name:  setConfig
 *--------------------------------------------------------------------*/
/**
 * Sets the configuration of this object.  This method is used by a 
 * <code>RASManager</code> to initialize a RAS object.  It should not 
 * be necessary for an application to use this method.
 *
 * @param ht  A <code>Hashtable</code> containing the configuration.
 *            This object searches for the following keys:
 *            <pre>
 *            "isLogging" "true" if the logger is logging data; 
 *                        otherwise, "false".
 *            "isSync"    "true" if the logger is logging
 *                        synchronously; otherwise, "false".
 *            "server"    The server.
 *            "client"    The client.
 *            </pre>
 *            All values are <code>Strings</code>.
 *            If a key is not found, that configuration element is not
 *            updated.
 *            The parent and extensions of this object may use additional keys.
 **/

public void setConfig(Hashtable ht)
{
super.setConfig(ht);
if (ht.containsKey("isLogging"))
  isLogging = new Boolean((String) ht.get("isLogging")).booleanValue();
if (ht.containsKey("isSync"))
  isSync = new Boolean((String) ht.get("isSync")).booleanValue();
if (ht.containsKey("server"))
  setServer((String) ht.get("server"));
if (ht.containsKey("client"))
  setClient((String) ht.get("client"));
}


/*----------------------------------------------------------------------
 * Name:  getClient
 *--------------------------------------------------------------------*/
/**
 * Gets the name of the client which is associated with this logger.
 * 
 * @return The client name, or an empty string ("") if the client has
 *         not been set.
 **/

public String getClient()
{
return client;
}


/*----------------------------------------------------------------------
 * Name:  setClient
 *--------------------------------------------------------------------*/
/**
 * Sets the name of the client which is associated with this logger.
 * If the name is <code>null</code>, the current name is not changed.
 * 
 * @param name The client name.
 **/

public void setClient(String name)
{
if (name != null)
  client = name;
}


/*----------------------------------------------------------------------
 * Name:  getServer
 *--------------------------------------------------------------------*/
/**
 * Gets the name of the server which is associated with this logger.
 * 
 * @return The server name, or an empty string ("") if the server has
 *         not been set.
 **/

public String getServer()
{
return server;
}


/*----------------------------------------------------------------------
 * Name:  setServer
 *--------------------------------------------------------------------*/
/**
 * Sets the name of the server which is associated with this logger.
 * If the name is <code>null</code>, the current name is not changed.
 * 
 * @param name  The server name.
 **/

public void setServer(String name)
{
if (name != null)
  server = name;
}


/*----------------------------------------------------------------------
 * Name:  addHandler
 *--------------------------------------------------------------------*/
/**
 * Registers a RAS handler with this logger.  More than one handler
 * may be associated with a logger to direct the RAS data to multiple
 * destinations.  If the handler is <code>null</code> or is already 
 * registered, this method does nothing.
 *
 * @param handler A RAS handler.
 **/

public void addHandler(RASIHandler handler)
{
if (handler != null && !handlers.contains(handler))
  {
  handlers.addElement(handler);
  handler.addMaskChangeListener(this);
  maskValueChanged();
  }
}


/*----------------------------------------------------------------------
 * Name:  removeHandler
 *--------------------------------------------------------------------*/
/**
 * Removes a RAS handler from this logger.  If the handler is 
 * <code>null</code> or is not registered, this method does nothing.
 * 
 * @param handler A RAS handler.
 **/

public void removeHandler(RASIHandler handler)
{
if (handler != null && handlers.contains(handler))
  {
  handlers.removeElement(handler);
  handler.removeMaskChangeListener(this);
  maskValueChanged();
  }
}


/*----------------------------------------------------------------------
 * Name:  getHandlers
 *--------------------------------------------------------------------*/
/**
 * Gets all of the handlers associated with this logger.
 *
 * @return An <code>Enumeration</code> of handlers.  If no handlers
 *         are registered, the <code>Enumeration</code> is empty.
 **/

public Enumeration getHandlers()
{
return handlers.elements();
}


/*----------------------------------------------------------------------
 * Name:  setSynchronous
 *--------------------------------------------------------------------*/
/**
 * Sets a flag that tells the logger whether to log data synchronously.
 * When logging synchronously, the logger will wait for the handlers
 * to write a log entry before returning to the caller.  Otherwise,
 * the log entry is passed to the handler and the logger returns.
 *
 * @param flag A <code>boolean</code> set <code>true</code> for 
 *             synchronous logging and <code>false</code> otherwise.
 **/

public void setSynchronous(boolean flag)
{
isSync = flag;
}


/*----------------------------------------------------------------------
 * Name:  isSynchronous
 *--------------------------------------------------------------------*/
/**
 * Determines if synchronous logging is in effect.
 * When logging synchronously, the logger will wait for the handlers
 * to write a log entry before returning to the caller.  Otherwise,
 * the log entry is passed to the handler and the logger returns.
 *
 * @return <code>true</code> for synchronous logging and 
 *         <code>false</code> otherwise.
 **/

public boolean isSynchronous()
{
return isSync;
}


/*----------------------------------------------------------------------
 * Name:  isLoggable
 *--------------------------------------------------------------------*/
/**
 * Determines if a log entry will be processed by the logger and any of
 * the handlers.  Wrapping a message or trace call with this method can
 * improve performance.  Log entries that will not be processed need not
 * even be built.  For example:
 * <pre>
 * if (isLoggable(RASTraceEvent.TYPE_PUBLIC)
 *   trace(RASTraceEvent.TYPE_PUBLIC...);
 * </pre>
 *
 * @param type The type of the log entry.  The set of possible values is
 *             defined by the <code>RASIMessageEvent.TYPE_XXXX</code>
 *             constants.
 *
 * @return <code>true</code> if the logger is enabled and at least one
 *         handler will process the log entry; <code>false</code>, 
 *         otherwise.
 */

public boolean isLoggable(long type)
{
boolean isLoggable = false; 

// If the result of the last mathematical operation is not zero, then
// the type is a subset of the logger & handler masks and the point 
// can be logged.

long invertedMask = ~isLoggableMask;
if (isLogging && ((type | invertedMask) ^ invertedMask) != 0)
  isLoggable = true;

return isLoggable;
}


/*----------------------------------------------------------------------
 * Name: maskValueChanged
 *--------------------------------------------------------------------*/
/**
 * Indicates that the value of the handler's message or trace mask has
 * changed.
 * <p>
 * This method is intended to improve the performance of the 
 * <code>RASLogger.isLoggable</code> method.  When notified of a change
 * in the value of a handler's mask, the logger can update its
 * internal data, which allows the logger to determine if a RAS event
 * will be logged.
 **/

public abstract void maskValueChanged();


/*----------------------------------------------------------------------
 * Name:  fireRASEvent
 *--------------------------------------------------------------------*/
/**
 * Sends a RAS event to all handlers which will process the event.
 *
 * @param event The event to be sent.
 */

protected void fireRASEvent(RASIEvent event)
{
Enumeration e = getHandlers();
if (e.hasMoreElements())
  {
  long invertedMask;
  long type = event.getType();
  RASIHandler h;

  while (e.hasMoreElements())
    {
    h = (RASIHandler) e.nextElement();
    if (event.isMessageEvent())
      invertedMask = ~h.getMessageMask();
    else
      invertedMask = ~h.getTraceMask();

    if (((type | invertedMask) ^ invertedMask) != 0)
      {
      if (isSynchronous())
        h.writeEvent(event);
      else
        h.logEvent(event);
      }
    }
  }

// No handlers are registered.

else
  {
  System.err.println(RASUtil.rasMsgs.getMessage("ERR_NO_HANDLERS", getName()));
  }
}


}
