/*----------------------------------------------------------------------
 * IBM Confidential
 * OCO Source Materials
 *
 * (C) Copyright IBM Corporation 1998.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *--------------------------------------------------------------------*/

package com.ibm.ras;

import java.util.*;
import java.text.*;

/**
 * The <code>RASMessageCatalog</code> class is used within the RAS system
 * to format messages according to the current locale (that is, in the
 * desired language).  The text associated with any RAS message is kept
 * in a <code>ResourceBundle</code>.  Each message text is associated with a 
 * "key," which is, essentially, a name for the message text.  
 * <code>RASMessageCatalog</code> formats the message text using a 
 * <code>java.text.MessageFormat</code> object.  One way to define the
 * messages is through a <code>PropertyResourceBundle</code> file, backed 
 * up by a <code>Properties</code> file.  Here is a sample:
 * <pre>
 * # {0} is the name of a file.                                            
 * # {1} is the detailed message created when the file creation error      
 * #     occurred                                                          
 *                                                                          
 * ERR_FILE_WRITER=Unable to create a FileWriter for {0}:  {1}
 *                                                                          
 * # {0} is a text string naming the missing message key.                  
 *                                                                          
 * ERR_MISSING_KEY=Message key {0} was not found in any searched catalog.
 *                                                                          
 * ERR_SOCKET=Unable to connect to the RAS Log Server.
 * </pre>
 * To create a message catalog, pass it the base name of the resource bundle.
 * For example, if your base file was
 * <code>com.ibm.ras.RASMsgs.properties</code> you would code:
 * <pre>
 * RASMessageCatalog msgCat = new RASMessageCatalog("com.ibm.ras.RASMsgs");
 * </pre>
 * Use the <code>RASMessageCatalog.getMessage()</code> methods to retrieve
 * the message text in the desired locale.  Java will look for the proper
 * translation of this file according to the supplied or default locale.
 * For example, it might first search for 
 * <code>RASMsgs_en_GB.properties</code>, then <code>RASMsgs_en.properties</code> 
 * and, finally <code>RASMsgs.properties</code>.
 * <p>
 * <I>It is not necessary for an application to create a 
 * <code>RASMessageCatalog</code>.  This class is used only by the
 * <code>RASMessageFormatter</code> to format a message text in the desired
 * language.</I>
 *
 * @see RASMessageFormatter
 */

public class RASMessageCatalog
{
private static final String S = RASCopyright.COPYRIGHT_98;

/**
 * The <code>Locale</code> to be used to format the messages.
 */

private Locale locale = Locale.getDefault();

/**
 * A formatter for the messages.
 */

private MessageFormat msgFormatter = new MessageFormat("");

/**
 * The <code>ResourceBundle</code> which holds the messages to be 
 * displayed.
 */

private ResourceBundle msgBundle;


/*----------------------------------------------------------------------
 * Name:  RASMessageCatalog
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASMessageCatalog</code> using the default locale.
 *
 * @param baseName The base name of the <code>ResourceBundle</code>
 *                 which contains the messages.
 *
 * @exception MissingResourceException
 *   This exception is thrown if the <code>ResourceBundle</code> 
 *   cannot be loaded.
 */

public RASMessageCatalog(String baseName) 
  throws MissingResourceException 
{
this(baseName, (Locale) null);
}  


/*----------------------------------------------------------------------
 * Name:  RASMessageCatalog
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASMessageCatalog</code> using the default locale.
 *
 * @param bundle The <code>ResourceBundle</code> which contains
 *               the messages.
 */

public RASMessageCatalog(ResourceBundle bundle)
{
this(bundle, (Locale) null);
}  


/*----------------------------------------------------------------------
 * Name:  RASMessageCatalog
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASMessageCatalog</code>.
 *
 * @param baseName The base name of the <code>ResourceBundle</code>
 *                 which contains the messages.
 * @param loc      The locale with which the messages should be 
 *                 displayed.
 *
 * @exception MissingResourceException
 *   This exception is thrown if the <code>ResourceBundle</code> 
 *   cannot be loaded.
 */

public RASMessageCatalog(String baseName, Locale loc) 
  throws MissingResourceException 
{
setLocale(loc);
msgBundle = ResourceBundle.getBundle(baseName, getLocale());
}  


/*----------------------------------------------------------------------
 * Name:  RASMessageCatalog
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASMessageCatalog</code>.
 *
 * @param bundle The <code>ResourceBundle</code> which contains
 *               the messages.
 * @param loc     The locale with which the messages should be 
 *               displayed.
 */

public RASMessageCatalog(ResourceBundle bundle, Locale loc)
{
setLocale(loc);
msgBundle = bundle;
}  


/*----------------------------------------------------------------------
 * Name:  getLocale
 *--------------------------------------------------------------------*/
/**
 * Gets the locale used by the <code>RASMessageCatalog</code>.
 * 
 * @return The message catalog locale.  If a locale has not be set, the
 *         default <code>Locale.getDefault</code> is returned.
 */

public Locale getLocale()
{
return locale;
}


/*----------------------------------------------------------------------
 * Name:  setLocale
 *--------------------------------------------------------------------*/
/**
 * Sets the locale to be used by the <code>RASMessageCatalog</code>.
 * 
 * @param loc   The new locale.  If this locale is <code>null</code>,
 *              the default locale is used.
 */

public void setLocale(Locale loc)
{
if (loc == null)
  locale = Locale.getDefault();
else
  locale = loc;
msgFormatter.setLocale(locale);
}


/*----------------------------------------------------------------------
 * Name:  getMessage
 *--------------------------------------------------------------------*/
/**
 * Gets a message with no inserts from the catalog.
 * 
 * @param key The key name of this message.
 *
 * @return The formatted message from the catalog.
 *
 * @exception MissingResourceException
 *   This exception is thrown if a message with the given key 
 *   cannot be found in the message catalog.
 */

public String getMessage(String key)
  throws MissingResourceException
{
return msgBundle.getString(key);
}


/*----------------------------------------------------------------------
 * Name:  getMessage
 *--------------------------------------------------------------------*/
/**
 * Gets a message with one insert from the catalog.
 * 
 * @param key     The key name of this message.
 * @param insert1 An element to be inserted into the message.
 *
 * @return The formatted message from the catalog.
 *
 * @exception MissingResourceException
 *   This exception is thrown if a message with the given key 
 *   cannot be found in the message catalog.
 */

public String getMessage(String key,
                         Object insert1)
  throws MissingResourceException
{
Object[] inserts = new Object[1];
inserts[0] = insert1;

return getMessage(key, inserts);
}


/*----------------------------------------------------------------------
 * Name:  getMessage
 *--------------------------------------------------------------------*/
/**
 * Gets a message with two inserts from the catalog.
 * 
 * @param key     The key name of this message.
 * @param insert1 An element to be inserted into the message.
 * @param insert2 An element to be inserted into the message.
 *
 * @return The formatted message from the catalog.
 *
 * @exception MissingResourceException
 *   This exception is thrown if a message with the given key 
 *   cannot be found in the message catalog.
 */

public String getMessage(String key,
                         Object insert1, 
                         Object insert2)
  throws MissingResourceException
{
Object[] inserts = new Object[2];
inserts[0] = insert1;
inserts[1] = insert2;

return getMessage(key, inserts);
}


/*----------------------------------------------------------------------
 * Name:  getMessage
 *--------------------------------------------------------------------*/
/**
 * Gets a message with an array of inserts from the catalog.
 * 
 * @param key     The key name of this message.
 * @param inserts An array of elements to be inserted into the message.
 *
 * @return The formatted message from the catalog.
 *
 * @exception MissingResourceException
 *   This exception is thrown if a message with the given key 
 *   cannot be found in the message catalog.
 */

public String getMessage(String key,
                         Object[] inserts)
  throws MissingResourceException
{
String msg = msgBundle.getString(key);
if (inserts != null)
  {  
  msgFormatter.applyPattern(msg);

  StringBuffer msgBfr = new StringBuffer();
  msgBfr = msgFormatter.format(inserts, msgBfr, (FieldPosition) null);
  msg = msgBfr.toString();
  }

return msg;
}

}
