/*----------------------------------------------------------------------
 * IBM Confidential
 * OCO Source Materials
 *
 * (C) Copyright IBM Corporation 1998.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *--------------------------------------------------------------------*/

package com.ibm.ras;

import java.io.*;
import java.util.*;

/**
 * A <code>RASMessageEvent</code> encapsulates all of the information generated
 * by a <code>RASMessageLogger</code>.  <I>It is not necessary
 * to create a <code>RASMessageEvent</code> directly, as the message logger
 * handles this task.  However, the methods of this class are public, should
 * it be necessary to extend <code>RASMessageEvent</code>.</I>
 * <p>
 * A <code>RASMessageEvent</code> contains the following components:
 * <ul>
 * <li>A flag which identifies this event as a message event.
 * <li>A time stamp, generated when the message is created.
 * <li>A message type, such as informational, warning or error.
 * <li>The message text.  If a message file is specified, this text is a 
 * "key" which selects the message text from the message file.
 * (Separating the text from the classes which use it allows the text to 
 * be translated into different national languages.)
 * <li>A set of <code>Strings</code> which can be displayed with the
 * message text.  These parameters are required only if the message expects them.
 * <li>A message file, which can be used to translate the message key
 * to a text string.  If this value is null, the text is displayed directly.
 * (That is, it is not a key to be used with a message file.)
 * <li>A set of additional attributes that may be part of the event data.  
 * The following attributes may be set.  Unless otherwise noted, all 
 * <code>Objects</code> associated with the attributes are <code>Strings</code>.
 * <pre>
 * "className"    The name of the Java class which generated the 
 *                message.
 * "methodName"   The name of the Java method which generated the
 *                message.
 * "organization" The name of the organization which owns the 
 *                application that created the message.
 * "product"      The name of the product that created the
 *                message.
 * "component"    The name of the component within the product 
 *                that created the message.
 * "server"       A server associated with the creation of the
 *                message.
 * "client"       The client on whose behalf the message was created.
 * "exception"    An <code>Exception</code>.
 * </pre>
 * </ul>
 * <p>
 * The <code>RASMessageEvent</code> type is defined by the set of 
 * <code>RASIMessageEvent.TYPE_XXXX</code> constants.
 * Values 0x0000L up to, but not including, 0x01000L are reserved.  
 * Extensions to the <code>RASMessageEvent</code> class may use values outside
 * this range for additional types.  The constant <code>DEFAULT_MESSAGE_MASK</code> 
 * contains the message types that the handlers log by default.  It includes all
 * message types.
 *
 * @see RASEvent
 * @see RASMessageLogger
 * @see RASHandler
 */

public class RASMessageEvent extends RASEvent implements RASIMessageEvent
{
private static final String S = RASCopyright.COPYRIGHT_98;

/**
 * @serial
 *   The base name of the message file (<code>Strings</code>) used to 
 *   convert the messages key to a text string.
 */

private String messageFile = null;


/*----------------------------------------------------------------------
 * Name:  RASMessageEvent
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASMessageEvent</code>.
 *
 * @param type        The message type.
 * @param text        The message text, or a message key if the message file
 *                    is not <code>null</code>.
 * @param parms       An array of <code>Strings</code> to be displayed 
 *                    with the message.
 * @param messageFile The base message file name.
 */

protected RASMessageEvent(long type,
                          String text, String[] parms, String messageFile)
{
super();
setType(type);
setText(text);
setParameters(parms);
setMessageFile(messageFile);
setMessageEvent(true);
}


/*----------------------------------------------------------------------
 * Name:  RASMessageEvent
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASMessageEvent</code>.  The caller is expected to provide
 * a message type, key and, if needed, any <code>Object</code> parameters for
 * the message text.  If any of the other parameters are not needed, they 
 * must be set to an empty string ("") or <code>null</code>.
 *
 * @param type        The message type.
 * @param text        The message text, or a message key if the message file
 *                    is not <code>null</code>.
 * @param parms       An array of <code>Strings</code> to be displayed 
 *                    with the message.
 * @param messageFile The base message file name.
 * @param className   The class.
 * @param methodName  The method.
 * @param org         The organization.
 * @param prod        The product.
 * @param comp        The component.
 * @param server      The server.
 * @param client      The client.
 * @param exception   An exception.
 */
                 
public RASMessageEvent(long type, 
                       String text, String[] parms, String messageFile,
                       String className, String methodName, 
                       String org, String prod, String comp,
                       String server, String client,
                       Exception exception)
{
this(type, text, parms, messageFile);

if (className != null)
  setAttribute("className", className);

if (methodName != null)
  setAttribute("methodName", methodName);

if (org != null)
  setAttribute("organization", org);

if (prod != null)
  setAttribute("product", prod);

if (comp != null)
  setAttribute("component", comp);

if (server != null)
  setAttribute("server", server);

if (client != null)
  setAttribute("client", client);

if (exception != null)
  setAttribute("exception", exception);

}


/*----------------------------------------------------------------------
 * Name:  getMessageFile
 *--------------------------------------------------------------------*/
/**
 * Gets the base message file name used to format a message.
 * 
 * @return A message file name.
 **/

public String getMessageFile()
{
return messageFile;
}


/*----------------------------------------------------------------------
 * Name:  setMessageFile
 *--------------------------------------------------------------------*/
/**
 * Sets the base message file name used to format a message.
 * 
 * @param messageFile The message file names.
 **/

public void setMessageFile(String messageFile)
{
this.messageFile = messageFile;
}


/*----------------------------------------------------------------------
 * Name:  getSupportedTypes
 *--------------------------------------------------------------------*/
/**
 * Gets the complete set of event types supported by this class.
 * <p>
 * This method can be used by a graphical log manager to display the 
 * set of types and allow a user to select those to be monitored.  Every
 * extending class should override this method to add the types it supports
 * to the <code>Hashtable</code> returned by its parent.
 *
 * @return A <code>Hashtable</code> containing the values of all of the
 *         event types.  The names of the types are used as keys and
 *         are returned in the current locale.  The type values,
 *         normally kept as <code>longs</code>, are returned as
 *         <code>Longs</code> because of the <code>Hashtable</code>
 *         requirement that all elements be <code>Objects</code>.
 */

public Hashtable getSupportedTypes()
{
Hashtable ht = super.getSupportedTypes();

ht.put(RASUtil.rasMsgs.getMessage("TYPE_INFO"), new Long(TYPE_INFO));
ht.put(RASUtil.rasMsgs.getMessage("TYPE_WARN"), new Long(TYPE_WARN));
ht.put(RASUtil.rasMsgs.getMessage("TYPE_ERR"), new Long(TYPE_ERR));

return ht;
}


/*----------------------------------------------------------------------
 * Name: maskLongValue
 *--------------------------------------------------------------------*/
/**
 * Converts a <code>String</code> representation of the message mask
 * to its long equivalent.  The string must have the following format:
 * <pre>
 * String types = "TYPE_INFO TYPE_WARN";
 * </pre
 * The value <code>DEFAULT_MESSAGE_MASK</code> may also be included in the string.
 * (These values are the names of the <code>RASIMessageEvent.TYPE_XXXX</code>
 * constants.)
 * Every extending class should override this method to add the types it
 * supports to the value returned by its parent.  
 * <p>
 * Unknown values in the <code>String</code> are ignored.
 *
 * @param types The blank-delimited <code>String</code> to be converted.
 *
 * @return The long equivalent of the <code>String</code>, formed by
 *         the logical OR of each element in the <code>String</code>.
 */

public static long maskLongValue(String types)
{
long result = 0;
StringTokenizer st = new StringTokenizer(types);

while (st.hasMoreElements())
  {
  String type = st.nextToken();
  if (type.equals("TYPE_INFO"))
    result |= TYPE_INFO;
  else if (type.equals("TYPE_WARN"))
    result |= TYPE_WARN;
  else if (type.equals("TYPE_ERR"))
    result |= TYPE_ERR;
  else if (type.equals("DEFAULT_MESSAGE_MASK"))
    result |= DEFAULT_MESSAGE_MASK;
  }

return result;
}


/*----------------------------------------------------------------------
 * Name: maskToString
 *--------------------------------------------------------------------*/
/**
 * Converts a long containing the logical OR of one or more
 * <code>RASIMessageEvent.TYPE_XXXX</code> constants to its <code>String</code>
 * equivalent.  The string will have the following format:
 * <pre>
 * String types = "TYPE_INFO TYPE_WARN";
 * </pre>
 * Every extending class should override this method to add the types it
 * supports to the value returned by its parent.
 * <p>
 * Unknown values in the <code>long</code> are ignored.
 *
 * @param types The message mask.
 *
 * @return The <code>String</code> equivalent of the message mask.
 */

public static String maskToString(long types)
{
StringBuffer bfr = new StringBuffer();

if ((types & TYPE_INFO) != 0)
  bfr.append("TYPE_INFO ");
if ((types & TYPE_WARN) != 0)
  bfr.append("TYPE_WARN ");
if ((types & TYPE_ERR) != 0)
  bfr.append("TYPE_ERR ");

return bfr.toString().trim();
}

}
