/*----------------------------------------------------------------------
 * IBM Confidential
 * OCO Source Materials
 *
 * (C) Copyright IBM Corporation 1998.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *--------------------------------------------------------------------*/

package com.ibm.ras;

import java.io.*;
import java.util.*;

/**
 * A <code>RASMessageFormatter</code> formats a <code>RASMessageEvent</code>
 * for display.  The formatted <code>String</code> includes the following
 * items:
 * <ul>
 * <li>The date at which the event was created.
 * <li>The time at which the event was created.
 * <li>The name of the Java class which generated the message.
 * <li>The name of the Java method which generated the message.
 * <li>The name of the organization which owns the application that created
 * the message.
 * <li>The name of the product that created the message.
 * <li>The name of the component within the product that created the message.
 * <li>A server associated with the creation of the message.
 * <li>The client on whose behalf the message was created.
 * <li>The event text.
 * </ul>
 *
 * @see RASMessageEvent
 * @see RASHandler
 */

public class RASMessageFormatter extends RASFormatter
{
private static final String S = RASCopyright.COPYRIGHT_98;

/**
 * A set of attribute names used to format the event.  They are listed
 * in the same order that they will be formatted.
 */

private static String[] attribs = 
  {
  "className",
  "methodName",
  "organization",
  "product",
  "component",
  "server",
  "client"
  };


/*----------------------------------------------------------------------
 * Name:  RASMessageFormatter
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASMessageFormatter</code>.
 */

public RASMessageFormatter()
{

// Add the default RAS event classes which this formatter can process.

addEventClass("com.ibm.ras.RASMessageEvent");
}


/*----------------------------------------------------------------------
 * Name:  format
 *--------------------------------------------------------------------*/
/**
 * Formats the RAS event.
 * <p>
 * If a message key is used and that key is not found in any message file,
 * the message text becomes an error message indicating that the key was
 * not found.
 * 
 * @param event The event to format.
 * 
 * @return The formatted event.
 **/

public String format(RASIEvent event)
{
String attrib;
String sep = getSeparator();
StringBuffer bfr = new StringBuffer(256);

RASMessageEvent mevent = (RASMessageEvent) event;

long timeStamp = mevent.getTimeStamp();
bfr.append(getDate(timeStamp) + sep);
bfr.append(getTime(timeStamp) + sep);

// Append the String attributes.

for (int j = 0; j < attribs.length; j++)
  {
  attrib = (String) mevent.getAttribute(attribs[j]);
  if (attrib != null)
    bfr.append(attrib + sep);
  }

// For an exception, append the stack trace.

String text = mevent.getText();
String[] parms = mevent.getParameters();
String messageFile = mevent.getMessageFile();
Exception e = (Exception) mevent.getAttribute("exception");

if (e != null)
  {
  StringWriter sw = new StringWriter();
  e.printStackTrace(new PrintWriter(sw));
  bfr.append(sw.toString());
  }

// If the key is null, we are formatting a text message only (no
// message keys).  Add the message text and any parameters.

else if (messageFile == null)
  {  
  bfr.append(text);
  if (parms != null)
    {
    for (int j = 0; j < parms.length; j++)
      {    
      bfr.append(", p" + (j+1) + "=" + parms[j]);
      }
    }
  }

// Search the message file (if any) for the message key.  If the key is not
// found a default message is used instead.

else
  {
  RASMessageCatalog cat = null;
  String msg = null;
  try
    {
    cat = new RASMessageCatalog(messageFile);
    msg = cat.getMessage(text, parms);
    }
  catch (MissingResourceException mrE)
    {
    msg = RASUtil.rasMsgs.getMessage("ERR_MISSING_KEY", text);
    }
  bfr.append(msg);
  }

return bfr.toString();
}

}
