/*----------------------------------------------------------------------
 * IBM Confidential
 * OCO Source Materials
 *
 * (C) Copyright IBM Corporation 1998.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *--------------------------------------------------------------------*/

package com.ibm.ras;

import java.text.DateFormat;
import java.util.*;

/**
 * The <code>RASMessageLogger</code> is one of two <code>RASLogger</code>
 * sub-classes provided in this package.  (The other is 
 * <code>RASTraceLogger</code>.)  The purpose of message logging is to 
 * provide information to the users of an application.  (Contrast this 
 * with tracing, which is intended for software developers and service 
 * teams.)  As a general guideline, a message should be logged only if 
 * the information it contains is of value to the user. 
 * <p>
 * By default, message logging is on when a <code>RASMessageLogger</code> 
 * is created.
 * <p>
 * Normally, the text for a set of messages is stored in one or more message
 * files.  Text strings are accessed by a "key," which is a name for the 
 * message text. This mechanism allows the message text to be separated 
 * from the program that uses it, making translation into different
 * national languages easier.
 * <p>
 * A message logger, therefore, must be told which message files to 
 * search.  Two mechanisms are available to accomplish this:
 * <ul>
 * <li>When the message is logged, name the message file containing the 
 * message key.  The <code>msg</code> methods in this class do this.
 * <li>Use the <code>setMessageFile</code> method to register one message file
 * with the message logger.  If more than one message file is required,
 * the <code>msg</code> methods can be used with the additional message files.
 * The <code>message</code> methods in this class assume that one
 * message file has been registered.
 * </ul>
 * <p>
 * The message key can be any desired string.  For example, a key 
 * <code>HUNGRY</code> might select the message text, "Are you ready
 * to eat?"  This key may be displayed as part of a formatted message 
 * and can be used more formally to identify messages.  For example, instead
 * of <code>HUNGRY</code>, the key might be <code>RAS0001I</code>.  In this
 * scheme the first three characters identify the product or, perhaps, 
 * component -- the RAS Toolkit, in this case.  The next four  numbers select
 * a RAS message.  The last character is set to <code>I</code> for 
 * informational messages, <code>W</code> for warning messages and <code>E</code>
 * for error messages.
 * <p>
 * If a message file is not specified, or the message key is not found in the
 * message file, an error message is formatted instead.
 * <p>
 * Run-time substitution of variables into the message text is possible.
 * The <code>message, msg</code> and <code>textMessage</code> methods accept
 * a number of <code>Objects</code> whose <code>String</code> representations
 * may be inserted into the message text according to the rules of the 
 * <code>java.text.MessageFormat</code> class.  The only restriction is that
 * the <code>Objects</code> are converted to <code>Strings</code> before they
 * are passed to the MessageFormat class, so that the MessageFormat style options 
 * (such as <code>{1,time}</code>) may not be used.
 * <p>
 * If a message file is not specified or does not exist, or the message key is not 
 * found in the message file, an error message is formatted instead.
 * <p>
 * <code>RASMessageLogger</code> provides the ability to suppress individual
 * messages.  This feature might be used if a program were logging the same 
 * message again and again and the system administrator did not want to see any 
 * more of them.  The <code>addSuppressedKey</code> method is used to stop the 
 * logging of specific messages.
 * <p>
 * The <code>message</code> and <code>msg</code> methods all include the name of
 * the class and method which generated the message.  When logging messages in 
 * non-static methods, you may use the methods that take "this" (the object
 * being traced) as the second parameter.  This is a convenience to the  
 * programmer, as the class name can be derived from any <code>Object</code>.
 * For static methods, an object does not exist.  Use the methods which take
 * "<code>String className</code>" as the second parameter.  Of course,
 * this latter approach may be used in all methods, if desired.  It will
 * also result in better performance (significant if many messages are logged), as
 * the RAS system will not have to translate an <code>Object</code> to a class name.
 * <p>
 * <code>RASMessageLogger</code> provides <I>minimal</I> support for the 
 * logging of plain text messages which do not use message catalogs.  
 * The <code>textMessage</code> methods are available for this.  They are 
 * provided as a convenience for an application developer who might want to 
 * log a quick debug message that will not make it into the final product.  
 * The <code>textMessage</code> method takes an arbitrary text string in
 * place of a message key.  <code>Objects</code> may be included with the 
 * message, but they are simply displayed with the message text (according to 
 * their <code>toString</code> representation), not substituted into the 
 * message text.
 * <p>
 * <code>RASMessageLogger</code> has several optional fields which may be 
 * included in the message.  These fields should not vary among 
 * messages produced by a given <code>RASMessageLogger</code>, so they are 
 * not included in the <code>message</code> methods of this class.  They can, 
 * however, be specified through a <code>RASMessageLogger</code> constructor 
 * or by the appropriate "set" and "get" methods of this class.  These fields
 * are:
 * <ul>
 * <li>The name of the organization which owns the application that creates
 * these messages.
 * <li>The name of the product that creates these messages.
 * <li>The name of the component within the product that creates these messages.
 * <li>A server associated with the creation of the messages.
 * <li>The client on whose behalf the messages are created.
 * </ul>
 * If not specified, each of these fields defaults to an empty string.
 * <p>
 * <code>RASMessageLogger</code> supports three types of messages:  
 * information, warning and error.  For warning and error messages, you should
 * provide details telling the customer how to rectify the situation.  These 
 * types are formally defined in the <code>RASIMessageEvent</code> class.
 * The <code>message, msg</code> and <code>textMessage</code> methods do not validate 
 * the type assigned to a message, so an error could result in the failure 
 * to log the message.
 *
 * @see RASIMessageLogger
 * @see RASLogger
 * @see RASMessageEvent
 * @see RASHandler
 */

public class RASMessageLogger extends RASLogger implements RASIMessageLogger
{
private static final String S = RASCopyright.COPYRIGHT_98;

/**
 * The mask which determines the types of messages that will
 * be processed.
 */

private long messageMask = RASIMessageEvent.DEFAULT_MESSAGE_MASK;

/**
 * The name of the component.
 */

private String component = "";

/**
 * The name of the organization.
 */

private String organization = "";

/**
 * The name of the product.
 */

private String product = "";

/**
 * A properties file name (containing message texts) registered
 * with this message logger.
 */

private String messageFile = null;

/**
 * Message keys which are not to be logged.
 */

private Vector suppressedKeys = new Vector();


/*----------------------------------------------------------------------
 * Name:  RASMessageLogger
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASMessageLogger</code>.  The name and
 * description of this object are empty strings.
 */

public RASMessageLogger() 
{
this("", "");
}


/*----------------------------------------------------------------------
 * Name:  RASMessageLogger
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASMessageLogger</code>.  The description
 * of this object is an empty string.
 *
 * @param name The name of this object.
 */

public RASMessageLogger(String name) 
{
this(name, "");
}


/*----------------------------------------------------------------------
 * Name:  RASMessageLogger
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASMessageLogger</code>.
 *
 * @param name The name of this object.
 * @param desc The description of this object.
 */

public RASMessageLogger(String name, String desc) 
{
super(name, desc);
isLogging = true;
}


/*----------------------------------------------------------------------
 * Name:  RASMessageLogger
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASMessageLogger</code>.  (See the description in the
 * prologue for each of these elements.)
 *
 * @param name   The name of this logger.
 * @param desc   The description of this logger.
 * @param org    The organization.
 * @param prod   The product.
 * @param comp   The component.
 * @param server The server.
 * @param client The client.
 */

public RASMessageLogger(String name, String desc,
                        String org, String prod, String comp, 
                        String server, String client)
{
super(name, desc, server, client);
isLogging = true;
setOrganization(org);
setProduct(prod);
setComponent(comp);
}


/*----------------------------------------------------------------------
 * Name:  getConfig
 *--------------------------------------------------------------------*/
/**
 * Gets the configuration of this object.
 *
 * @return A <code>Hashtable</code> containing the configuration.
 *         This object inserts the following key/value pairs into the
 *         configuration:
 *         <pre>
 *         "organization"   The organization.
 *         "product"        The product.
 *         "component"      The component.
 *         "messageMask"    The mask which selects the message types to
 *                          be processed.
 *         "messageFile"    A message file name, used to translate message
 *                          keys.
 *         "suppressedKeys" A blank-delimited set of message keys which
 *                          are not to be processed.
 *         </pre>
 *         All values are <code>Strings</code>.
 *         The parent and extensions of this object may add additional keys.
 **/

public Hashtable getConfig()
{
Hashtable ht = super.getConfig();

ht.put("organization", getOrganization());
ht.put("product", getProduct());
ht.put("component", getComponent());
ht.put("messageMask", RASMessageEvent.maskToString(getMessageMask()));

String file = getMessageFile();
if (file != null)
  ht.put("messageFile", file);

ht.put("suppressedKeys", RASUtil.vectorToString(suppressedKeys));

return ht;
}


/*----------------------------------------------------------------------
 * Name:  setConfig
 *--------------------------------------------------------------------*/
/**
 * Sets the configuration of this object.  This method is used by a 
 * <code>RASManager</code> to initialize a RAS object.  It should not 
 * be necessary for an application to use this method.
 *
 * @param ht  A <code>Hashtable</code> containing the configuration.
 *            This object searches for the following keys:
 *            <pre>
 *            "organization"   The organization.
 *            "product"        The product.
 *            "component"      The component.
 *            "messageMask"    The mask which selects the message types to
 *                             be processed.
 *            "messageFile"    A message file name, used to translate message
 *                             keys.
 *            "suppressedKeys" A blank-delimited set of message keys which
 *                             are not to be processed.
 *            </pre>
 *            All values are <code>Strings</code>.
 *            If a key is not found, that configuration element is not
 *            updated.
 *            The parent and extensions of this object may use additional keys.
 **/

public void setConfig(Hashtable ht)
{
super.setConfig(ht);

if (ht.containsKey("organization"))
  setOrganization((String) ht.get("organization"));
if (ht.containsKey("product"))
  setProduct((String) ht.get("product"));
if (ht.containsKey("component"))
  setComponent((String) ht.get("component"));
if (ht.containsKey("messageMask"))
  setMessageMask(RASMessageEvent.maskLongValue((String) ht.get("messageMask")));
if (ht.containsKey("messageFile"))
  messageFile = (String) ht.get("messageFile");
if (ht.containsKey("suppressedKeys"))
  suppressedKeys = RASUtil.stringToVector((String) ht.get("suppressedKeys"));
}


/*----------------------------------------------------------------------
 * Name:  getMessageMask
 *--------------------------------------------------------------------*/
/**
 * Gets the mask which defines the set of message types that will be 
 * processed.  Possible values are defined by the logical OR of the  
 * <code>RASIMessageEvent.TYPE_XXXX</code> constants.
 *
 * @return The trace mask.
 */

public long getMessageMask() 
{
return messageMask;
}


/*----------------------------------------------------------------------
 * Name:  setMessageMask
 *--------------------------------------------------------------------*/
/**
 * Sets the mask which defines the set of message types that will be 
 * processed.  Possible values are defined by the 
 * <code>RASIMessageEvent.TYPE_XXXX</code> constants.
 * The mask value is not validated against these types.
 *
 * @param mask The message mask.
 */

public void setMessageMask(long mask) 
{
messageMask = mask;
maskValueChanged();
}


/*----------------------------------------------------------------------
 * Name:  getOrganization
 *--------------------------------------------------------------------*/
/**
 * Gets the name of the organization which wrote the application that
 * is using this logger.
 * 
 * @return The organization name, or an empty string ("") if the
 *         organization has not been set.
 **/

public String getOrganization()
{
return organization;
}


/*----------------------------------------------------------------------
 * Name:  setOrganization
 *--------------------------------------------------------------------*/
/**
 * Sets the name of the organization which wrote the application that
 * is using this logger.  If the name is <code>null</code>, the current
 * name is not changed.
 * 
 * @param name The organization name.
 **/

public void setOrganization(String name)
{
if (name != null)
  organization = name;
}


/*----------------------------------------------------------------------
 * Name:  getProduct
 *--------------------------------------------------------------------*/
/**
 * Gets the name of the product that is using this logger.
 * 
 * @return The product name, or an empty string ("") if the
 *         product has not been set.
 **/

public String getProduct()
{
return product;
}


/*----------------------------------------------------------------------
 * Name:  setProduct
 *--------------------------------------------------------------------*/
/**
 * Sets the name of the product that is using this logger.  If the name
 * is <code>null</code>, the current name is not changed.
 * 
 * @param name The product name.
 **/

public void setProduct(String name)
{
if (name != null)
  product = name;
}


/*----------------------------------------------------------------------
 * Name:  getComponent
 *--------------------------------------------------------------------*/
/**
 * Gets the name of the component that is using this logger.
 * 
 * @return The component name, or an empty string ("") if the
 *         component has not been set.
 **/

public String getComponent()
{
return component;
}


/*----------------------------------------------------------------------
 * Name:  setComponent
 *--------------------------------------------------------------------*/
/**
 * Sets the name of the component that is using this logger.  If the name
 * is <code>null</code>, the current name is not changed.
 * 
 * @param name The component name.
 **/

public void setComponent(String name)
{
if (name != null)
  component = name;
}


/*----------------------------------------------------------------------
 * Name:  getMessageFile
 *--------------------------------------------------------------------*/
/**
 * Gets the message file associated with this logger.
 *
 * @return The message file name, or <code>null</code> if a message file
 *         has not been added to the logger. 
 **/

public String getMessageFile()
{
return messageFile;
}


/*----------------------------------------------------------------------
 * Name:  setMessageFile
 *--------------------------------------------------------------------*/
/**
 * Sets the message file associated with this logger.  If the file
 * is <code>null</code>, the current message file is not changed.
 * If the message file does not exist, a message indicating this will
 * be displayed in place of the formatted message.
 *
 * @param file The message file name.
 **/

public void setMessageFile(String file)
{
if (file != null)
  messageFile = file;
}


/*----------------------------------------------------------------------
 * Name:  addSuppressedKey
 *--------------------------------------------------------------------*/
/**
 * Adds a message key to a set of keys which should not be logged.
 * More than one message key may be in this set.  If the key is 
 * <code>null</code> or is already registered, this method does nothing.
 *
 * @param key The message key which is not to be logged.
 **/

public void addSuppressedKey(String key)
{
if (key != null && !suppressedKeys.contains(key))
  suppressedKeys.addElement(key);
}


/*----------------------------------------------------------------------
 * Name:  removeSuppressedKey
 *--------------------------------------------------------------------*/
/**
 * Removes a message key from the set of keys which should not be logged.
 * If the key is <code>null</code> or is not registered, this method
 * does nothing.
 *
 * @param key The message key to be removed.
 **/

public void removeSuppressedKey(String key)
{
if (key != null && suppressedKeys.contains(key))
  suppressedKeys.removeElement(key);
}


/*----------------------------------------------------------------------
 * Name:  getSuppressedKeys
 *--------------------------------------------------------------------*/
/**
 * Gets all of the suppressed message keys associated with this logger.
 *
 * @return A <code>Vector</code> of message keys.  If no suppressed keys
 *         are registered, the <code>Vector</code> is empty.
 **/

public Vector getSuppressedKeys()
{
return suppressedKeys;
}


/*----------------------------------------------------------------------
 * Name:  message
 *--------------------------------------------------------------------*/
/**
 * Logs a message, by key, with no parameters.
 * This call can be used in a non-static method and assumes that at
 * least one message file is registered with the logger.
 *
 * @param type       The type of the message.
 * @param obj        The logging object ("this").
 * @param methodName The name of the logging method.
 * @param key        The message key.
 */

public void message(long type, Object obj, String methodName, String key)
{
String className = obj.getClass().getName();
msg(type, className, methodName, key, (String) null, (Object[]) null);
}


/*----------------------------------------------------------------------
 * Name:  message
 *--------------------------------------------------------------------*/
/**
 * Logs a message, by key, with one parameter.
 * This call can be used in a non-static method and assumes that at
 * least one message file is registered with the logger.
 *
 * @param type       The type of the message.
 * @param obj        The logging object ("this").
 * @param methodName The name of the logging method.
 * @param key        The message key.
 * @param parm1      An element to be inserted into the message.
 */

public void message(long type, Object obj, String methodName, String key,
                    Object parm1)
{
Object[] parms = {parm1};
String className = obj.getClass().getName();
msg(type, className, methodName, key, (String) null, parms);
}


/*----------------------------------------------------------------------
 * Name:  message
 *--------------------------------------------------------------------*/
/**
 * Logs a message, by key, with two parameters.
 * This call can be used in a non-static method and assumes that at
 * least one message file is registered with the logger.
 *
 * @param type       The type of the message.
 * @param obj        The logging object ("this").
 * @param methodName The name of the logging method.
 * @param key        The message key.
 * @param parm1      An element to be inserted into the message.
 * @param parm2      An element to be inserted into the message.
 */

public void message(long type, Object obj, String methodName, String key,
                    Object parm1,
                    Object parm2)
{
Object[] parms = {parm1, parm2};
String className = obj.getClass().getName();
msg(type, className, methodName, key, (String) null, parms);
}


/*----------------------------------------------------------------------
 * Name:  message
 *--------------------------------------------------------------------*/
/**
 * Logs a message, by key, with an array of parameters.
 * This call can be used in a non-static method and assumes that at
 * least one message file is registered with the logger.
 *
 * @param type       The type of the message.                   
 * @param obj        The logging object ("this").
 * @param methodName The name of the logging method.
 * @param key        The message key.
 * @param parms      An array of elements to be inserted into the message.
 */

public void message(long type, Object obj, String methodName, String key,
                    Object[] parms)
{
String className = obj.getClass().getName();
msg(type, className, methodName, key, (String) null, parms);
}


/*----------------------------------------------------------------------
 * Name:  message
 *--------------------------------------------------------------------*/
/**
 * Logs a message, by key, with no parameters.
 * This call can be used in any method and assumes that at
 * least one message file is registered with the logger.
 *
 * @param type       The type of the message.
 * @param className  The name of the logging class.
 * @param methodName The name of the logging method.
 * @param key        The message key.
 */

public void message(long type, String className, String methodName, String key)
{
msg(type, className, methodName, key, (String) null, (Object[]) null);
}


/*----------------------------------------------------------------------
 * Name:  message
 *--------------------------------------------------------------------*/
/**
 * Logs a message, by key, with one parameter.
 * This call can be used in any method and assumes that at
 * least one message file is registered with the logger.
 *
 * @param type       The type of the message.
 * @param className  The name of the logging class. 
 * @param methodName The name of the logging method.
 * @param key        The message key.
 * @param parm1      An element to be inserted into the message.
 */

public void message(long type, String className, String methodName, String key,
                    Object parm1)
{
Object[] parms = {parm1};
msg(type, className, methodName, key, (String) null, parms);
}


/*----------------------------------------------------------------------
 * Name:  message
 *--------------------------------------------------------------------*/
/**
 * Logs a message, by key, with two parameters.
 * This call can be used in any method and assumes that at
 * least one message file is registered with the logger.
 *
 * @param type       The type of the message.
 * @param className  The name of the logging class. 
 * @param methodName The name of the logging method.
 * @param key        The message key.
 * @param parm1      An element to be inserted into the message.
 * @param parm2      An element to be inserted into the message.
 */

public void message(long type, String className, String methodName, String key,
                    Object parm1, 
                    Object parm2)
{
Object[] parms = {parm1, parm2};
msg(type, className, methodName, key, (String) null, parms);
}


/*----------------------------------------------------------------------
 * Name:  message
 *--------------------------------------------------------------------*/
/**
 * Logs a message, by key, with an array of parameters.
 * This call can be used in any method and assumes that at
 * least one message file is registered with the logger.
 *
 * @param type       The type of the message.
 * @param className  The name of the logging class. 
 * @param methodName The name of the logging method.
 * @param key        The message key.
 * @param parms      An array of elements to be inserted into the message.
 */                

public void message(long type, String className, String methodName, String key,
                    Object[] parms)
{
msg(type, className, methodName, key, (String) null, parms);
}


/*----------------------------------------------------------------------
 * Name:  msg
 *--------------------------------------------------------------------*/
/**
 * Logs a message, by key and file, with no parameters.
 * This call can be used in a non-static method.
 *
 * @param type       The type of the message.
 * @param obj        The logging object ("this").
 * @param methodName The name of the logging method.
 * @param key        The message key.
 * @param file       The name of the message file to use.
 */

public void msg(long type, Object obj, String methodName, String key, String file)
{
String className = obj.getClass().getName();
msg(type, className, methodName, key, file, (Object[]) null);
}


/*----------------------------------------------------------------------
 * Name:  msg
 *--------------------------------------------------------------------*/
/**
 * Logs a message, by key and file with one parameter.
 * This call can be used in a non-static method.
 *
 * @param type       The type of the message.
 * @param obj        The logging object ("this").
 * @param methodName The name of the logging method.
 * @param key        The message key.
 * @param file       The name of the message file to use.
 * @param parm1      An element to be inserted into the message.
 */

public void msg(long type, Object obj, String methodName, String key, String file,
                Object parm1)
{
Object[] parms = {parm1};
String className = obj.getClass().getName();
msg(type, className, methodName, key, file, parms);
}


/*----------------------------------------------------------------------
 * Name:  msg
 *--------------------------------------------------------------------*/
/**
 * Logs a message, by key and file, with two parameters.
 * This call can be used in a non-static method.
 *
 * @param type       The type of the message.
 * @param obj        The logging object ("this").
 * @param methodName The name of the logging method.
 * @param key        The message key.
 * @param file       The name of the message file to use.
 * @param parm1      An element to be inserted into the message.
 * @param parm2      An element to be inserted into the message.
 */

public void msg(long type, Object obj, String methodName, String key, String file,
                Object parm1, 
                Object parm2)
{
Object[] parms = {parm1, parm2};
String className = obj.getClass().getName();
msg(type, className, methodName, key, file, parms);
}


/*----------------------------------------------------------------------
 * Name:  msg
 *--------------------------------------------------------------------*/
/**
 * Logs a message, by key and file, with an array of parameters.
 * This call can be used in a non-static method.
 *
 * @param type       The type of the message.                   
 * @param obj        The logging object ("this").
 * @param methodName The name of the logging method.
 * @param key        The message key.
 * @param file       The name of the message file to use.
 * @param parms      An array of elements to be inserted into the message.
 */

public void msg(long type, Object obj, String methodName, String key, String file,
                Object[] parms)
{
String className = obj.getClass().getName();
msg(type, className, methodName, key, file, parms);
}


/*----------------------------------------------------------------------
 * Name:  msg
 *--------------------------------------------------------------------*/
/**
 * Logs a message, by key and file, with no parameters.
 * This call can be used in any method.
 *
 * @param type       The type of the message.
 * @param className  The name of the logging class.
 * @param methodName The name of the logging method.
 * @param key        The message key.
 * @param file       The name of the message file to use.
 */

public void msg(long type, String className, String methodName, String key, String file)
{
msg(type, className, methodName, key, file, (Object[]) null);
}


/*----------------------------------------------------------------------
 * Name:  msg
 *--------------------------------------------------------------------*/
/**
 * Logs a message, by key and file, with one parameter.
 * This call can be used in any method.
 *
 * @param type        The type of the message.
 * @param className   The name of the logging class. 
 * @param methodName  The name of the logging method.
 * @param key         The message key.
 * @param file        The name of the message file to use.
 * @param parm1       An element to be inserted into the message.
 */

public void msg(long type, String className, String methodName, String key, String file,
                Object parm1)
{
Object[] parms = {parm1};
msg(type, className, methodName, key, file, parms);
}


/*----------------------------------------------------------------------
 * Name:  msg
 *--------------------------------------------------------------------*/
/**
 * Logs a message, by key and file, with two parameters.
 * This call can be used in any method.
 *
 * @param type       The type of the message.
 * @param className  The name of the logging class. 
 * @param methodName The name of the logging method.
 * @param key        The message key.
 * @param file       The name of the message file to use.
 * @param parm1      An element to be inserted into the message.
 * @param parm2      An element to be inserted into the message.
 */

public void msg(long type, String className, String methodName, String key, String file,
                Object parm1, 
                Object parm2)
{
Object[] parms = {parm1, parm2};
msg(type, className, methodName, key, file, parms);
}


/*----------------------------------------------------------------------
 * Name:  msg
 *--------------------------------------------------------------------*/
/**
 * Logs a message, by key and file, with an array of parameters.
 * This call can be used in any method.
 *
 * @param type       The type of the message.                   
 * @param className  The name of the logging class.
 * @param methodName The name of the logging method.
 * @param key        The message key.
 * @param file       The name of the message file to use.
 * @param parms      An array of elements to be inserted into the message.
 */

public void msg(long type, String className, String methodName, String key, String file,
                Object[] parms)
{
String[] inserts = null;

// Log the message if it will pass the masks of the logger and at
// least one handler and the message key is not suppressed.

if (isLoggable(type) && !getSuppressedKeys().contains(key))
  {

  // If the message file is null, use the file specified by 
  // setMessageFile().  Otherwise, use the one passed to this method.

  if (file == null)
    file = getMessageFile();

  if (parms != null)
    {
    inserts = new String[parms.length];
    for (int j = 0; j < parms.length; j++)
      inserts[j] = parms[j].toString();
    }

  RASMessageEvent event = 
    new RASMessageEvent(type, 
                        key, inserts, file,
                        className, methodName, 
                        getOrganization(), getProduct(), getComponent(), 
                        getServer(), getClient(),
                        (Exception) null);
  fireRASEvent(event); 
  }
}


/*----------------------------------------------------------------------
 * Name:  textMessage
 *--------------------------------------------------------------------*/
/**
 * Logs a text message (no key) with no parameters.
 * This call can be used in a non-static method.
 *
 * @param type       The type of the message.
 * @param obj        The logging object ("this").
 * @param methodName The name of the logging method.
 * @param text       The message text.
 */

public void textMessage(long type, Object obj, String methodName, String text)
{
textMessage(type, obj, methodName, text, (Object[]) null);
}


/*----------------------------------------------------------------------
 * Name:  textMessage
 *--------------------------------------------------------------------*/
/**
 * Logs a text message (no key) with one parameter.
 * This call can be used in a non-static method.
 *
 * @param type       The type of the message.
 * @param obj        The logging object ("this").
 * @param methodName The name of the logging method.
 * @param text       The message text.
 * @param parm1      An element to be displayed with the message.
 */

public void textMessage(long type, Object obj, String methodName, String text,
                        Object parm1)
{
Object[] parms = {parm1};
textMessage(type, obj, methodName, text, parms);
}


/*----------------------------------------------------------------------
 * Name:  textMessage
 *--------------------------------------------------------------------*/
/**
 * Logs a text message (no key) with two parameters.
 * This call can be used in a non-static method.
 *
 * @param type       The type of the message.
 * @param obj        The logging object ("this").
 * @param methodName The name of the logging method.
 * @param text       The message text.
 * @param parm1      An element to be displayed with the message.
 * @param parm2      An element to be displayed with the message.
 */

public void textMessage(long type, Object obj, String methodName, String text,
                        Object parm1, 
                        Object parm2)
{
Object[] parms = {parm1, parm2};
textMessage(type, obj, methodName, text, parms);
}


/*----------------------------------------------------------------------
 * Name:  textMessage
 *--------------------------------------------------------------------*/
/**
 * Logs a text message (no key) with an array of parameters.
 * This call can be used in a non-static method.
 *
 * @param type       The type of the message.
 * @param obj        The logging object ("this").
 * @param methodName The name of the logging method.
 * @param text       The message text.
 * @param parms      An array of elements to be displayed with the message.
 */

public void textMessage(long type, Object obj, String methodName, String text,
                        Object[] parms) 
{
String className = obj.getClass().getName();
textMessage(type, className, methodName, text, parms);
}


/*----------------------------------------------------------------------
 * Name:  textMessage
 *--------------------------------------------------------------------*/
/**
 * Logs a text message (no key) with no parameters.
 * This call can be used in any method.
 *
 * @param type       The type of the message.
 * @param className  The name of the logging class. 
 * @param methodName The name of the logging method.
 * @param text       The message text.
 */                

public void textMessage(long type, String className, String methodName, String text)
{
textMessage(type, className, methodName, text, (Object[]) null);
}


/*----------------------------------------------------------------------
 * Name:  textMessage
 *--------------------------------------------------------------------*/
/**
 * Logs a text message (no key) with one parameter.
 * This call can be used in any method.
 *
 * @param type       The type of the message.
 * @param className  The name of the logging class. 
 * @param methodName The name of the logging method.
 * @param text       The message text.
 * @param parm1      An element to be displayed with the message.
 */                

public void textMessage(long type, String className, String methodName, String text,
                        Object parm1)
{
Object[] parms = {parm1};
textMessage(type, className, methodName, text, parms);
}


/*----------------------------------------------------------------------
 * Name:  textMessage
 *--------------------------------------------------------------------*/
/**
 * Logs a text message (no key) with two parameters.
 * This call can be used in any method.
 *
 * @param type       The type of the message.
 * @param className  The name of the logging class. 
 * @param methodName The name of the logging method.
 * @param text       The message text.
 * @param parm1      An element to be displayed with the message.
 * @param parm2      An element to be displayed with the message.
 */                

public void textMessage(long type, String className, String methodName, String text,
                        Object parm1, 
                        Object parm2)
{
Object[] parms = {parm1, parm2};
textMessage(type, className, methodName, text, parms);
}


/*----------------------------------------------------------------------
 * Name:  textMessage
 *--------------------------------------------------------------------*/
/**
 * Logs a text message (no key) with an array of parameters.
 * This call can be used in any method.
 *
 * @param type       The type of the message.
 * @param className  The name of the logging class.
 * @param methodName The name of the logging method.
 * @param text       The message text.
 * @param parms      An array of elements to be displayed with the message.
 */

public void textMessage(long type, String className, String methodName, String text,
                        Object[] parms)
{
String[] inserts = null;

// Log the message if it will pass the masks of the logger and at
// least one handler.

if (isLoggable(type))
  {
  if (parms != null)
    {
    inserts = new String[parms.length];
    for (int j = 0; j < parms.length; j++)
      inserts[j] = parms[j].toString();
    }

  RASMessageEvent event = 
    new RASMessageEvent(type, 
                        text, inserts, (String) null,
                        className, methodName, 
                        getOrganization(), getProduct(), getComponent(), 
                        getServer(), getClient(),
                        (Exception) null);
  fireRASEvent(event);
  }
}


/*----------------------------------------------------------------------
 * Name:  exception
 *--------------------------------------------------------------------*/
/**
 * Logs an <code>Exception</code> in a non-static method.
 *
 * @param type       The type of the trace point.
 * @param obj        The traced object ("this").
 * @param methodName The name of the traced method.
 * @param exc        An exception.
 */

public void exception(long type, Object obj, String methodName,
                      Exception exc)
{
String className = obj.getClass().getName();
exception(type, className, methodName, exc);
}


/*----------------------------------------------------------------------
 * Name:  exception
 *--------------------------------------------------------------------*/
/**
 * Logs an <code>Exception</code> in a static method.
 *
 * @param type       The type of the trace point.
 * @param className  The name of the traced class.
 * @param methodName The name of the traced method.
 * @param exc        An exception.
 */

public void exception(long type, String className, String methodName,
                      Exception exc)
{

// Log the exception if it will pass the masks of the logger and at
// least one handler.

if (isLoggable(type))
  {
  RASMessageEvent event = 
    new RASMessageEvent(type, 
                        (String) null, (String[]) null, (String) null,
                        className, methodName, 
                        getOrganization(), getProduct(), getComponent(), 
                        getServer(), getClient(),
                        exc);
  fireRASEvent(event);
  }
}


/*----------------------------------------------------------------------
 * Name: maskValueChanged
 *--------------------------------------------------------------------*/
/**
 * Indicates that the value of the handler's message mask has
 * changed.
 * <p>
 * This method is intended to improve the performance of the 
 * <code>RASLogger.isLoggable</code> method.  When notified of a change
 * in the value of a handler's mask, the logger can update its
 * internal data, which allows the logger to determine if a RAS event
 * will be logged.
 **/

public synchronized void maskValueChanged()
{

// Form the logical OR of the message masks of each handler.

isLoggableMask = 0;
Enumeration e = getHandlers();
while (e.hasMoreElements())
  {
  RASIHandler h = (RASIHandler) e.nextElement();
  isLoggableMask |= h.getMessageMask();
  }

// AND this with the logger's message mask.  The resulting value indicates
// which message types will be processed by the logger and at least one
// handler.

isLoggableMask &= getMessageMask();
}

}
