/*----------------------------------------------------------------------
 * IBM Confidential
 * OCO Source Materials
 *
 * (C) Copyright IBM Corporation 1998.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *--------------------------------------------------------------------*/

package com.ibm.ras;

import java.io.*;
import java.net.*;
import java.util.*;

/**
 * <code>RASSocketHandler</code> implements a <code>RASHandler</code> 
 * that writes to a TCP socket.  If the server or port is changed through 
 * the <code>setServer</code> or <code>setPort</code> method, call the 
 * <code>connect</code> method to open a socket to the server.
 * <p>
 * The default server is "localhost" and the default port is 9991.
 *
 * @see RASHandler
 */

public class RASSocketHandler extends RASHandler
{
private static final String S = RASCopyright.COPYRIGHT_98;

/**
 * The TCP port for the connection.
 */

private int port = 9991;

/**
 * The server to which the <code>RASSocketHandler</code> connects.
 */

private String server = "localhost";

/**
 * The TCP socket over which data is written.
 */

private Socket socket = null;


/*----------------------------------------------------------------------
 * Name:  RASSocketHandler
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASSocketHandler</code>.  The name and
 * description of this object are empty strings.
 */

public RASSocketHandler() 
{
this("", "");
}


/*----------------------------------------------------------------------
 * Name:  RASSocketHandler
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASSocketHandler</code>.  The description
 * of this object is an empty string.
 *
 * @param name The name of this object.
 */

public RASSocketHandler(String name) 
{
this(name, "");
}


/*----------------------------------------------------------------------
 * Name:  RASSocketHandler
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASSocketHandler</code>.
 *
 * @param name The name of this object.
 * @param desc The description of this object.
 */

public RASSocketHandler(String name, String desc) 
{
super(name, desc);
}


/*----------------------------------------------------------------------
 * Name:  RASSocketHandler
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASSocketHandler</code>.
 *
 * @param name   The name of this object.
 * @param desc   The description of this object.
 * @param server The name of the server to which this handler connects.
 * @param port   The TCP port number on which this handler connects.
 */

public RASSocketHandler(String name, String desc, String server, int port) 
{
this(name, desc);
setServer(server);
setPort(port);
}


/*----------------------------------------------------------------------
 * Name:  getConfig
 *--------------------------------------------------------------------*/
/**
 * Gets the configuration of this object.
 *
 * @return A <code>Hashtable</code> containing the configuration.
 *         This object inserts the following key/value pairs into the
 *         configuration:
 *         <pre>
 *         "port"   The TCP port on which this handler will connect.
 *         "server" The name of the server to which this handler
 *                  will connect.
 *         </pre>
 *         All values are <code>Strings</code>.
 *         The parent and extensions of this object may add additional keys.
 **/

public Hashtable getConfig()
{
Hashtable ht = super.getConfig();

ht.put("port", new Integer(getPort()).toString());
ht.put("server", getServer());

return ht;
}


/*----------------------------------------------------------------------
 * Name:  setConfig
 *--------------------------------------------------------------------*/
/**
 * Sets the configuration of this object.  This method is used by a 
 * <code>RASManager</code> to initialize a RAS object.  It should not 
 * be necessary for an application to use this method.
 *
 * @param ht  A <code>Hashtable</code> containing the configuration.
 *            This object searches for the following keys:
 *            <pre>
 *            "port"   The TCP port on which this handler will connect.
 *            "server" The name of the server to which this handler
 *                     will connect.
 *            </pre>
 *            All values are <code>Strings</code>.
 *            If a key is not found, that configuration element is not
 *            updated.
 *            The parent and extensions of this object may add additional keys.
 **/

public void setConfig(Hashtable ht)
{
super.setConfig(ht);
if (ht.containsKey("port"))
  setPort(new Integer((String) ht.get("port")).intValue());
if (ht.containsKey("server"))
  setServer((String) ht.get("server"));
}


/*----------------------------------------------------------------------
 * Name:  getPort
 *--------------------------------------------------------------------*/
/**
 * Gets the port number of the TCP socket.
 *
 * @return The TCP port number on which the handler connects.  If a
 *         port has not been set, the default 9991 is returned.
 **/

public int getPort()
{
return port;
}


/*----------------------------------------------------------------------
 * Name:  setPort
 *--------------------------------------------------------------------*/
/**
 * Sets the port number of the TCP socket.
 *
 * @param port The TCP port number on which the handler connects.
 **/

public void setPort(int port)
{
this.port = port;
}


/*----------------------------------------------------------------------
 * Name:  getServer
 *--------------------------------------------------------------------*/
/**
 * Gets the server to which the handler connects.
 *  
 * @return The name of the server.  If a server has not been set, the 
 *         default "localhost" is returned.
 **/

public String getServer()
{
return server;
}

/*----------------------------------------------------------------------
 * Name:  setServer
 *--------------------------------------------------------------------*/
/**
 * Sets the name of the server to which the handler connects.  If the 
 * server name is <code>null</code>, the current server is not changed.
 *
 * @param server The name of the server.
 **/

public void setServer(String server)
{
if (server != null)
  this.server = server;
}


/*----------------------------------------------------------------------
 * Name:  openDevice
 *--------------------------------------------------------------------*/
/**
 * Opens the device to which the handler will send its log entries.
 * If an error occurs during this process, the handler will write a
 * message to <code>System.err</code>.
 * <p>
 * The caller should ensure that this method is not called if the device
 * is already open or if cannot tolerate attempts to reopen it.
 **/

public void openDevice()
{
try
  {
  socket = new Socket(getServer(), getPort());
  pWriter = new PrintWriter(new DataOutputStream(socket.getOutputStream()), true);
  deviceOpen = true;
  }
catch (Exception e)
  {
  System.err.println(RASUtil.rasMsgs.getMessage("ERR_OPEN_SOCKET", 
                                                getServer(), 
                                                new Integer(getPort())));
  }
}

}
