/*----------------------------------------------------------------------
 * IBM Confidential
 * OCO Source Materials
 *
 * (C) Copyright IBM Corporation 1998.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *--------------------------------------------------------------------*/

package com.ibm.ras;

import java.io.*;
import java.util.*;

/**
 * A <code>RASTraceEvent</code> encapsulates all of the information generated by
 * a <code>RASTraceLogger</code>.  <I>It is not necessary
 * to create a <code>RASTraceEvent</code> directly, as the <code>RASTraceLogger</code>
 * handles this task.  However, the methods of this class are public, should
 * it be necessary to extend <code>RASTraceEvent</code>.</I>
 * <p>
 * A <code>RASTraceEvent</code> object contains the following components:
 * <ul>
 * <li>A flag which identifies this event as a trace event.
 * <li>A time stamp, generated when the <code>RASTraceEvent</code> is created.
 * <li>A trace type.                                           
 * <li>The message text, which conveys some useful information
 * to the developer.  Since tracing is only intended for the software development
 * team, no provision is made for translating <code>RASTraceEvent</code> messages.
 * <li>A set of <code>Strings</code> which can be displayed with the message text.
 * These parameters are required only if the message expects them.
 * <li>A set of additional attributes that may be part of the event data.  
 * The following attributes may be set.  Unless otherwise noted, all 
 * <code>Objects</code> associated with the attributes are <code>Strings</code>.
 * <pre>
 * "className"    The name of the Java class which generated the 
 *                message.
 * "methodName"   The name of the Java method which generated the
 *                message.
 * "threadID"     The thread identifier.
 * "logger"       The name of the <code>RASTraceLogger</code> which
 *                created the message.
 * "server"       A server associated with the creation of the
 *                message.
 * "client"       The client on whose behalf the message was created.
 * "exception"    An <code>Exception</code>
 * </pre>
 * </ul>
 * <p>
 * The <code>RASTraceEvent</code> type is defined by the set of 
 *<code>RASITraceEvent.TYPE_XXXX</code> constants.
 * Values 0x000000 up to, but not including, 0x100000L are reserved.  
 * Extensions to the <code>RASTraceEvent</code>
 * class may use values outside this range for additional types.
 * The constant <code>DEFAULT_TRACE_MASK</code> contains the set of <code>RASTraceEvent</code> 
 * types that the handlers will log by default.  The default mask is defined as follows:
 * <pre>
 * public static final long DEFAULT_TRACE_MASK = 
 *   TYPE_API |
 *   TYPE_CALLBACK |
 *   TYPE_ENTRY_EXIT |
 *   TYPE_ERROR_EXC |
 *   TYPE_MISC_DATA |
 *   TYPE_OBJ_CREATE |
 *   TYPE_OBJ_DELETE |
 *   TYPE_PRIVATE |
 *   TYPE_PUBLIC |
 *   TYPE_STATIC |
 *   TYPE_SVC;
 * </pre>
 *
 * @see RASEvent
 * @see RASTraceLogger
 * @see RASHandler
 */

public class RASTraceEvent extends RASEvent implements RASITraceEvent
{
private static final String S = RASCopyright.COPYRIGHT_98;


/*----------------------------------------------------------------------
 * Name:  RASTraceEvent
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASTraceEvent</code> object.
 *
 * @param type      The type of trace event.
 * @param text      The trace text.
 * @param parms     An array of <code>Strings</code> to be displayed 
 *                  with the message.
 */

protected RASTraceEvent(long type, 
                        String text, String[] parms)
{
super();
setType(type);
setText(text);
setParameters(parms);
setAttribute("threadID", Thread.currentThread().getName());
setMessageEvent(false);
}


/*----------------------------------------------------------------------
 * Name:  RASTraceEvent
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASTraceEvent</code> object.
 *
 * @param type       The type of trace event.
 * @param text       The trace text.
 * @param parms      An array of <code>Strings</code> to be displayed 
 *                   with the message.
 * @param className  The class.
 * @param methodName The method.
 * @param logger     The <code>RASITraceLogger</code> which created this 
 *                   <code>RASTraceEvent</code>.
 * @param server     The server.
 * @param client     The client.
 * @param exception  An exception.
 */

public RASTraceEvent(long type, 
                     String text, String[] parms, 
                     String className, String methodName,
                     RASITraceLogger logger,
                     String server, String client,
                     Exception exception)
{
this(type, text, parms);

if (className != null)
  setAttribute("className", className);

if (methodName != null)
  setAttribute("methodName", methodName);

if (logger != null)
  setAttribute("logger", logger.getName());

if (server != null)
  setAttribute("server", server);

if (client != null)
  setAttribute("client", client);

if (exception != null)
  setAttribute("exception", exception);
}


/*----------------------------------------------------------------------
 * Name:  getSupportedTypes
 *--------------------------------------------------------------------*/
/**
 * Gets the complete set of trace event types supported by this class.
 * <p>
 * This method can be used by a graphical log manager to display the 
 * set of types and allow a user to select those to be monitored.  Every
 * extending class should override this method to add the types it supports
 * to the <code>Hashtable</code> returned by its parent.
 *
 * @return A <code>Hashtable</code> containing the values of all of the
 *         trace event types.  The names of the types are used as keys and
 *         are returned in the current locale.  The type values,
 *         normally kept as <code>longs</code>, are returned as
 *         <code>Longs</code> because of the <code>Hashtable</code>
 *         requirement that all elements be <code>Objects</code>.
 */

public Hashtable getSupportedTypes()
{
Hashtable ht = super.getSupportedTypes();

ht.put(RASUtil.rasMsgs.getMessage("TYPE_API"), new Long(TYPE_API));
ht.put(RASUtil.rasMsgs.getMessage("TYPE_CALLBACK"), new Long(TYPE_CALLBACK));
ht.put(RASUtil.rasMsgs.getMessage("TYPE_ENTRY_EXIT"), new Long(TYPE_ENTRY_EXIT));
ht.put(RASUtil.rasMsgs.getMessage("TYPE_ERROR_EXC"), new Long(TYPE_ERROR_EXC));
ht.put(RASUtil.rasMsgs.getMessage("TYPE_MISC_DATA"), new Long(TYPE_MISC_DATA));
ht.put(RASUtil.rasMsgs.getMessage("TYPE_OBJ_CREATE"), new Long(TYPE_OBJ_CREATE));
ht.put(RASUtil.rasMsgs.getMessage("TYPE_OBJ_DELETE"), new Long(TYPE_OBJ_DELETE));
ht.put(RASUtil.rasMsgs.getMessage("TYPE_PRIVATE"), new Long(TYPE_PRIVATE));
ht.put(RASUtil.rasMsgs.getMessage("TYPE_PUBLIC"), new Long(TYPE_PUBLIC));
ht.put(RASUtil.rasMsgs.getMessage("TYPE_STATIC"), new Long(TYPE_STATIC));
ht.put(RASUtil.rasMsgs.getMessage("TYPE_SVC"), new Long(TYPE_SVC));
ht.put(RASUtil.rasMsgs.getMessage("TYPE_LEVEL1"), new Long(TYPE_LEVEL1));
ht.put(RASUtil.rasMsgs.getMessage("TYPE_LEVEL2"), new Long(TYPE_LEVEL2));
ht.put(RASUtil.rasMsgs.getMessage("TYPE_LEVEL3"), new Long(TYPE_LEVEL3));

return ht;
}


/*----------------------------------------------------------------------
 * Name: maskLongValue
 *--------------------------------------------------------------------*/
/**
 * Converts a <code>String</code> representation of the trace mask
 * to its long equivalent.  The string must have the following format:
 * <pre>
 * String types = "TYPE_API TYPE_PRIVATE TYPE_HIGH_USE";
 * </pre
 * The value <code>DEFAULT_TRACE_MASK</code> may also be included in the string.
 * (These values are the names of the <code>RASITraceEvent.TYPE_XXXX</code>
 * constants.)
 * Every extending class should override this method to add the types it
 * supports to the value returned by its parent.  
 * <p>
 * Unknown values in the <code>String</code> are ignored.
 *
 * @param types The blank-delimited <code>String</code> to be converted.
 *
 * @return The long equivalent of the <code>String</code>, formed by
 *         the logical OR of each element in the <code>String</code>.
 */

public static long maskLongValue(String types)
{
long result = 0;
StringTokenizer st = new StringTokenizer(types);

while (st.hasMoreElements())
  {
  String type = st.nextToken();
  if (type.equals("TYPE_API"))
    result |= TYPE_API;
  else if (type.equals("TYPE_CALLBACK"))
    result |= TYPE_CALLBACK;
  else if (type.equals("TYPE_ENTRY_EXIT"))
    result |= TYPE_ENTRY_EXIT;
  else if (type.equals("TYPE_ERROR_EXC"))
    result |= TYPE_ERROR_EXC;
  else if (type.equals("TYPE_MISC_DATA"))
    result |= TYPE_MISC_DATA;
  else if (type.equals("TYPE_OBJ_CREATE"))
    result |= TYPE_OBJ_CREATE;
  else if (type.equals("TYPE_OBJ_DELETE"))
    result |= TYPE_OBJ_DELETE;
  else if (type.equals("TYPE_PRIVATE"))
    result |= TYPE_PRIVATE;
  else if (type.equals("TYPE_PUBLIC"))
    result |= TYPE_PUBLIC;
  else if (type.equals("TYPE_STATIC"))
    result |= TYPE_STATIC;
  else if (type.equals("TYPE_SVC"))
    result |= TYPE_SVC;
  else if (type.equals("TYPE_LEVEL1"))
    result |= TYPE_LEVEL1;
  else if (type.equals("TYPE_LEVEL2"))
    result |= TYPE_LEVEL2;
  else if (type.equals("TYPE_LEVEL3"))
    result |= TYPE_LEVEL3;
  else if (type.equals("DEFAULT_TRACE_MASK"))
    result |= DEFAULT_TRACE_MASK;
  }

return result;
}


/*----------------------------------------------------------------------
 * Name: maskToString
 *--------------------------------------------------------------------*/
/**
 * Converts a long containing the logical OR of one or more
 * <code>RASITraceEvent.TYPE_XXXX</code> constants to its <code>String</code>
 * equivalent.  The string will have the following format:
 * <pre>
 * String types = "TYPE_API TYPE_PRIVATE TYPE_HIGH_USE";
 * </pre>
 * Every extending class should override this method to add the types it
 * supports to the value returned by its parent.
 * <p>
 * Unknown values in the <code>long</code> are ignored.
 *
 * @param types The trace mask.
 *
 * @return The <code>String</code> equivalent of the trace mask.
 */

public static String maskToString(long types)
{
StringBuffer bfr = new StringBuffer();

if ((types & TYPE_API) != 0)
  bfr.append("TYPE_API ");
if ((types & TYPE_CALLBACK) != 0)
  bfr.append("TYPE_CALLBACK ");
if ((types & TYPE_ENTRY_EXIT) != 0)
  bfr.append("TYPE_ENTRY_EXIT ");
if ((types & TYPE_ERROR_EXC) != 0)
  bfr.append("TYPE_ERROR_EXC ");
if ((types & TYPE_MISC_DATA) != 0)
  bfr.append("TYPE_MISC_DATA ");
if ((types & TYPE_OBJ_CREATE) != 0)
  bfr.append("TYPE_OBJ_CREATE ");
if ((types & TYPE_OBJ_DELETE) != 0)
  bfr.append("TYPE_OBJ_DELETE ");
if ((types & TYPE_PRIVATE) != 0)
  bfr.append("TYPE_PRIVATE ");
if ((types & TYPE_PUBLIC) != 0)
  bfr.append("TYPE_PUBLIC ");
if ((types & TYPE_STATIC) != 0)
  bfr.append("TYPE_STATIC ");
if ((types & TYPE_SVC) != 0)
  bfr.append("TYPE_SVC ");
if ((types & TYPE_LEVEL1) != 0)
  bfr.append("TYPE_LEVEL1 ");
if ((types & TYPE_LEVEL2) != 0)
  bfr.append("TYPE_LEVEL2 ");
if ((types & TYPE_LEVEL3) != 0)
  bfr.append("TYPE_LEVEL3 ");

return bfr.toString().trim();
}

}
