/*----------------------------------------------------------------------
 * IBM Confidential
 * OCO Source Materials
 *
 * (C) Copyright IBM Corporation 1998.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *--------------------------------------------------------------------*/

package com.ibm.ras;

import java.io.*;
import java.util.*;

/**
 * A <code>RASTraceFormatter</code> formats a <code>RASTraceEvent</code>
 * for display.  The formatted <code>String</code> includes the following
 * items:
 * <ul>
 * <li>The time at which the event was created.
 * <li>The name of the Java class which generated the message.
 * <li>The name of the Java method which generated the message.
 * <li>A thread identifier.
 * <li>The name of the <code>RASITraceLogger</code>.
 * <li>The event text.
 * </ul>
 * If the trace event includes an <code>Exception</code>, the event text
 * is taken from <code>Exception.getMessage</code> and the exception's
 * stack trace is appended to the formatted output.
 *
 * @see RASTraceEvent
 * @see RASHandler
 */

public class RASTraceFormatter extends RASFormatter
{
private static final String S = RASCopyright.COPYRIGHT_98;

/**
 * A set of attribute names used to format the event.  They are listed
 * in the same order that they will be formatted.
 */

private static String[] attribs = 
  {
  "className",
  "methodName",
  "threadID",
  "logger"
  };


/*----------------------------------------------------------------------
 * Name:  RASTraceFormatter
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASTraceFormatter</code>.
 */

public RASTraceFormatter()
{

// Add the default RAS event classes which this formatter can process.

addEventClass("com.ibm.ras.RASTraceEvent");
}


/*----------------------------------------------------------------------
 * Name:  format
 *--------------------------------------------------------------------*/
/**
 * Formats the trace event.
 * 
 * @param event The event to format.
 * 
 * @return The formatted event.
 **/

public String format(RASIEvent event)
{

String attrib;
String sep = getSeparator();
StringBuffer bfr = new StringBuffer(256);

bfr.append(getTime(event.getTimeStamp()) + sep);

// Append the String attributes.

for (int j = 0; j < attribs.length; j++)
  {
  attrib = (String) event.getAttribute(attribs[j]);
  if (attrib != null)
    bfr.append(attrib + sep);
  }

// For an exception, append the stack trace.

Exception e = (Exception) event.getAttribute("exception");
if (e != null)
  {
  StringWriter sw = new StringWriter();
  e.printStackTrace(new PrintWriter(sw));
  bfr.append(sw.toString());
  }

// Otherwise, append the trace text and any additional objects.

else
  {
  bfr.append(event.getText());
  String[] parms = event.getParameters();
  if (parms != null)
    {
    for (int j = 0; j < parms.length; j++)
      {    
      bfr.append(", p" + (j+1) + "=" + parms[j]);
      }
    }
  }

return bfr.toString();
}

}
