/*----------------------------------------------------------------------
 * IBM Confidential
 * OCO Source Materials
 *
 * (C) Copyright IBM Corporation 1998.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *--------------------------------------------------------------------*/

package com.ibm.ras;

import java.text.DateFormat;
import java.util.*;

/**
 * The <code>RASTraceLogger</code> is one of two <code>RASLogger</code> 
 * sub-classes provided in this package.  (The other is 
 * <code>RASMessageLogger</code>.)  The purpose of tracing is to provide 
 * sufficient information to a developer or service team to diagnose a 
 * problem remotely (for example, when the application is in production on 
 * a customer site).  Normally an application will run with tracing turned 
 * off.  When a problem occurs, it can be turned on to gather the data
 * needed to understand the problem.  (A logger's "on/off" state can be
 * controlled by setting its <code>isLogging</code> flag.  It might also be 
 * controlled through a graphical program, not included in this package.)
 * <p>
 * When tracing in non-static methods, use the <code>entry, exit, trace</code> 
 * and <code>exception</code> methods that take "this" (the object being traced) as 
 * the second parameter.  This is a convenience to  the programmer, as the 
 * class name can be derived from any <code>Object</code>.  For static methods, 
 * an object does not exist.  Use the <code>entry, exit, trace</code> and 
 * <code>exception</code> methods which take <code>String className</code> as
 * the second parameter.
 * <p>
 * The set of types that may be used in the <code>entry(), exit()</code> and 
 * <code>trace()</code> methods is defined in the <code>RASITraceEvent</code>
 * class.  These methods do not validate the type assigned to the trace point,
 * so an error could result in the failure to trace a point.
 * <p>
 * <I>Note that the <code>entry</code> and <code>exit</code> methods 
 * always add <code>RASITraceEvent.TYPE_ENTRY_EXIT</code> to whatever type is 
 * selected.  It is not necessary to include <code>TYPE_ENTRY_EXIT</code></I>.
 * <p>
 * <code>RASTraceLogger</code> has several optional fields which may be 
 * included in the message.  These fields should not vary among 
 * messages produced by a given <code>RASTraceLogger</code>, so they are 
 * not included in the <code>entry, exit, trace</code> or <code>exception</code>
 * methods of this class.  They can, 
 * however, be specified through a <code>RASTraceLogger</code> constructor 
 * or by the appropriate "set" and "get" methods of this class.  These fields
 * are:
 * <ul>
 * <li>A server associated with the creation of the messages.
 * <li>The client on whose behalf the messages are created.
 * </ul>
 * If not specified, each of these fields defaults to an empty string.
 *
 * @see RASITraceLogger
 * @see RASLogger
 * @see RASTraceEvent
 * @see RASHandler
 */

public class RASTraceLogger extends RASLogger implements RASITraceLogger
{
private static final String S = RASCopyright.COPYRIGHT_98;

/**
 * Text used in the <code>entry</code> methods.
 */

private static final String ENTRY = "Entry";

/**
 * Text used in the <code>exit</code> methods.
 */

private static final String EXIT = "Exit";

/**
 * The mask which determines the types of trace points that will
 * be processed.
 */

private long traceMask = RASITraceEvent.DEFAULT_TRACE_MASK;


/*----------------------------------------------------------------------
 * Name:  RASTraceLogger
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASTraceLogger</code>.  The name and
 * description of this object are empty strings.
 */

public RASTraceLogger() 
{
this("", "");
}


/*----------------------------------------------------------------------
 * Name:  RASTraceLogger
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASTraceLogger</code>.  The description
 * of this object is an empty string.
 *
 * @param name The name of this object.
 */

public RASTraceLogger(String name) 
{
this(name, "");
}


/*----------------------------------------------------------------------
 * Name:  RASTraceLogger
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASTraceLogger</code>.
 *
 * @param name The name of this object.
 * @param desc The description of this object.
 */

public RASTraceLogger(String name, String desc) 
{
super(name, desc);
}


/*----------------------------------------------------------------------
 * Name:  RASTraceLogger
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASTraceLogger</code>.
 *
 * @param name   The name of this object.
 * @param desc   The description of this object.
 * @param server The server.
 * @param client The client.
 */

public RASTraceLogger(String name, String desc,
                      String server, String client)
{
super(name, desc, server, client);
}


/*----------------------------------------------------------------------
 * Name:  getConfig
 *--------------------------------------------------------------------*/
/**
 * Gets the configuration of this object.
 *
 * @return A <code>Hashtable</code> containing the configuration.
 *         This object inserts the following key/value pairs into the
 *         configuration:
 *         <pre>
 *         "traceMask"   The mask which selects the trace types to be
 *                       processed.
 *         </pre>
 *         All values are <code>Strings</code>.
 *         The parent and extensions of this object may add additional keys.
 **/

public Hashtable getConfig()
{
Hashtable ht = super.getConfig();

ht.put("traceMask", RASTraceEvent.maskToString(getTraceMask()));

return ht;
}


/*----------------------------------------------------------------------
 * Name:  setConfig
 *--------------------------------------------------------------------*/
/**
 * Sets the configuration of this object.  This method is used by a 
 * <code>RASManager</code> to initialize a RAS object.  It should not 
 * be necessary for an application to use this method.
 *
 * @param ht  A <code>Hashtable</code> containing the configuration.
 *            This object searches for the following keys:
 *            <pre>
 *            "traceMask"   The mask which selects the trace types to be
 *                          processed.
 *            </pre>
 *            All values are <code>Strings</code>.
 *            If a key is not found, that configuration element is not
 *            updated.
 *            The parent and extensions of this object may add additional keys.
 **/

public void setConfig(Hashtable ht)
{
super.setConfig(ht);

if (ht.containsKey("traceMask"))
  setTraceMask(RASTraceEvent.maskLongValue((String) ht.get("traceMask")));
}


/*----------------------------------------------------------------------
 * Name:  getTraceMask
 *--------------------------------------------------------------------*/
/**
 * Gets the mask which defines the set of trace types that will be 
 * processed.  Possible values are defined by the logical OR of the
 * <code>RASITraceEvent.TYPE_XXXX</code> constants.
 *
 * @return The trace mask.
 */

public long getTraceMask() 
{
return traceMask;
}


/*----------------------------------------------------------------------
 * Name:  setTraceMask
 *--------------------------------------------------------------------*/
/**
 * Sets the mask which defines the set of trace types that will be 
 * processed.  Possible values are defined by the 
 * <code>RASITraceEvent.TYPE_XXXX</code> constants.
 * The mask value is not validated against these types.
 *
 * @param mask The trace mask.
 */

public void setTraceMask(long mask) 
{
traceMask = mask;
maskValueChanged();
}


/*----------------------------------------------------------------------
 * Name:  entry
 *--------------------------------------------------------------------*/
/**
 * Traces entry into a non-static method.
 *
 * @param type       The type of the trace point.
 * @param obj        The traced object ("this").
 * @param methodName The name of the traced method.
 */

public void entry(long type, Object obj, String methodName)
{
type = type | RASTraceEvent.TYPE_ENTRY_EXIT;
String className = obj.getClass().getName();
trace(type, className, methodName, ENTRY, (Object[]) null);
}


/*----------------------------------------------------------------------
 * Name:  entry
 *--------------------------------------------------------------------*/
/**
 * Traces entry into a non-static method.
 *
 * @param type       The type of the trace point.
 * @param obj        The traced object ("this").
 * @param methodName The name of the traced method.
 * @param parm1      An element to be displayed as trace data.
 */

public void entry(long type, Object obj, String methodName, 
                  Object parm1) 
{
type = type | RASTraceEvent.TYPE_ENTRY_EXIT;
String className = obj.getClass().getName();
Object[] parms = {parm1};
trace(type, className, methodName, ENTRY, parms);
}


/*----------------------------------------------------------------------
 * Name:  entry
 *--------------------------------------------------------------------*/
/**
 * Traces entry into a non-static method.
 *
 * @param type       The type of the trace point.
 * @param obj        The traced object ("this").
 * @param methodName The name of the traced method.
 * @param parm1      An element to be displayed as trace data.
 * @param parm2      An element to be displayed as trace data.
 */

public void entry(long type, Object obj, String methodName, 
                  Object parm1, 
                  Object parm2)
{
type = type | RASTraceEvent.TYPE_ENTRY_EXIT;
String className = obj.getClass().getName();
Object[] parms = {parm1, parm2};
trace(type, className, methodName, ENTRY, parms);
}


/*----------------------------------------------------------------------
 * Name:  entry
 *--------------------------------------------------------------------*/
/**
 * Traces entry into a non-static method.
 *
 * @param type       The type of the trace point.
 * @param obj        The traced object ("this").
 * @param methodName The name of the traced method.
 * @param parms      An array of parameters passed to the method.
 */

public void entry(long type, Object obj, String methodName, 
                  Object[] parms)
{
type = type | RASTraceEvent.TYPE_ENTRY_EXIT;
String className = obj.getClass().getName();
trace(type, className, methodName, ENTRY, parms);
}


/*----------------------------------------------------------------------
 * Name:  entry
 *--------------------------------------------------------------------*/
/**
 * Traces entry into a static method.
 *
 * @param type       The type of the trace point.
 * @param className  The name of the traced class.
 * @param methodName The name of the traced method.
 */

public void entry(long type, String className, String methodName)
{
type = type | RASTraceEvent.TYPE_ENTRY_EXIT;
trace(type, className, methodName, ENTRY, (Object[]) null);
}


/*----------------------------------------------------------------------
 * Name:  entry
 *--------------------------------------------------------------------*/
/**
 * Traces entry into a static method.
 *
 * @param type       The type of the trace point.
 * @param className  The name of the traced class.             
 * @param methodName The name of the traced method.
 * @param parm1      An element to be displayed as trace data.
 */

public void entry(long type, String className, String methodName, 
                  Object parm1)
{
type = type | RASTraceEvent.TYPE_ENTRY_EXIT;
Object[] parms = {parm1};
trace(type, className, methodName, ENTRY, parms);
}


/*----------------------------------------------------------------------
 * Name:  entry
 *--------------------------------------------------------------------*/
/**
 * Traces entry into a static method.
 *
 * @param type       The type of the trace point.
 * @param className  The name of the traced class.             
 * @param methodName The name of the traced method.
 * @param parm1      An element to be displayed as trace data.
 * @param parm2      An element to be displayed as trace data.
 */

public void entry(long type, String className, String methodName, 
                  Object parm1, 
                  Object parm2)
{
type = type | RASTraceEvent.TYPE_ENTRY_EXIT;
Object[] parms = {parm1, parm2};
trace(type, className, methodName, ENTRY, parms);
}


/*----------------------------------------------------------------------
 * Name:  entry
 *--------------------------------------------------------------------*/
/**
 * Traces entry into a static method.
 *
 * @param type       The type of the trace point.
 * @param className  The name of the traced class.             
 * @param methodName The name of the traced method.
 * @param parms      An array of elements to be displayed as trace data.
 */

public void entry(long type, String className, String methodName, 
                  Object[] parms)
{
type = type | RASTraceEvent.TYPE_ENTRY_EXIT;
trace(type, className, methodName, ENTRY, parms);
}


/*----------------------------------------------------------------------
 * Name:  exit
 *--------------------------------------------------------------------*/
/**
 * Traces exit from a non-static method.
 *
 * @param type       The type of the trace point.
 * @param obj        The traced object ("this").
 * @param methodName The name of the traced method.
 */

public void exit(long type, Object obj, String methodName)
{
type = type | RASTraceEvent.TYPE_ENTRY_EXIT;
String className = obj.getClass().getName();
trace(type, className, methodName, EXIT, (Object[]) null);
}


/*----------------------------------------------------------------------
 * Name:  exit
 *--------------------------------------------------------------------*/
/**
 * Traces exit from a non-static method.
 *
 * @param type       The type of the trace point.
 * @param obj        The traced object ("this").
 * @param methodName The name of the traced method.
 * @param retValue   The returned value.
 */

public void exit(long type, Object obj, String methodName,
                 Object retValue)
{
type = type | RASTraceEvent.TYPE_ENTRY_EXIT;
String className = obj.getClass().getName();
Object[] parms = {retValue};
trace(type, className, methodName, EXIT, parms);
}


/*----------------------------------------------------------------------
 * Name:  exit
 *--------------------------------------------------------------------*/
/**
 * Traces exit from a static method.
 *
 * @param type       The type of the trace point.
 * @param className  The name of the traced class.             
 * @param methodName The name of the traced method.
 */

public void exit(long type, String className, String methodName)
{
type = type | RASTraceEvent.TYPE_ENTRY_EXIT;
trace(type, className, methodName, EXIT, (Object[]) null);
}


/*----------------------------------------------------------------------
 * Name:  exit
 *--------------------------------------------------------------------*/
/**
 * Traces exit from a static method.
 *
 * @param type       The type of the trace point.
 * @param className  The name of the traced class.             
 * @param methodName The name of the traced method.
 * @param retValue   The returned value.
 */

public void exit(long type, String className, String methodName, 
                 Object retValue)
{
type = type | RASTraceEvent.TYPE_ENTRY_EXIT;
Object[] parms = {retValue};
trace(type, className, methodName, EXIT, parms);
}


/*----------------------------------------------------------------------
 * Name:  trace
 *--------------------------------------------------------------------*/
/**
 * Traces data in a non-static method.
 *
 * @param type       The type of the trace point.
 * @param obj        The traced object ("this").
 * @param methodName The name of the traced method.
 * @param text       The trace text.
 */

public void trace(long type, Object obj, String methodName, String text)
{
String className = obj.getClass().getName();
trace(type, className, methodName, text, (Object[]) null);
}


/*----------------------------------------------------------------------
 * Name:  trace
 *--------------------------------------------------------------------*/
/**
 * Traces data in a non-static method.
 *
 * @param type       The type of the trace point.
 * @param obj        The traced object ("this").
 * @param methodName The name of the traced method.
 * @param text       The trace text.
 * @param parm1      An element to be displayed as trace data.
 */

public void trace(long type, Object obj, String methodName, String text,
                  Object parm1)
{
String className = obj.getClass().getName();
Object[] parms = {parm1};
trace(type, className, methodName, text, parms);
}


/*----------------------------------------------------------------------
 * Name:  trace
 *--------------------------------------------------------------------*/
/**
 * Traces data in a non-static method.
 *
 * @param type       The type of the trace point.
 * @param obj        The traced object ("this").
 * @param methodName The name of the traced method.
 * @param text       The trace text.
 * @param parm1      An element to be displayed as trace data.
 * @param parm2      An element to be displayed as trace data.
 */

public void trace(long type, Object obj, String methodName, String text,
                  Object parm1, 
                  Object parm2)
{
String className = obj.getClass().getName();
Object[] parms = {parm1, parm2};
trace(type, className, methodName, text, parms);
}


/*----------------------------------------------------------------------
 * Name:  trace
 *--------------------------------------------------------------------*/
/**
 * Traces data in a non-static method.
 *
 * @param type       The type of the trace point.
 * @param obj        The traced object ("this").
 * @param methodName The name of the traced method.
 * @param text       The trace text.
 * @param parms      An array of elements to be displayed with the text.
 */

public void trace(long type, Object obj, String methodName, String text,
                  Object[] parms)
{
String className = obj.getClass().getName();
trace(type, className, methodName, text, parms);
}


/*----------------------------------------------------------------------
 * Name:  trace
 *--------------------------------------------------------------------*/
/**
 * Traces data in a static method.
 *
 * @param type       The type of the trace point.
 * @param className  The name of the traced class.
 * @param methodName The name of the traced method.
 * @param text       The trace text.
 */

public void trace(long type, String className, String methodName, String text)
{
trace(type, className, methodName, text, (Object[]) null);
}


/*----------------------------------------------------------------------
 * Name:  trace
 *--------------------------------------------------------------------*/
/**
 * Traces data in a static method.
 *
 * @param type       The type of the trace point.
 * @param className  The name of the traced class.
 * @param methodName The name of the traced method.
 * @param text       The trace text.
 * @param parm1      An element to be displayed as trace data.
 */

public void trace(long type, String className, String methodName, String text,
                  Object parm1)
{
Object[] parms = {parm1};
trace(type, className, methodName, text, parms);
}


/*----------------------------------------------------------------------
 * Name:  trace
 *--------------------------------------------------------------------*/
/**
 * Traces data in a static method.
 *
 * @param type       The type of the trace point.
 * @param className  The name of the traced class.
 * @param methodName The name of the traced method.
 * @param text       The trace text.
 * @param parm1      An element to be displayed as trace data.
 * @param parm2      An element to be displayed as trace data.
 */

public void trace(long type, String className, String methodName, String text,
                  Object parm1, 
                  Object parm2)
{
Object[] parms = {parm1, parm2};
trace(type, className, methodName, text, parms);
}


/*----------------------------------------------------------------------
 * Name:  trace
 *--------------------------------------------------------------------*/
/**
 * Traces data in a static method.
 *
 * @param type       The type of the trace point.
 * @param className  The name of the traced class.
 * @param methodName The name of the traced method.
 * @param text       The trace text.
 * @param parms      An array of elements to be displayed as trace data.
 */

public void trace(long type, String className, String methodName, String text,
                  Object[] parms)
{
String[] inserts = null;

// Log the trace data if it will pass the masks of the logger and at
// least one handler.

if (isLoggable(type))
  {

  // Convert the Objects to Strings.

  if (parms != null)
    {
    inserts = new String[parms.length];
    for (int j = 0; j < parms.length; j++)
      inserts[j] = parms[j].toString();
    }

  RASTraceEvent event = new RASTraceEvent(type, 
                                          text, inserts, 
                                          className, methodName, 
                                          this,
                                          getServer(), getClient(),
                                          (Exception) null);
  fireRASEvent(event);
  }
}


/*----------------------------------------------------------------------
 * Name:  exception
 *--------------------------------------------------------------------*/
/**
 * Traces an <code>Exception</code> in a non-static method.
 *
 * @param type       The type of the trace point.
 * @param obj        The traced object ("this").
 * @param methodName The name of the traced method.
 * @param exc        An exception.
 */

public void exception(long type, Object obj, String methodName,
                      Exception exc)
{
String className = obj.getClass().getName();
exception(type, className, methodName, exc);
}


/*----------------------------------------------------------------------
 * Name:  exception
 *--------------------------------------------------------------------*/
/**
 * Traces an <code>Exception</code> in a static method.
 *
 * @param type       The type of the trace point.
 * @param className  The name of the traced class.
 * @param methodName The name of the traced method.
 * @param exc        An exception.
 */

public void exception(long type, String className, String methodName,
                      Exception exc)
{

// Log the exception if it will pass the masks of the logger and at
// least one handler.

if (isLoggable(type))
  {
  RASTraceEvent event = new RASTraceEvent(type, 
                                          (String) null, (String[]) null,
                                          className, methodName, 
                                          this,
                                          getServer(), getClient(),
                                          exc);
  fireRASEvent(event);
  }
}


/*----------------------------------------------------------------------
 * Name: maskValueChanged
 *--------------------------------------------------------------------*/
/**
 * Indicates that the value of the handler's trace mask has
 * changed.
 * <p>
 * This method is intended to improve the performance of the 
 * <code>RASLogger.isLoggable</code> method.  When notified of a change
 * in the value of a handler's mask, the logger can update its
 * internal data, which allows the logger to determine if a RAS event
 * will be logged.
 **/

public synchronized void maskValueChanged()
{

// Form the logical OR of the trace masks of each handler.

isLoggableMask = 0;
Enumeration e = getHandlers();
while (e.hasMoreElements())
  {
  RASIHandler h = (RASIHandler) e.nextElement();
  isLoggableMask |= h.getTraceMask();
  }

// AND this with the logger's trace mask.  The resulting value indicates
// which trace types will be processed by the logger and at least one
// handler.

isLoggableMask &= getTraceMask();
}

}
