/*----------------------------------------------------------------------* 
 * IBM Confidential
 * OCO Source Materials
 *
 * (C) Copyright IBM Corporation 1998.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *--------------------------------------------------------------------*/

package com.ibm.ras.mgr;

import java.util.*;
import com.ibm.ras.*;

/**
 * <code>RASGroup</code> organizes the configuration data for a related
 * set of RAS objects (loggers, handlers and formatters).  An application
 * will typically group all of its RAS objects in a RAS group,
 * or set of RAS groups, to distinguish them from other RAS objects
 * controlled by the <code>RASManager</code>.  Data grouped in this
 * manner can be used by a graphical program to display RAS information
 * application-by-application.
 * <p>
 * A RAS group contains collections of the following items:
 * <ul>
 * <li><code>RASIMessageLogger</code> configurations
 * <li><code>RASITraceLogger</code> configurations
 * <li><code>RASIHandler</code> configurations
 * <li><code>RASIFormatter</code> configurations
 * </ul>
 *
 * @see RASManager
 */
 
public class RASGroup extends RASObject 
{
private static final String S = RASCopyright.COPYRIGHT_98;

/**
 * Selects the message logger configurations.
 */

private static final int ML_CFG = 1;

/**
 * Selects the trace logger configurations.
 */

private static final int TL_CFG = 2;

/**
 * Selects the handler configurations.
 */

private static final int H_CFG = 3;

/**
 * Selects the formatter configurations.
 */

private static final int F_CFG = 4;

/**
 * The <code>RASIMessageLogger</code> configurations in this group.
 */

private Hashtable messageLoggerCfgs = new Hashtable();

/**
  * The <code>RASITraceLogger</code> configurations in this group.
  */

private Hashtable traceLoggerCfgs = new Hashtable();

/**
 * The <code>RASIHandler</code> configurations in this group.
 */

private Hashtable handlerCfgs = new Hashtable();

/**
 * The <code>RASIFormatter</code> configurations in this group.
 */

private Hashtable formatterCfgs = new Hashtable();


/*----------------------------------------------------------------------
 * Name:  RASGroup
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASGroup</code>.  The description
 * of this object is an empty string.
 *
 * @param name The name of this object.
 */

public RASGroup(String name) 
{
this(name, "");
}


/*----------------------------------------------------------------------
 * Name:  RASGroup
 *--------------------------------------------------------------------*/
/**
  * Creates a <code>RASGroup</code>.
  *
  * @param name The name of this object.
  * @param desc The description of this object.
  */

public RASGroup(String name, String desc) 
{
super(name, desc);
}


/*----------------------------------------------------------------------
 * Name:  addMessageLoggerConfig
 *--------------------------------------------------------------------*/
/**
  * Adds a message logger configuration to this group.  If the logger
  * configuration does not contain a name, the configuration is not added.
  * If a configuration with the same name is already a member of this
  * group, it is replaced.
  *
  * @param cfg A <code>RASConfig</code> containing the logger 
  *            configuration.
  **/

public void addMessageLoggerConfig(RASConfig cfg)
{
addConfig(cfg, ML_CFG);
}


/*----------------------------------------------------------------------
 * Name:  removeMessageLoggerConfig
 *--------------------------------------------------------------------*/
/**
 * Removes a message logger configuration from this group.
 * If the named configuration does not exist, this method does nothing.
 *
 * @param name The name of the logger configuration.
 **/

public void removeMessageLoggerConfig(String name)
{
removeConfig(name, ML_CFG);
}


/*----------------------------------------------------------------------
 * Name:  getMessageLoggerConfig
 *--------------------------------------------------------------------*/
/**
 * Gets a message Logger configuration from this group.
 *
 * @param name The name of the logger configuration.
 *
 * @return The message logger configuration, or <code>null</code> if
 *         the named configuration does not exist.
 **/

public RASConfig getMessageLoggerConfig(String name)
{
return getConfig(name, ML_CFG);
}


/*----------------------------------------------------------------------
 * Name:  getMessageLoggerConfigs
 *--------------------------------------------------------------------*/
/**
 * Gets all of the message Logger configurations associated with this group.
 *
 * @return A <code>Hashtable</code> of <code>RASConfig</code> objects.
 **/

public Hashtable getMessageLoggerConfigs()
{
return messageLoggerCfgs;
}


/*----------------------------------------------------------------------
 * Name:  addTraceLoggerConfig
 *--------------------------------------------------------------------*/
/**
 * Adds a trace logger configuration to this group.  If the logger
 * configuration does not contain a name, the configuration is not added.
  * If a configuration with the same name is already a member of this
  * group, it is replaced.
 *
 * @param cfg A <code>RASConfig</code> containing the logger 
 *            configuration.
 **/

public void addTraceLoggerConfig(RASConfig cfg)
{
addConfig(cfg, TL_CFG);
}


/*----------------------------------------------------------------------
 * Name:  removeTraceLoggerConfig
 *--------------------------------------------------------------------*/
/**
 * Removes a trace logger configuration from this group.
 * If the named configuration does not exist, this method does nothing.
 *
 * @param name The name of the logger configuration.
 **/

public void removeTraceLoggerConfig(String name)
{
removeConfig(name, TL_CFG);
}


/*----------------------------------------------------------------------
 * Name:  getTraceLoggerConfig
 *--------------------------------------------------------------------*/
/**
 * Gets a trace logger configuration from this group.
 *
 * @param name The name of the logger configuration.
 *
 * @return The logger configuration, or <code>null</code> if
 *         the named configuration does not exist.
 **/

public RASConfig getTraceLoggerConfig(String name)
{
return getConfig(name, TL_CFG);
}


/*----------------------------------------------------------------------
 * Name:  getTraceLoggerConfigs
 *--------------------------------------------------------------------*/
/**
 * Gets all of the trace logger configurations associated with this group.
 *
 * @return A <code>Hashtable</code> of <code>RASConfig</code> objects.
 **/

public Hashtable getTraceLoggerConfigs()
{
return traceLoggerCfgs;
}


/*----------------------------------------------------------------------
 * Name:  addHandlerConfig
 *--------------------------------------------------------------------*/
/**
 * Adds a handler configuration to this group.  If the handler
 * configuration does not contain a name, the configuration is not added.
 * If a configuration with the same name is already a member of this
 * group, it is replaced.
 *
 * @param cfg A <code>RASConfig</code> containing the handler
 *            configuration.
**/

public void addHandlerConfig(RASConfig cfg)
{
addConfig(cfg, H_CFG);
}


/*----------------------------------------------------------------------
 * Name:  removeHandlerConfig
 *--------------------------------------------------------------------*/
/**
 * Removes a handler configuration from this group.
 * If the named configuration does not exist, this method does nothing.
 *
 * @param name The name of the handler configuration.
 **/

public void removeHandlerConfig(String name)
{
removeConfig(name, H_CFG);
}


/*----------------------------------------------------------------------
 * Name:  getHandlerConfig
 *--------------------------------------------------------------------*/
/**
 * Gets a handler configuration from this group.
 *
 * @param name The name of the handler configuration.
 *
 * @return The handler configuration, or <code>null</code> if
 *         the named configuration does not exist.
 **/

public RASConfig getHandlerConfig(String name)
{
return getConfig(name, H_CFG);
}


/*----------------------------------------------------------------------
 * Name:  getHandlerConfigs
 *--------------------------------------------------------------------*/
/**
 * Gets all of the handler configurations associated with this group.
 *
 * @return A <code>Hashtable</code> of <code>RASConfig</code> objects.
 **/

public Hashtable getHandlerConfigs()
{
return handlerCfgs;
}

 
/*----------------------------------------------------------------------
 * Name:  addFormatterConfig
 *--------------------------------------------------------------------*/
/**
 * Adds a formatter configuration to this group.  If the formatter
 * configuration does not contain a name, the configuration is not added.
 * If a configuration with the same name is already a member of this
 * group, it is replaced.
 *
 * @param cfg A <code>RASConfig</code> containing the formatter
 *            configuration.
**/

public void addFormatterConfig(RASConfig cfg)
{
addConfig(cfg, F_CFG);
}


/*----------------------------------------------------------------------
 * Name:  removeFormatterConfig
 *--------------------------------------------------------------------*/
/**
 * Removes a formatter configuration from this group.
 * If the named configuration does not exist, this method does nothing.
 *
 * @param name The name of the formatter configuration.
 **/

public void removeFormatterConfig(String name)
{
removeConfig(name, F_CFG);
}


/*----------------------------------------------------------------------
 * Name:  getFormatterConfig
 *--------------------------------------------------------------------*/
/**
 * Gets a formatter configuration from this group.
 *
 * @param name The name of the formatter configuration.
 *
 * @return The formatter configuration, or <code>null</code> if
 *         the named configuration does not exist.
 **/

public RASConfig getFormatterConfig(String name)
{
return getConfig(name, F_CFG);
}


/*----------------------------------------------------------------------
 * Name:  getFormatterConfigs
 *--------------------------------------------------------------------*/
/**
 * Gets all of the formatter configurations associated with this group.
 *
 * @return A <code>Hashtable</code> of <code>RASConfig</code> objects.
 **/

public Hashtable getFormatterConfigs()
{
return formatterCfgs;
}

 
/*----------------------------------------------------------------------
 * Name:  addConfig
 *--------------------------------------------------------------------*/
/**
 * Adds an object configuration to a group.  If the object
 * configuration does not contain a name, the configuration is not added.
 * If a configuration with the same name is already a member of this
 * group, it is replaced.
 *
 * @param cfg   A <code>RASConfig</code> containing the object
 *              configuration.
 * @param table A value that selects one of the supported tables.
 *              Supported values are <code>ML_CFG, TL_CFG, H_CFG</code>
 *              and <code>F_CFG</code>.
**/

private void addConfig(RASConfig cfg, int table)
{
if (cfg != null)
  {
  String name = (String) cfg.getConfig().get("name");
  if (name != null)
    {
    Hashtable ht = getConfigTable(table);
    ht.put(name, cfg);
    }
  }
}


/*----------------------------------------------------------------------
 * Name:  removeConfig
 *--------------------------------------------------------------------*/
/**
 * Removes an object configuration from a group.
 * If the named configuration does not exist, this method does nothing.
 *
 * @param name  The name of the object configuration.
 * @param table A value that selects one of the supported tables.
 *              Supported values are <code>ML_CFG, TL_CFG, H_CFG</code>
 *              and <code>F_CFG</code>.
 **/

private void removeConfig(String name, int table)
{
if (name != null)
  {
  Hashtable ht = getConfigTable(table);
  ht.remove(name);
  }
}


/*----------------------------------------------------------------------
 * Name:  getConfig
 *--------------------------------------------------------------------*/
/**
 * Gets an object configuration from a group.
 *
 * @param name The name of the object configuration.
 * @param table A value that selects one of the supported tables.
 *              Supported values are <code>ML_CFG, TL_CFG, H_CFG</code>
 *              and <code>F_CFG</code>.
 *
 * @return The object configuration, or <code>null</code> if
 *         the named configuration does not exist.
 **/

private RASConfig getConfig(String name, int table)
{
RASConfig cfg = null;
if (name != null)
  {
  Hashtable ht = getConfigTable(table);
  cfg = (RASConfig) ht.get(name);
  }

return cfg;
}


/*----------------------------------------------------------------------
 * Name:  getConfigTable
 *--------------------------------------------------------------------*/
/**
 * Selects one of the supported set of configuration tables.
 *
 * @param table A value that selects one of the supported tables.
 *              Supported values are <code>ML_CFG, TL_CFG, H_CFG</code>
 *              and <code>F_CFG</code>.
 *
 * @return A <code>Hashtable</code> of <code>RASConfig</code> objects.
 **/

private Hashtable getConfigTable(int table)
{
Hashtable ht = null;

switch(table)
  {
  case ML_CFG:
    ht = messageLoggerCfgs;
    break;
  case TL_CFG:
    ht = traceLoggerCfgs;
    break;
  case H_CFG:
    ht = handlerCfgs;
    break;
  case F_CFG:
    ht = formatterCfgs;
    break;
  }

return ht;
}

}
