/*----------------------------------------------------------------------
 * IBM Confidential
 * OCO Source Materials
 *
 * (C) Copyright IBM Corporation 1998.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *--------------------------------------------------------------------*/

package com.ibm.ras.mgr;

import java.util.*;
import com.ibm.ras.*;

/**
 * <code>RASManager</code> is a manager of RAS objects.  It is an 
 * additional feature to the RAS Toolkit.  That is, an application 
 * can use the RAS Tookit without a RAS Manager by creating
 * all of the required toolkit objects in an application.  This may be 
 * suitable for "smaller" applications in which the trace state and output
 * device can be hard-coded, or passed to the application when it starts.
 * In more complex environments, an application can be written to use a
 * RAS Manager to acquire its logging configuration.  This generally
 * lightens the load on the application programmer.
 * <p>
 * The RAS Manager can also provide RAS configuration information to a 
 * graphical program, which would allow an end-user or administrator
 * to reconfigure the RAS system -- turning specific loggers on or off,
 * for example.
 * <p>
 * The RAS Manager deals primarily in configuration data.  That is, it
 * must be told which RAS objects it can create, what their options are
 * and how they are connected.  When an object, such as a message logger,
 * is requested by an application, the RAS Manager creates it from its
 * configuration data and returns the object.  Only one instance of a
 * RAS object is created.  This allows different applications, or 
 * components of a single application, to share RAS objects.
 * <p>
 * In some environments, a RAS Manager may be able to get its
 * initial configuration from persistent storage (such as a properties 
 * file or an LDAP database) and, possibly, save it there as well.  This
 * medium is represented by the <code>RASIDataStore</code> interface, whose
 * <code>restoreConfig</code> and <code>saveConfig</code> methods
 * provide this function.  Two examples of a <code>RASIDataStore</code> are
 * provided in this package:
 * <ul>
 * <li><code>RASDefaultDataStore</code> assumes that no persistent store
 * is available.  The <code>restoreConfig</code> and 
 * <code>saveConfig</code> methods do nothing.
 * <li><code>RASPropertyDataStore</code> gets its configuration from a 
 * properties file, whose format is specified by that class.
 * </ul>
 * If a persistent store is not available, the RAS Manager is generally
 * not interesting.  The configuration can be built by creating one or
 * more <code>RASGroups</code> and passing this to the RAS Manager
 * through the <code>setConfig</code> method.  However, this is more work
 * than managing the RAS objects in the application, itself.
 * <p>
 * The RAS Manager configuration is organized as one or more "groups."
 * Groups are a way for an application to distinguish its loggers and
 * handlers from others that may be controlled by the RAS Manager.
 * It is recommended that each application organize its loggers into one
 * or more RAS groups.  Handlers are usually considered to be 
 * system-wide resources and are, therfore, typically contained in the 
 * "base group."
 * <p>
 * This base group can also contain other groups.  The RAS Manager thus
 * implements a "flat tree" of groups.  That is, groups may <I>only</I>
 * be added to the base group.  An application can impose a logical 
 * structure of groups within groups by carefully selecting group names.
 * One way to do this is to use a naming scheme similar to Java package
 * names.  This would be very useful to a graphical problem that wished
 * to display RAS configurations to an end-user.
 * <p>
 * An application normally only uses the <code>getMessageLogger</code>
 * and <code>getTraceLogger</code> methods.  The RAS Manager will attach
 * to the logger any handlers which are included in the logger's
 * configuration and any formatters which are included in the handler's
 * configuration.  The RAS Manager will search the group containing the
 * logger and then the base group for the handler configuration
 * information.  If no handlers are included in the logger's
 * configuration, the RAS Manager will use the set of default handlers
 * instead.  Formatter configurations are searched for in the same manner
 * as handler configurations.
 *
 * @see RASIDataStore
 * @see RASGroup
 * @see RASBaseGroup
 */

public class RASManager
{
private static final String S = RASCopyright.COPYRIGHT_98;

/**
 * The <code>RASIDataStore</code>, a persistent store for the RAS
 * configuration.
 */

private static RASIDataStore dataStore = null;

/**
 * The <code>RASBaseGroup</code> containing the RAS configuration.
 */

private static RASBaseGroup baseGroup = new RASBaseGroup();

/**
 * The one and only RASManager.
 */

private static RASManager mgr = null;


/*----------------------------------------------------------------------
 * Name:  getManager
 *--------------------------------------------------------------------*/
/**
 * Gets the <code>RASManager</code>.  If the RAS Manager has already been
 * created through a previous call to <code>getManager</code>, that 
 * manager is returned.  Otherwise, a new manager is created and returned.
 * <p>
 * A manager created with this method has no data store from which it
 * can read a configuration.  The <code>addDataStore</code> method can 
 * be used to add a data store to an existing manager.
 *
 * @return The RAS manager.
 */

public static synchronized RASManager getManager()
{
getManager((RASIDataStore) null);

return mgr;
}


/*----------------------------------------------------------------------
 * Name:  getManager
 *--------------------------------------------------------------------*/
/**
 * Gets the <code>RASManager</code>.  If the RAS Manager has already been
 * created through a previous call to <code>getManager</code>, that 
 * manager is returned.  Otherwise, a new manager is created and returned.
 *
 * @param store An object which implements the <code>RASIDataStore</code>
 *              interface.  
 *
 * @return The RAS manager.
 */

public static synchronized RASManager getManager(RASIDataStore store) 
{
if (mgr == null)
  mgr = new RASManager(store);

return mgr;
}


/*----------------------------------------------------------------------
 * Name:  RASManager
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASManager</code>.  This method is private to 
 * force applications to use the factory method <code>getManager</code>.
 *
 * @param store An object which implements the <code>RASIDataStore</code>
 *              interface.  
 */

private RASManager(RASIDataStore store) 
{
dataStore = store;
restoreConfig();
}


/*----------------------------------------------------------------------
 * Name:  addDataStore
 *--------------------------------------------------------------------*/
/**
 * Adds a <code>RASIDataStore</code> to this manager.
 * <p>
 * (In this implementation, only one data store is supported.  A new 
 * data store simply replaces the current store.)
 *
 * @param store An object which implements the <code>RASIDataStore</code>
 *              interface.  
 **/

public synchronized void addDataStore(RASIDataStore store)
{
if (store != null)
  dataStore = store;
}


/*----------------------------------------------------------------------
 * Name:  removeDataStore
 *--------------------------------------------------------------------*/
/**
 * Removes a <code>RASIDataStore</code> from this manager.
 * <p>
 * (In this implementation, only one data store is supported.  If the 
 * requested store is in use, it is removed.  Otherwise, nothing is changed.)
 *
 * @param store An object which implements the <code>RASIDataStore</code>
 *              interface.  
 **/

public synchronized void removeDataStore(RASIDataStore store)
{
if (store != null && dataStore.equals(store))
  dataStore = null;
}


/*----------------------------------------------------------------------
 * Name:  getDataStores
 *--------------------------------------------------------------------*/
/**
 * Gets the data store objects in use by this manager.
 * <p>
 * (In this implementation, only one data store is supported.  It is  
 * returned in a <code>Vector</code> to support future enhancements.)
 *
 * @return An <code>Enumeration</code> of data stores.
 **/

public synchronized Enumeration getDataStores()
{
Vector stores = new Vector(1);
if (dataStore != null)
  stores.addElement(dataStore);
return stores.elements();
}


/*----------------------------------------------------------------------
 * Name:  getMessageLogger
 *--------------------------------------------------------------------*/
/**
 * Gets a message logger from the base group.
 * <p>
 * If the logger configuration includes handlers, they will 
 * automatically be attached to the logger.  It should not be necessary
 * for an application to use the <code>getHandler</code> method.
 *
 * @param name The name of the message logger.
 *
 * @return A message logger or <code>null</code> if the requested 
 *         logger was not found.
 */

public synchronized RASIMessageLogger getMessageLogger(String name)
{
return getMessageLogger(name, baseGroup); 
}


/*----------------------------------------------------------------------
 * Name:  getMessageLogger
 *--------------------------------------------------------------------*/
/**
 * Gets a message logger from the named group.
 * <p>
 * If the logger configuration includes handlers, they will 
 * automatically be attached to the logger.  It should not be necessary
 * for an application to use the <code>getHandler</code> method.
 *
 * @param name      The name of the message logger.
 * @param groupName The name of the <code>RASGroup</code>.
 *
 * @return A message logger or <code>null</code> if the requested 
 *         logger was not found.
 */

public synchronized RASIMessageLogger getMessageLogger(String name, String groupName)
{
RASIMessageLogger logger = null;

RASGroup group = baseGroup.getGroup(groupName);
if (group != null)
  logger = getMessageLogger(name, group);

return logger;
}


/*----------------------------------------------------------------------
 * Name:  getMessageLogger
 *--------------------------------------------------------------------*/
/**
 * Gets a message logger from a group.
 * <p>
 * If the logger configuration includes handlers, they will 
 * automatically be attached to the logger.  It should not be necessary
 * for an application to use the <code>getHandler</code> method.
 *
 * @param name  The name of the message logger.
 * @param group The <code>RASGroup</code>.
 *
 * @return A message logger or <code>null</code> if the requested 
 *         logger was not found.
 */

public synchronized RASIMessageLogger getMessageLogger(String name, RASGroup group)
{
RASIMessageLogger logger = null;

RASConfig cfg = group.getMessageLoggerConfig(name);
if (cfg != null)
  {
  logger = (RASIMessageLogger) getObject(cfg);
  addHandlers(logger, cfg, group);
  }

return logger;
}


/*----------------------------------------------------------------------
 * Name:  returnMessageLogger
 *--------------------------------------------------------------------*/
/**
 * Returns a message logger to the base group.
 * <p>
 * If the logger configuration includes handlers, they will 
 * automatically be detached from the logger.  It should not be necessary
 * for an application to use the <code>returnHandler</code> method.
 *
 * @param logger The logger being returned.
 */

public synchronized void returnMessageLogger(RASIMessageLogger logger)
{
returnMessageLogger(logger, baseGroup); 
}


/*----------------------------------------------------------------------
 * Name:  returnMessageLogger
 *--------------------------------------------------------------------*/
/**
 * Returns a message logger to the named group.
 * <p>
 * If the logger configuration includes handlers, they will 
 * automatically be detached from the logger.  It should not be necessary
 * for an application to use the <code>returnHandler</code> method.
 *
 * @param logger    The logger being returned.
 * @param groupName The name of the <code>RASGroup</code>.
 */

public synchronized void returnMessageLogger(RASIMessageLogger logger, String groupName)
{
RASGroup group = baseGroup.getGroup(groupName);
if (group != null)
  returnMessageLogger(logger, group);
}


/*----------------------------------------------------------------------
 * Name:  returnMessageLogger
 *--------------------------------------------------------------------*/
/**
 * Returns a message logger to a group.
 * <p>
 * If the logger configuration includes handlers, they will 
 * automatically be detached from the logger.  It should not be necessary
 * for an application to use the <code>returnHandler</code> method.
 *
 * @param logger   The logger being returned.
 * @param group    The <code>RASGroup</code>.
 */

public synchronized void returnMessageLogger(RASIMessageLogger logger, RASGroup group)
{
RASConfig cfg = group.getMessageLoggerConfig(logger.getName());
if (cfg != null)
  {

  // Decrement the in-use count for the logger.  If it is zero, remove the
  // handlers from the logger and get rid of the logger.

  if (cfg.decrementInUseCount() == 0)
    {
    removeHandlers(logger, group);
    logger = null;
    }
  }
}


/*----------------------------------------------------------------------
 * Name:  getTraceLogger
 *--------------------------------------------------------------------*/
/**
 * Gets a trace logger from the base group.
 * <p>
 * If the logger configuration includes handlers, they will 
 * automatically be attached to the logger.  It should not be necessary
 * for an application to use the <code>getHandler</code> method.
 *
 * @param name The name of the trace logger.
 *
 * @return A trace logger or <code>null</code> if the requested 
 *         logger was not found.
 */

public synchronized RASITraceLogger getTraceLogger(String name)
{
return getTraceLogger(name, baseGroup); 
}


/*----------------------------------------------------------------------
 * Name:  getTraceLogger
 *--------------------------------------------------------------------*/
/**
 * Gets a trace logger from the named group.
 * <p>
 * If the logger configuration includes handlers, they will 
 * automatically be attached to the logger.  It should not be necessary
 * for an application to use the <code>getHandler</code> method.
 *
 * @param name      The name of the trace logger.
 * @param groupName The name of the <code>RASGroup</code>.
 *
 * @return A trace logger or <code>null</code> if the requested 
 *         logger was not found.
 */

public synchronized RASITraceLogger getTraceLogger(String name, String groupName)
{
RASITraceLogger logger = null;

RASGroup group = baseGroup.getGroup(groupName);
if (group != null)
  logger = getTraceLogger(name, group);

return logger;
}


/*----------------------------------------------------------------------
 * Name:  getTraceLogger
 *--------------------------------------------------------------------*/
/**
 * Gets a trace logger from a group.
 * <p>
 * If the logger configuration includes handlers, they will 
 * automatically be attached to the logger.  It should not be necessary
 * for an application to use the <code>getHandler</code> method.
 *
 * @param name  The name of the trace logger.
 * @param group The <code>RASGroup</code>.
 *
 * @return A trace logger or <code>null</code> if the requested 
 *         logger was not found.
 */

public synchronized RASITraceLogger getTraceLogger(String name, RASGroup group)
{
RASITraceLogger logger = null;

RASConfig cfg = group.getTraceLoggerConfig(name);
if (cfg != null)
  {
  logger = (RASITraceLogger) getObject(cfg);
  addHandlers(logger, cfg, group);
  }

return logger;
}


/*----------------------------------------------------------------------
 * Name:  returnTraceLogger
 *--------------------------------------------------------------------*/
/**
 * Returns a trace logger to the base group.
 * <p>
 * If the logger configuration includes handlers, they will 
 * automatically be detached from the logger.  It should not be necessary
 * for an application to use the <code>returnHandler</code> method.
 *
 * @param logger The logger being returned.
 */

public synchronized void returnTraceLogger(RASITraceLogger logger)
{
returnTraceLogger(logger, baseGroup); 
}


/*----------------------------------------------------------------------
 * Name:  returnTraceLogger
 *--------------------------------------------------------------------*/
/**
 * Returns a trace logger to the named group.
 * <p>
 * If the logger configuration includes handlers, they will 
 * automatically be detached from the logger.  It should not be necessary
 * for an application to use the <code>returnHandler</code> method.
 *
 * @param logger    The logger being returned.
 * @param groupName The name of the <code>RASGroup</code>.
 */

public synchronized void returnTraceLogger(RASITraceLogger logger, String groupName)
{
RASGroup group = baseGroup.getGroup(groupName);
if (group != null)
  returnTraceLogger(logger, group);
}


/*----------------------------------------------------------------------
 * Name:  returnTraceLogger
 *--------------------------------------------------------------------*/
/**
 * Returns a trace logger to a group.
 * <p>
 * If the logger configuration includes handlers, they will 
 * automatically be detached from the logger.  It should not be necessary
 * for an application to use the <code>returnHandler</code> method.
 *
 * @param logger   The logger being returned.
 * @param group    The <code>RASGroup</code>.
 */

public synchronized void returnTraceLogger(RASITraceLogger logger, RASGroup group)
{
RASConfig cfg = group.getTraceLoggerConfig(logger.getName());
if (cfg != null)
  {

  // Decrement the in-use count for the logger.  If it is zero, remove the handlers
  // from the logger and get rid of the logger.

  if (cfg.decrementInUseCount() == 0)
    {
    removeHandlers(logger, group);
    logger = null;
    }
  }
}


/*----------------------------------------------------------------------
 * Name:  getHandler
 *--------------------------------------------------------------------*/
/**
 * Gets a handler from the base group.
 * <p>
 * If the logger configuration includes handlers, they will 
 * automatically be attached to the logger.  It should not be necessary
 * for an application to use the <code>getHandler</code> method.
 *
 * @param name The name of the handler.
 *
 * @return A handler or <code>null</code> if the requested 
 *         handler was not found.
 */
 
public synchronized RASIHandler getHandler(String name)
{
return getHandler(name, baseGroup); 
}


/*----------------------------------------------------------------------
 * Name:  getHandler
 *--------------------------------------------------------------------*/
/**
 * Gets a handler from the named group.
 * <p>
 * If the logger configuration includes handlers, they will 
 * automatically be attached to the logger.  It should not be necessary
 * for an application to use the <code>getHandler</code> method.
 *
 * @param name      The name of the handler.
 * @param groupName The name of the <code>RASGroup</code>.
 *
 * @return A handler or <code>null</code> if the requested 
 *         handler was not found.
 */

public synchronized RASIHandler getHandler(String name, String groupName)
{
RASIHandler handler = null;

RASGroup group = baseGroup.getGroup(groupName);
if (group != null)
  handler = getHandler(name, group);

return handler;
}


/*----------------------------------------------------------------------
 * Name:  getHandler
 *--------------------------------------------------------------------*/
/**
 * Gets a handler from a group.
 * <p>
 * If the logger configuration includes handlers, they will 
 * automatically be attached to the logger.  It should not be necessary
 * for an application to use the <code>getHandler</code> method.
 *
 * @param name  The name of the handler.
 * @param group The <code>RASGroup</code>.
 *
 * @return A handler or <code>null</code> if the requested 
 *         handler was not found.
 */

public synchronized RASIHandler getHandler(String name, RASGroup group)
{
RASIHandler handler = null;

RASConfig cfg = group.getHandlerConfig(name);
if (cfg != null)
  handler = (RASIHandler) getObject(cfg);

return handler;
}


/*----------------------------------------------------------------------
 * Name:  returnHandler
 *--------------------------------------------------------------------*/
/**
 * Returns a handler to the base group.
 * <p>
 * If the logger configuration includes handlers, they will 
 * automatically be detached from the logger.  It should not be necessary
 * for an application to use the <code>returnHandler</code> method.
 *
 * @param handler The handler being returned.
 */

public synchronized void returnHandler(RASIHandler handler)
{
returnHandler(handler, baseGroup); 
}


/*----------------------------------------------------------------------
 * Name:  returnHandler
 *--------------------------------------------------------------------*/
/**
 * Returns a handler to the named group.
 * <p>
 * If the logger configuration includes handlers, they will 
 * automatically be detached from the logger.  It should not be necessary
 * for an application to use the <code>returnHandler</code> method.
 *
 * @param handler   The handler being returned.
 * @param groupName The name of the <code>RASGroup</code>.
 */

public synchronized void returnHandler(RASIHandler handler, String groupName)
{
RASGroup group = baseGroup.getGroup(groupName);
if (group != null)
  returnHandler(handler, group);
}


/*----------------------------------------------------------------------
 * Name:  returnHandler
 *--------------------------------------------------------------------*/
/**
 * Returns a handler to a group.
 * <p>
 * If the logger configuration includes handlers, they will 
 * automatically be detached from the logger.  It should not be necessary
 * for an application to use the <code>returnHandler</code> method.
 *
 * @param handler  The handler being returned.
 * @param group    The <code>RASGroup</code>.
 */

public synchronized void returnHandler(RASIHandler handler, RASGroup group)
{

// First search the specified group for the handler.  If the handler is 
// not found, try the base group.

String handlerName = handler.getName();
RASConfig cfg = group.getHandlerConfig(handlerName);
if (cfg == null)
  cfg = baseGroup.getHandlerConfig(handlerName);

if (cfg != null)
  {

  // Decrement the in-use count for the handler.  If it is zero, get rid of
  // the handler.

  if (cfg.decrementInUseCount() == 0)
    {
    handler.stop();
    handler = null;
    }
  }
}


/*----------------------------------------------------------------------
 * Name:  getFormatter
 *--------------------------------------------------------------------*/
/**
 * Gets a formatter from the base group.
 * <p>
 * If the handler configuration includes formatters, they will 
 * automatically be attached to the handler.  It should not be necessary
 * for an application to use the <code>getFormatter</code> method.
 *
 * @param name The name of the formatter.
 *
 * @return A formatter or <code>null</code> if the requested 
 *         formatter was not found.
 */
 
public synchronized RASIFormatter getFormatter(String name)
{
return getFormatter(name, baseGroup); 
}


/*----------------------------------------------------------------------
 * Name:  getFormatter
 *--------------------------------------------------------------------*/
/**
 * Gets a formatter from the named group.
 * <p>
 * If the handler configuration includes formatters, they will 
 * automatically be attached to the handler.  It should not be necessary
 * for an application to use the <code>getFormatter</code> method.
 *
 * @param name      The name of the formatter.
 * @param groupName The name of the <code>RASGroup</code>.
 *
 * @return A formatter or <code>null</code> if the requested 
 *         formatter was not found.
 */

public synchronized RASIFormatter getFormatter(String name, String groupName)
{
RASIFormatter formatter = null;

RASGroup group = baseGroup.getGroup(groupName);
if (group != null)
  formatter = getFormatter(name, group);

return formatter;
}


/*----------------------------------------------------------------------
 * Name:  getFormatter
 *--------------------------------------------------------------------*/
/**
 * Gets a formatter from a group.
 * <p>
 * If the handler configuration includes formatters, they will 
 * automatically be attached to the handler.  It should not be necessary
 * for an application to use the <code>getFormatter</code> method.
 *
 * @param name  The name of the formatter.
 * @param group The <code>RASGroup</code>.
 *
 * @return A formatter or <code>null</code> if the requested 
 *         formatter was not found.
 */

public synchronized RASIFormatter getFormatter(String name, RASGroup group)
{
RASIFormatter formatter = null;

RASConfig cfg = group.getFormatterConfig(name);
if (cfg != null)
  formatter = (RASIFormatter) getObject(cfg);

return formatter;
}


/*----------------------------------------------------------------------
 * Name:  returnFormatter
 *--------------------------------------------------------------------*/
/**
 * Returns a formatter to the base group.
 * <p>
 * If the handler configuration includes formatters, they will 
 * automatically be detached from the handler.  It should not be necessary
 * for an application to use the <code>returnFormatter</code> method.
 *
 * @param formatter The formatter being returned.
 */

public synchronized void returnFormatter(RASIFormatter formatter)
{
returnFormatter(formatter, baseGroup); 
}


/*----------------------------------------------------------------------
 * Name:  returnFormatter
 *--------------------------------------------------------------------*/
/**
 * Returns a formatter to the named group.
 * <p>
 * If the handler configuration includes formatters, they will 
 * automatically be detached from the handler.  It should not be necessary
 * for an application to use the <code>returnFormatter</code> method.
 *
 * @param formatter The formatter being returned.
 * @param groupName The name of the <code>RASGroup</code>.
 */

public synchronized void returnFormatter(RASIFormatter formatter, String groupName)
{
RASGroup group = baseGroup.getGroup(groupName);
if (group != null)
  returnFormatter(formatter, group);
}


/*----------------------------------------------------------------------
 * Name:  returnFormatter
 *--------------------------------------------------------------------*/
/**
 * Returns a formatter to a group.
 * <p>
 * If the handler configuration includes formatters, they will 
 * automatically be detached from the handler.  It should not be necessary
 * for an application to use the <code>returnFormatter</code> method.
 *
 * @param formatter The formatter being returned.
 * @param group     The <code>RASGroup</code>.
 */

public synchronized void returnFormatter(RASIFormatter formatter, RASGroup group)
{

// First search the specified group for the formatter.  If the formatter is 
// not found, try the base group.

String formatterName = formatter.getName();
RASConfig cfg = group.getFormatterConfig(formatterName);
if (cfg == null)
  cfg = baseGroup.getFormatterConfig(formatterName);

if (cfg != null)
  {

  // Decrement the in-use count for the formatter.  If it is zero, get rid of
  // the formatter.

  if (cfg.decrementInUseCount() == 0)
    {
    formatter = null;
    }
  }
}


/*----------------------------------------------------------------------
 * Name:  getBaseGroup
 *--------------------------------------------------------------------*/
/**
 * Gets the entire RAS configuration.
 *
 * @return The base RAS group containing the entire configuration.
 */

public synchronized RASBaseGroup getBaseGroup()
{
return baseGroup;
}


/*----------------------------------------------------------------------
 * Name:  setBaseGroup
 *--------------------------------------------------------------------*/
/**
 * Sets the entire RAS configuration.
 *
 * @param baseGroup The base RAS group containing the entire configuration.
 */

public synchronized void setBaseGroup(RASBaseGroup baseGroup)
{
this.baseGroup = baseGroup;
}


/*----------------------------------------------------------------------
 * Name:  restoreConfig
 *--------------------------------------------------------------------*/
/**
 * Restores the <code>RASManager</code> configuration from persistent
 * storage.  If the RAS Manager has not been given a 
 * <code>RASIDataStore</code> to use, this method does nothing.
 */

public synchronized void restoreConfig()
{
if (dataStore != null)
  dataStore.restoreConfig(baseGroup);
}


/*----------------------------------------------------------------------
 * Name:  saveConfig
 *--------------------------------------------------------------------*/
/**
 * Saves the <code>RASManager</code> configuration to persistent
 * storage.  If the RAS Manager has not been given a 
 * <code>RASIDataStore</code> to use, this method does nothing.
 */

public synchronized void saveConfig()
{
if (dataStore != null)
  dataStore.saveConfig(baseGroup);
}


/*----------------------------------------------------------------------
 * Name:  getObject
 *--------------------------------------------------------------------*/
/**
 * Gets a <code>RASObject</code> from its configuration.  If the object
 * has been created already, it is returned.  If not, a new object is
 * instantiated and returned.
 *
 * @param cfg The configuration of the object.
 *
 * @return A RAS object or <code>null</code> if the requested 
 *         object was not found.
 */

protected RASObject getObject(RASConfig cfg)
{
Class objClass = null;
Hashtable configData = cfg.getConfig();
RASObject object = null;

if (cfg != null)
  {

  // If the object has been created, it is saved in the configuration.
  // Return this.

  object = cfg.getObject();

  // The object has not been created.  Try to create a new instance
  // from the class name in the configuration.  If this fails, we
  // just return null.

  if (object == null)
    object = (RASObject) RASUtil.createObject((String) configData.get("className"));

  if (object != null)
    {
    object.setConfig(configData);
    cfg.setObject(object);
    }
  }

if (object != null)
  cfg.incrementInUseCount();

return object;
}


/*----------------------------------------------------------------------
 * Name:  addHandlers
 *--------------------------------------------------------------------*/
/**
 * Add handlers to a newly-created logger, according to the logger's
 * configuration.
 *
 * @param logger The RAS logger.
 * @param cfg    The RAS logger configuration.
 * @param group  The <code>RASGroup</code> in which the logger configuration
 *               was found.
 */

protected void addHandlers(RASILogger logger, RASConfig cfg, RASGroup group)
{
if (logger != null)
  {
  RASIHandler handler;
  String handlerNames = (String) cfg.getConfig().get("handlerNames");

  // If the logger's configuration does not include handlers, try the
  // list of default handlers.

  if (handlerNames == null)
    handlerNames = baseGroup.getDefaultHandlers();

  // If the logger's configuration includes handlers, add them.

  if (handlerNames != null)
    {
    StringTokenizer st = new StringTokenizer(handlerNames);
    while (st.hasMoreTokens())
      {
      String name = st.nextToken();

      // First search the group containing the logger for the handler.
      // If the handler is not found, try the base group.

      handler = getHandler(name, group);
      if (handler == null)
        {
        group = baseGroup;
        handler = getHandler(name, group);
        }

      if (handler != null)
        {
        RASConfig handlerCfg = group.getHandlerConfig(name);
        addFormatters(handler, handlerCfg, group); 
        logger.addHandler(handler);
        }
      }
    }
  }
}


/*----------------------------------------------------------------------
 * Name:  removeHandlers
 *--------------------------------------------------------------------*/
/**
 * Removes handlers from a logger.
 *
 * @param logger The RAS logger.
 * @param group  The <code>RASGroup</code> in which the logger
 *               configuration was found.
 */

protected void removeHandlers(RASILogger logger, RASGroup group)
{
Enumeration e = logger.getHandlers();

while (e.hasMoreElements())
  {
  RASIHandler handler = (RASIHandler) e.nextElement();
  returnHandler(handler, group);
  }
}


/*----------------------------------------------------------------------
 * Name:  addFormatters
 *--------------------------------------------------------------------*/
/**
 * Add formatters to a newly-created handler, according to the handler's
 * configuration.
 *
 * @param handler The RAS handler.
 * @param cfg     The RAS handler configuration.
 * @param group   The <code>RASGroup</code> in which the handler configuration
 *                was found.
 */

protected void addFormatters(RASIHandler handler, RASConfig cfg, RASGroup group)
{
if (handler != null)
  {
  RASIFormatter formatter;
  String formatterNames = (String) cfg.getConfig().get("formatterNames");

  // If the handler's configuration includes formatters, add them.

  if (formatterNames != null)
    {
    StringTokenizer st = new StringTokenizer(formatterNames);
    while (st.hasMoreTokens())
      {
      String name = st.nextToken();

      // First search the group containing the handler for the formatter.
      // If the formatter is not found, try the base group.

      formatter = getFormatter(name, group);
      if (formatter == null)
        formatter = getFormatter(name, baseGroup);

      if (formatter != null)
        handler.addFormatter(formatter);
      }
    }
  }
}


/*----------------------------------------------------------------------
 * Name:  removeFormatters
 *--------------------------------------------------------------------*/
/**
 * Removes formatters from a handler.
 *
 * @param handler The RAS handler.
 * @param group   The <code>RASGroup</code> in which the handler
 *                configuration was found.
 */

protected void removeFormatters(RASIHandler handler, RASGroup group)
{
Enumeration e = handler.getFormatters();

while (e.hasMoreElements())
  {
  RASIFormatter formatter = (RASIFormatter) e.nextElement();
  returnFormatter(formatter, group);
  }
}

}
