/*----------------------------------------------------------------------
 * IBM Confidential
 * OCO Source Materials
 *
 * (C) Copyright IBM Corporation 1998.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *--------------------------------------------------------------------*/

package com.ibm.ras.mgr;

import java.io.*;
import java.util.*;
import com.ibm.ras.*;

/**
 * <code>RASPropertyDataStore</code> is an implementation of the
 * <code>RASIDataStore</code> interface which uses a properties file of
 * a specific format to contain the RAS configuration.  The file
 * <code>com.ibm.ras.samples.RASSampleDataStore.properties</code>
 * describes this format.
 *
 * @see RASIDataStore
 */

public class RASPropertyDataStore implements RASIDataStore
{
private static final String S = RASCopyright.COPYRIGHT_98;

/**
 * The persistent data store.
 */

private Properties props;


/*----------------------------------------------------------------------
 * Name:  RASPropertyDataStore
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASPropertyDataStore</code>.
 *
 * @param props The persistent store, a <code>Properties</code> file.
 */

public RASPropertyDataStore(Properties props)
{
setPropertyDataStore(props);
}


/*----------------------------------------------------------------------
 * Name:  RASPropertyDataStore
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASPropertyDataStore</code>.
 *
 * @param name The name of the properties file containing the
 *             configuration.  The properties file is loaded through the
 *             method <code>Class.getResourceAsStream</code>.  The name
 *             must conform to the conventions of this method.
 *
 * @exception IOException
 *   An <code>IOException</code> is thrown if the file cannot be opened.
 */

public RASPropertyDataStore(String name) throws IOException 
{
setPropertyDataStore(new Properties());
props.load(getClass().getResourceAsStream(name));
}


/*----------------------------------------------------------------------
 * Name:  getPropertyDataStore
 *--------------------------------------------------------------------*/
/**
 * Gets the properties file used as the persistent data store.
 *
 * @return The properties file
 **/

public Properties getPropertyDataStore()
{
return props;
}


/*----------------------------------------------------------------------
 * Name:  setPropertyDataStore
 *--------------------------------------------------------------------*/
/**
 * Sets the properties file used as the persistent data store.
 *
 * @param props The properties file
 **/

public void setPropertyDataStore(Properties props)
{
this.props = props;
}


/*----------------------------------------------------------------------
 * Name:  restoreConfig
 *--------------------------------------------------------------------*/
/**
 * Restores the <code>RASManager</code> configuration from persistent
 * storage.  This method is called automatically when the 
 * <code>RASManager</code> is created and can be called anytime the
 * stored configuration should be reloaded.
 *
 * @param baseGroup The <code>RASBaseGroup</code> which holds the 
 *                  configuration loaded from persistent store.
 */

public void restoreConfig(RASBaseGroup baseGroup)
{

// Restore the configuration.

addRASObjects("msgLoggers", baseGroup);
addRASObjects("trcLoggers", baseGroup);
addRASObjects("handlers", baseGroup);
addRASObjects("formatters", baseGroup);

// Get the list of default handlers, if one was specified.

String list = props.getProperty("defaultHandlerNames");
if (list != null)
  baseGroup.setDefaultHandlers(list);
}


/*----------------------------------------------------------------------
 * Name:  saveConfig
 *--------------------------------------------------------------------*/
/**
 * Saves the <code>RASManager</code> configuration to persistent
 * storage.
 * <p>
 * This particular implementation assumes that the persistent store is
 * static -- it cannot be updated.  This method does nothing.
 *
 * @param baseGroup The <code>RASBaseGroup</code> which holds the 
 *                  configuration to be saved in persistent store.
 */

public void saveConfig(RASBaseGroup baseGroup)
{
}


/*----------------------------------------------------------------------
 * Name:  addRASObjects
 *--------------------------------------------------------------------*/
/**
 * Adds a group of RAS objects to a <code>RASGroup</code>, as specified
 * in a properties file (the persistent store).
 *
 * @param type      A <code>String</code> which indicates whether to 
 *                  add message loggers, trace loggers or handlers.
 * @param baseGroup The <code>RASBaseGroup</code> which holds the 
 *                  configuration loaded from persistent store.
 */

private void addRASObjects(String type, RASBaseGroup baseGroup)
{
RASGroup group;

// Read the persistent store for one of the basic keys:  "msgLoggers," 
// "trcLoggers," "handlers" or "formatters."

String list = props.getProperty(type);
if (list != null)
  {
  StringTokenizer st = new StringTokenizer(list);

  // Each token in the list points to another properties file containing
  // the configuration of some RAS object.  Get that file name.

  while (st.hasMoreElements())
    {
    String propFileName = props.getProperty(st.nextToken());
    if (propFileName != null)
      {

      // Change the '.' in the properties file name to the file separator
      // character and tack on the ".properties" extension.  Load the file.

      propFileName = "/" + propFileName.replace('.', File.separatorChar) + ".properties";
      Properties propFile = new Properties();
      try
        {
        propFile.load(getClass().getResourceAsStream(propFileName));

        // If the properties file does not have a "group" key, place the
        // object in the base group.

        String groupName = propFile.getProperty("group");
        if (groupName == null)
          {
          group = baseGroup;
          }

        // The object goes in its own group.  Create it, if it does not
        // already exist.

        else
          {
          group = baseGroup.getGroup(groupName);
          if (group == null);
            {
            group = new RASGroup(groupName);
            baseGroup.addGroup(group);
            }
          }

        // Add the new configuration to its group.

        if (type.equals("msgLoggers"))
          group.addMessageLoggerConfig(new RASConfig(propFile));
        else if (type.equals("trcLoggers"))
          group.addTraceLoggerConfig(new RASConfig(propFile));
        else if (type.equals("handlers"))
          group.addHandlerConfig(new RASConfig(propFile));
        else if (type.equals("formatters"))
          group.addFormatterConfig(new RASConfig(propFile));
        }
      catch (FileNotFoundException fnfE)
        {
        }
      catch (IOException ioE)
        {
        }
      }
    }
  }
}


}
