/*----------------------------------------------------------------------
 * (C) Copyright IBM Corporation 1998.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *--------------------------------------------------------------------*/

package com.ibm.ras.samples;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;

import com.ibm.ras.*;
import com.ibm.ras.mgr.*;

/**
 * This sample program demonstrates how one might use the RAS Toolkit
 * to log messages and trace the flow of a program.
 **/

// If your programs implements RASIMessageEvent and RASITraceEvent, you
// do not have to preface every use of the TYPE_XXX constants with the
// interface name.

public class RASSample extends Thread implements RASIMessageEvent, RASITraceEvent
{

// Name the file containing our message texts.

private static final String MSG_FILE = "com.ibm.ras.samples.RASMsgs";

// Define a few RAS objects that are needed in this sample application.
// They could certainly be initialized here, but we will do it in 
// run(), just to keep it all in one place.  These objects are
// global in scope because this is a small application.  In a larger
// program, you may want to create more with limited scope to selectively 
// control logging in different parts of the program.
//
// To start, we need at least one message logger and one trace logger.
// We'll create several different handlers to demonstrate how they
// can be connected.  See the initRAS method for more on this.

private RASConsoleHandler conLog;

private RASFileHandler msgLog;
private RASFileHandler trcLog;

private RASMessageLogger msgLogger;

private RASTraceLogger trcLogger;

/*----------------------------------------------------------------------
 * Name:  main
 *--------------------------------------------------------------------*/
/**
 * Main entry point for the RASSample application. 
 * This sample takes no arguments.
 */

public static void main(String args[])
{

// Create a RASSample object and start it.

RASSample sample = new RASSample();
sample.start();
}


/*----------------------------------------------------------------------
 * Name:  RASSample
 *--------------------------------------------------------------------*/
/**
 * Creates a RASSample.
 */

public RASSample()
{
}


/*----------------------------------------------------------------------
 * Name:  run
 *--------------------------------------------------------------------*/
/**
 * Demonstrates programming using the RAS Toolkit.
 */

public void run()
{

// Initialize and configure the RAS objects needed by this sample.

initRAS();
                                                                
// Illustrate message logging.

messageSample();

// Illustrate trace logging.

traceSample();

// Demonstrate how a RAS Manager is used.

managerSample();

// Before we terminate, stop the RASHandlers, which are running on 
// their own threads.

conLog.stop();
msgLog.stop();
trcLog.stop();
}


/*----------------------------------------------------------------------
 * Name:  initRAS
 *--------------------------------------------------------------------*/
/**
 * Initializes the RAS system.
 */

public void initRAS()
{

// To log messages, we must create at least one RASMessageLogger.  We
// recommend always giving loggers a name and a description.

msgLogger = new RASMessageLogger("MsgLogger", "Sample Message Logger");

// To trace program execution, we need at least one RASTraceLogger.  Like the
// RASMessageLogger, we will give this one a name and a description.

trcLogger = new RASTraceLogger("TrcLogger", "Sample Trace Logger");

// Without at least one RASHandler, the message and trace data is lost.  
// In this sample, we will create several different handlers:  one
// to send data to the console and two to log to separate files.  The
// console will receive message and trace data.  One file will receive
// messages and the other, trace data.  The point is that, if you want,
// you can send the same data to different devices.  Perhaps you want error
// messages displayed on a console, and all messages logged to a file.

conLog = new RASConsoleHandler("Console", "Console Handler");

msgLog = new RASFileHandler("MsgLog", "RAS Message Log", "RASMessage.log");

trcLog = new RASFileHandler("TrcLog", "RAS Trace Log");
trcLog.setFileName("RASTrace.log");

// The basic RASMessageLogger configuration is complete when you attach
// to it at least one RASHandler.  Ours get two RASHandlers.

msgLogger.addHandler(conLog);
msgLogger.addHandler(msgLog);

// The text for RAS messages is usually kept in a resource bundle and the
// message text is accessed by a "key," which is, essentially, a name for
// the text.  If you have only a few resource bundles containing your 
// message text, it is probably easier to pass their names to the message
// logger and use the message() methods that don't include a message  
// file name.  We'll do just that.

msgLogger.setMessageFile(MSG_FILE);

// There are a few fields in a message that tend not to vary from event
// to event.  If we want them included in the formatted message, we have
// to tell the RASMessageLogger.  Let's include them all.  In practice,
// an application might not have a server associated with it -- but an
// applet would.  We'll just make up a server name.  We'll get our 
// IP name and use that as our client name.

msgLogger.setOrganization("IBM");
msgLogger.setProduct("RAS Toolkit");
msgLogger.setComponent("Samples");
msgLogger.setServer("ras.raleigh.ibm.com");

try
  {
  msgLogger.setClient(InetAddress.getLocalHost().getHostName());
  }
catch (UnknownHostException uE)
  {
  msgLogger.setClient("localhost");
  }
  
// For the trace logger, we just add the desired handlers.
// Note that the message logger and the trace logger share the
// console handler, but their data is going to separate files.

trcLogger.addHandler(conLog);
trcLogger.addHandler(trcLog);

// The default message mask in the handlers logs all messages
// defined by the RASMessageEvent class.  Let's change this so that only
// warning and error messages go to the console.  We'll leave the
// message log alone, so that it receives all messages.
//
// Note that it is possible to set a message mask or a trace mask in
// the loggers, too.  In practice, you will probably pick one place
// or the other, not both.  If you set different masks in different
// loggers, you can control the kind of data logged in different
// parts of your program.  If you set the masks in the handlers,
// you control what goes to each output device.

conLog.setMessageMask(TYPE_WARN | TYPE_ERR);

// Message logging is enabled when the RASMessageLogger is created.
// Trace logging is off.  Let's turn it on.

trcLogger.isLogging = true;
}


/*----------------------------------------------------------------------
 * Name:  messageSample
 *--------------------------------------------------------------------*/
/**
 * Illustrates message logging.
 */

public void messageSample()
{
 
// Here are some samples of message logging.  In this example, a 
// static message (no run-time information is added) is logged.
// It is an informational message.  The class name will be derived
// from "this" and the method name is "messageSample."  (This message 
// will not be written to the console, since its message mask only 
// includes warning and error messages.

msgLogger.message(TYPE_INFO, this, "messageSample", "START_SERVER"); 

// Here is a message that shows how to insert run-time information
// into a message.  This is similar to the C-language printf function,
// which accepts an arbitrary number of values to be plugged into a
// message.  We'll insert the day and a temperature into this message.
// In RASSampleMsgs.java, you will see that "day" is represented by
// "{0}" and "temp" by "{1}".  Substitution of variables into the text
// depends on the order in which the variables are passed to the 
// message method.

SimpleDateFormat sf = new SimpleDateFormat("EEEEE");
Date date = new Date();
String day = sf.format(date);
Integer temp = new Integer(91);

msgLogger.message(TYPE_WARN, this, "messageSample", "HOT_DAY", day, temp);

// If you have more than two objects to insert in a message, you have to
// build an Object array.  We'll just use the same string and two objects.
// If you like arrays, you can always use this approach!

Object[] inserts = new Object[2];
inserts[0] = day;
inserts[1] = new Integer(101);          // It's really hot now!

msgLogger.message(TYPE_WARN, this, "messageSample", "HOT_DAY", inserts);

// If your message is being logged in a static method, there is
// no "this" object from which the class name can be derived.  You have
// to name the class.  Of course, you can use this approach even in a
// non-static method, if you are so inclined.

msgLogger.message(TYPE_ERR, "com.ibm.ras.samples.RASSample", "messageSample", "STATIC");

// These message logging examples work because the message file to use 
// was specified using the setMessageFile() method.  We can, instead, name
// the message file on every message() call.  We'll remove the message file
// to demonstrate this this approach works as well.  (Note that the msg()
// methods, not message(), are used when you want to specify the key and the
// message file.)
//
// You have to name the message file if you have more than one, though you 
// could call RASMessageLogger.setMessageFile for the most commonly used 
// message file and use the msg methods only when needed.

msgLogger.setMessageFile("");

msgLogger.msg(TYPE_ERR, "com.ibm.ras.samples.RASSample", "messageSample", 
              "STATIC", MSG_FILE);

// Message logging is intended to be done through message keys and 
// resource bundles so that the messages can be translated.  However,
// some support is available for logging non-translatable text messages.
// These methods accept additional Objects, which are displayed through
// their toString methods.  These objects are not inserted into
// the message text using the MessageFormat class, as is done with the
// message and msg methods.

msgLogger.textMessage(TYPE_WARN, this, "messageSample",
                      "This method has no additional parameters.");

msgLogger.textMessage(TYPE_WARN, this, "messageSample",
                      "This method adds two additional parameters",
                      new Integer(51), "Parm 2");

// Exceptions can also be traced.

msgLogger.exception(TYPE_ERR, this, "messageSample", 
                      new IllegalArgumentException("Exception demo."));
}


/*----------------------------------------------------------------------
 * Name:  traceSample
 *--------------------------------------------------------------------*/
/**
 * Illustrates trace logging.
 */

public void traceSample()
{

// Entry/exit tracing is useful to see the methods that are
// executing.  Knowing that something bad happened in a
// program is often not useful -- unless you know how you
// got there!
//
// Note that the entry method is wrapped in a test of the 
// public boolean "isLogging."  This quick test avoids the
// overhead of the method call if tracing is off.  We've 
// given this entry a type of "public" -- a general public
// method.  The entry method will add TYPE_ENTRY_EXIT to 
// this as a convenience to the programmer.  This example shows
// that the entry method can accept additional parameters to be
// traced.  These should be the parameters passed into the method
// whose entry you are tracing.

int i = 5;
String s = "This is a test";
if (trcLogger.isLogging)
  trcLogger.entry(TYPE_PUBLIC, this, "traceSample",
                  s, new Integer(i));

// We would normally only have only one entry call in a method.
// To illustrate a few points, this methods gets more than one!
// The test of the isLogging boolean is a quick-and-dirty check
// that is necessary, but not sufficient, to know if an entry
// will be logged.  Another check, involving the overhead of a 
// method call, checks the mask of each RASHandler against
// the type of the trace point.  If at least one RASHandler
// will process the trace point, "true" is returned.  Otherwise,
// the response is "false."  (This isLoggable method also checks
// the isLogging boolean.  If isLogging is false, it doesn't matter
// what the RASHandlers will do.
//
// This example also shows an entry trace with no parameters.  If 
// you have more than four parameters, you have to create an Object[],
// just as was shown for message tracing.

if (trcLogger.isLoggable(TYPE_PUBLIC | TYPE_ENTRY_EXIT))
  trcLogger.entry(TYPE_PUBLIC, this, "traceSample");

// For general tracing, use the trace method.  You can trace any String
// that you care to build.  This sample illustrates the use of trace types.

if (trcLogger.isLogging)
  {
  String msg = "1. Trace is valuable when Bad Things happen to Gooodd Code.";
  trcLogger.trace(TYPE_SVC, this, "traceSample", msg);
  }

// Exceptions can also be traced.

if (trcLogger.isLogging)
  {
  trcLogger.exception(TYPE_ERROR_EXC, this, "traceSample", 
                      new IllegalArgumentException());
  }

// Just before we leave a method, add an exit trace.  It comes in two
// flavors:  with and without a return code.  We'll show both, but, just
// like entry tracing, you only have one exit trace in a method.  The 
// first example also shows that, in static methods where the "this"
// object does not exist, you can use a string to name the class.

if (trcLogger.isLogging)
  trcLogger.exit(TYPE_PUBLIC, "com.ibm.ras.samples.RASSample", "traceSample");

if (trcLogger.isLogging)
  trcLogger.exit(TYPE_PUBLIC, this, "traceSample", new Boolean(true));
}


/*----------------------------------------------------------------------
 * Name:  managerSample
 *--------------------------------------------------------------------*/
/**
 * Illustrates use of the RASManager.
 */

public void managerSample()
{
RASPropertyDataStore ds = null;

// A RAS Manager knows about RAS objects which have been configured in
// some persistent storage.  We have to tell the RAS Manager which 
// storage we are using.  We'll create one based on the sample properties
// file used to illustrate the format used by the RASPropertyDataStore
// class.
//
// (Using the System property file.separator is important when running on
// multiple operating systems.)

String sep = File.separator;
String propFile = "/com" + sep + "ibm" + sep + "ras" + sep + "samples" + sep +
                  "RASDataStore.properties";
try
  {
  ds = new RASPropertyDataStore(propFile);
  }
catch (FileNotFoundException fnfE)
  {
  System.err.println("Yuck:  " + fnfE.getMessage());
  }
catch (IOException ioE)
  {
  System.err.println("Oooooo:  " + ioE.getMessage());
  }

// After creating the data store, a RAS Manager is next.  This method ensures
// that only one RAS Manager exists.

RASManager mgr = RASManager.getManager(ds);

// Once you have a RAS Manager, you can get loggers by name.  If the
// configuration has been done properly, they should be ready to go, 
// with handlers attached.

RASMessageLogger ml1 = (RASMessageLogger) mgr.getMessageLogger("RASMessageLogger1");
RASMessageLogger ml2 = (RASMessageLogger) mgr.getMessageLogger("RASMessageLogger2", "George");

ml1.message(TYPE_WARN, this, "messageSample", "START_SERVER");

ml1.textMessage(TYPE_WARN, this, "managerSample",
               "This is a RAS Manager demonstration",
               new Integer(88), "Parm X");

ml2.textMessage(TYPE_WARN, this, "managerSample",
               "This is another RAS Manager demonstration",
               new Integer(99), "Parm Y");

RASTraceLogger tl1 = (RASTraceLogger) mgr.getTraceLogger("RASTraceLogger");
tl1.isLogging = true;

if (tl1.isLogging)
  tl1.exit(TYPE_PUBLIC, this, "traceSample", new Integer(999));

// When you are done with the loggers, return them to the RAS Manager.

mgr.returnMessageLogger(ml1);
mgr.returnMessageLogger(ml2, "George");
mgr.returnTraceLogger(tl1);
}
 
}
