/*----------------------------------------------------------------------
 * IBM Confidential
 * OCO Source Materials
 *
 * (C) Copyright IBM Corporation 1998.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *--------------------------------------------------------------------*/

package com.ibm.ras.server;

import java.net.*;
import java.io.*;
import java.text.DateFormat;
import java.util.Date;

import com.ibm.ras.*;

/**
 * <code>RASLogServer</code> works in tandem with the 
 * <code>RASSocketHandler</code> class.  It is run as a
 * Java application on a server.  It binds to a socket, listening and
 * accepting connections from instances of <code>RASSocketHandler</code>.
 * Once a connection is established between the <code>RASLogServer</code>
 * and a client, log data is sent from the client to the
 * <code>RASLogServer</code>, where it is displayed on the console and,
 * optionally, written to a file.
 * <p>
 * Usage:
 * <pre>java com.ibm.ras.server.RASLogServer [port] [filename]</pre>
 * The default port for the <code>RASLogServer</code> process is 9991.
 * If <code>filename</code> is specified, the output is also written to
 * the file.
 **/

public class RASLogServer implements Runnable
{
private static final String s = RASCopyright.COPYRIGHT_98;

/**
 * The default TCP port on which this <code>RASLogServer</code> connects.
 */

private static final int DEFAULT_PORT = 9991;

/**
 * A flag set <code>true</code> if this server will write to a file.
 */

private boolean toFile = false;

/**
 * The TCP port on which this <code>RASLogServer</code> connects.
 */

private int port = DEFAULT_PORT;

/**
 * The <code>PrintWriter</code> that sends a formatted log
 * entry to its destination.
 */

private PrintWriter pWriter = null;

/**
 * The catalog containing the messages displayed by the this class.
 */

private RASMessageCatalog cat = 
  new RASMessageCatalog("com.ibm.ras.server.RASLogServerMsgs");

/**
 * The file which will hold the log entries.
 */

private String filename = null;


/*----------------------------------------------------------------------
 * Name:  main
 *--------------------------------------------------------------------*/
/**
 * Main entry point for the <code>RASLogServer</code> application. 
 *
 * @param port     The port on which the <code>RASLogServer</code> listens.
 * @param filename The name of the file to which the log data will be
 *                 appended.  If this is <code>null</code>, the data is  
 *                 only written to the console.
 */

public static void main(String args[])
{
int port = 0;
String filename = null;

switch ( args.length )
  {
  case 0:
    port = DEFAULT_PORT;
    break;

  case 1:
    port = Integer.valueOf(args[0]).intValue();
    break;

  case 2:
    port = Integer.valueOf(args[0]).intValue();
    filename = args[1];
    break;

  default:
    RASMessageCatalog cat = new RASMessageCatalog("com.ibm.ras.server.RASLogServerMsgs");
    System.err.println(cat.getMessage("USAGE"));
    System.exit(1);
  }

RASLogServer d = new RASLogServer(port, filename);
}


/*----------------------------------------------------------------------
 * Name:  RASLogServer
 *--------------------------------------------------------------------*/
/**
 * Creates a <code>RASLogServer</code>.
 *
 * @param port     The port on which the <code>RASLogServer</code> listens.
 * @param filename The name of the file to which the log data will be
 *                 appended.  If this is <code>null</code>, the data is  
 *                 only written to the console.
 */

public RASLogServer(int port, String filename)
{
this.port = port;
this.filename = filename;

Thread thread = new Thread(this);
thread.start();
}


/*----------------------------------------------------------------------
 * Name:  run
 *--------------------------------------------------------------------*/
/**
 * Starts the <code>RASLogServer</code> on its own thread.
 **/

public void run()
{

if (filename != null)
  {
  toFile = true;
  try
    {
    pWriter = new PrintWriter(new FileOutputStream(filename), true);
    }
  catch (IOException ioE)
    {
    System.err.println(cat.getMessage("ERR_OPEN_FILE", filename));
    System.err.println(ioE.getMessage());
    ioE.printStackTrace();
    }
  }
else
  {
  toFile = false;
  }

try
  {

  // Create the well known server port for client connections

  write(cat.getMessage("START_SERVER"));
  write(cat.getMessage("CREATE_SOCKET", new Integer(port).toString()));

  if (toFile)
    write(cat.getMessage("OUTPUT_TO_FILE", filename));
  else
    write(cat.getMessage("OUTPUT_TO_CONSOLE"));

  ServerSocket serverSocket = new ServerSocket(port);

  // Listen on the well known port, blocking on accept.  When a
  // client connection occurs, a new socket for that client is
  // returned.  Start a new thread and pass that socket to the
  // new thread, which will manage the connection.

  while (true)
    {
    Socket socket = serverSocket.accept();

    RASConnectionHandler connectionHandler =
      new RASConnectionHandler(socket, toFile, pWriter);
    Thread thread = new Thread(connectionHandler);
    thread.start();
    }
  }
catch (Throwable e)
  {
  e.printStackTrace();
  }
}


/*----------------------------------------------------------------------
 * Name:  write
 *--------------------------------------------------------------------*/
/**
 * Writes log data to the screen and, optionally, to a file.
 *
 * @param msg The message to write.
 **/

private void write(String msg)
{
if (toFile)
  pWriter.println(msg);

System.out.println(msg);
}

}
